if (!Object.keys) {
	Object.keys = (function() {
		var hasOwnProperty = Object.prototype.hasOwnProperty,
			hasDontEnumBug = !({
				toString: null
			}).propertyIsEnumerable('toString'),
			dontEnums = ['toString', 'toLocaleString', 'valueOf', 'hasOwnProperty', 'isPrototypeOf', 'propertyIsEnumerable', 'constructor'];

		return function(obj) {
			if (typeof obj !== 'object' && typeof obj !== 'function' || obj === null) throw new TypeError('Object.keys called on non-object');

			var result = [];

			for (var prop in obj) {
				if (hasOwnProperty.call(obj, prop)) result.push(prop);
			}

			if (hasDontEnumBug) {
				for (var i = 0; i < dontEnums.length; i++) {
					if (hasOwnProperty.call(obj, dontEnums[i])) result.push(dontEnums[i]);
				}
			}

			return result;
		};
	})();
}

// Taken from underscore.js
if (!Array.range) {
	Array.range = function(start, stop, step) {
		if (arguments.length <= 1) {
			stop = start || 0;
			start = 0;
		}
		step = arguments[2] || 1;
		
		var range = new Array(Math.max(Math.ceil(Math.abs(stop - start) / step), 0));
		
		for(var i = 0; i < range.length; i++) {
			range[i] = start;
			start += step;
		}
		
		return range;
	};
}

// fix for IE missing window.location.origin
if (!window.location.origin) {
  window.location.origin = window.location.protocol + "//" + window.location.hostname + (window.location.port ? ':' + window.location.port: '');
}

//	this file is the entry point for the application. 
//	The Gruntfile loads start.js, then all js files in src/javascript/, then end.js

// 'app' is the top level - all views, models, routers, etc are contained in 'app'.
var app = app || { view: {}, collection: {}, model: {}, router: {}, event: {}, config: {}, graphs: {}, utils: {}};

// load the config.js file
/**
 * Intercept clicks of Download buttons so we can see if they have agreed to the User Agreement
 * @param event
 */
app.utils.interceptDownload = function(event) {
    console.log('download intercepted');
    console.log(event);
    // if the project has not added a User Agreement, then we won't prompt them, so this function exits here
    if (!app.config.userAgreement) {
        return this;
    }

    // a User Agreement exists, so prevent the file from automatically downloading until we have them ack the form
    event.preventDefault();

    // if they haven't accepted the User Agreement, there won't be a cookie
    if (!Cookies.get(app.config.title + '_agreement')) {
        // the template ID is the script ID set in main.scala.html for the user_agreement.scala.html form
        var template = $("#user-agreement-template").html();

        // the user-agreement ID is set in the bottom of index.scala.html. It is the hidden modal that we will
        // activate when we want to prompt for the agreement
        // we will pass in the original URL that the user wanted to download
        $("#user-agreement").html(_.template(template, {url: event.currentTarget.href}));

        // the user-agreement-modal ID is found in user_agreement.scala.html.
        // We must call .modal to register the modal, and we are calling show at the same time
        $("#user-agreement-modal").modal('show');

    } else {
        // if the user has already agreed to the user agreement, then we will trigger a click
        // of the link to start the download. We don't use the original element, since that
        // has the class of 'download' on it and it would recursively call interceptDownload since
        // the event bus is setup to listen for clicks of 'a.download' links.
        // We set the target attribute to open in a new window, and the download attribute
        // so it will suppress the new tab and just directly download the file.
        // (the download 'attribute' is not the same as the download 'class')
        var passThroughLink = document.createElement('a');
        passThroughLink.href = event.currentTarget.href;
        passThroughLink.target = "_blank";
        passThroughLink.download = true;
        passThroughLink.click();
    }
};

app.utils.setCookie = function() {
    // this gets called by an 'onclick' attribute in user_agreement.scala.html
    Cookies.set(app.config.title + '_agreement', Date.now());
    app.utils.closeModals();
};

app.utils.closeModals = function() {
    // this is called by app.utils.setCookie
    $(".modal").modal('hide');
};
// customize this file to suit your branch
app.config = {
    features: {
      boxWhiskerRaw: true
    },
    clowderDevUrl: "http://seagrant-dev.ncsa.illinois.edu/clowder",
    alert: false,
    title: "Geodashboard",
    googleAnalytics: "",
    metadata: {
        title: "Geodashboard",
        keywords: "geodashboard, illinois, ncsa, university of illinois, uiuc",
        description: "Geodashboard Web App",
        url: "http://opensource.ncsa.illinois.edu/projects/GEOD"
    },
    binning: {
        time: 'smart',
        depth: 1
    },
    areas: {
        title: "Lakes",
        noneFound: "No lakes have been specified.",
        items: [
            // override this array in ./config/areas.js
        ]
    },
    layers: {
        title: "Layers",
        noneFound: "No layers have been specified.",
        items: [
            // override this array in ./config/layers.js
        ]
    },
    basins: {
        title: "Basins",
        noneFound: "No basins have been specified.",
        items: [
            // override this array in ./config/basins.js
        ]
    },
    parameters: {
        title: "Parameters",
        noneFound: "No parameters have been specified.",
        defaults: {
            detailView: 'time',
            searchView: 'visible',
            exploreView: 'visible'
        },
        items: [
            // override this array in ./config/parameters.js
        ]
    },
    sources: {
        title: "Data Sources",
        noneFound: "No data sources have been specified.",
        items: [
            // override this array in ./config/sources.js
        ]
    },
    sensors: {
        title: "Sensors",
        noneFound: "No sensors have been specified.",
        items: [
            // override this array in ./config/sensors.js
        ]
    },
    views: {
        home: {
            title: "Home",
            link: "#explore",
            attributes: {
                id: 'home-view'
            },
            templateSelector: "#explore-template",
            explore: {
                attributes: {
                    class: "accordion"
                },
                map: {
                    //maxBounds: [[50, -70], [40, -100]],
                    //minZoom: 6,
                    //maxZoom: 12,
                    scrollWheelZoom: false,
                    id: "#explore-map-view"
                },
                accordion: {
                    containerID: "#explore-accordion-container",
                    // set the defaults for an accordion item
                    // can be overridden in an item object
                    template: '#explore-accordion-item-template',
                    class: 'accordion-group',
                    items: [
                        {
                            attributes: {
                                id: "sources"
                            },
                            title: "Explore Sources",
                            defaultOpen: true,
                            contents: {
                                viewType: "ExploreSourcesView",
                                containerID: "explore-sources-accordion-contents",
                                viewConfig: {
                                    templateID: '#explore-sources-template',
                                    map: {
                                        id: "#explore-map-view",
                                        scrollWheelZoom: false
                                    },
                                    modalsID: "#source-modals-template",
                                    pills: {
                                        className: ".explore-pill",
                                        tooltipOptions: {
                                            animation: true,
                                            placement: "right",
                                            container: "body"
                                        }
                                    },
                                    collapse: {
                                        className: '.collapse',
                                        inputSelector: 'input[name="source"]'
                                    }
                                }
                            }
                        },
                        {
                            attributes: {
                                id: "layers",
                                class: 'accordion-group'
                            },
                            title: "Explore Layers",
                            defaultOpen: false,
                            contents: {
                                viewType: "LayersCollectionView",
                                containerID: "explore-layers-accordion-contents",
                                viewConfig: {
                                    templateID: '#explore-layers-template'
                                }
                            }
                        }
                    ]
                },
                "moreinfo": {
                    "text": "More information about EPA/GLMPO",
                    "link": "http://www.epa.gov/greatlakes/"
                }
            }
        },
        detail: {
            attributes: {
                id: "detail-view",
                class: 'row-fluid'
            },
            templateSelector: "#detail-template",
            miniMap: [
                {
                    id: "#mini-map",
                    config: {
                        tagName: 'div',
                        mapOptions: {
                            zoomControl: false,
                            attributionControl: false,
                            dragging: false,
                            touchZoom: false,
                            scrollWheelZoom: false,
                            doubleClickZoom: false,
                            boxZoom: false
                        },
                        geoJsonFeature: []
                    }
                }
            ],
            controlsViews: [
                {
                    name: "controlsView",
                    config: {
                        name: "time",
                        el: "#modal-content",
                        parametersID: "parameters-time",
                        seasonFormID: "season-time",
                        seasonSelectID: "season-select-time",
                        dateRangeID: "date-range-time",
                        sliderID: "slider-time",
                        attributes: {
                            tagName: 'div',
                            id: 'controls'
                        }
                    }

                },
                {
                    name: "controlsDepthView",
                    config: {
                        name: "depth",
                        el: "#modal-content",
                        yearFormID: "year-depth",
                        yearSelectID: "year-depth-select",
                        seasonFormID: "season-depth",
                        seasonSelectID: "season-depth-select",
                        parametersID: "parameters-depth",
                        attributes: {
                            tagName: 'div',
                            id: 'controls'
                        }
                    }

                }
            ],
            timeSeriesViews: [],
            limnologyViews: []
        },
        search: {
            title: "Search",
            link: "#search",
            attributes: {
                id: "search-body",
                class: 'row-fluid'
            },
            templateSelector: "#search-template",
            map: {
                id: '#search-map',
                //maxBounds: [[24, -125], [49, -66]],
                minZoom: 12,
                maxZoom: 16,
                scrollWheelZoom: false,
                markerFillColor: "#ff9800",
                markerColor: "#000000",
                radius: 3,
                weight: 1,
                opacity: 1,
                fillOpacity: 1
            },
            shapeOptions: {
                stroke: true,
                color: '#7eb4fc',
                weight: 4,
                opacity: 0.5,
                fill: true,
                fillColor: null, //same as color by default
                fillOpacity: 0.2,
                clickable: true
            },
            drawOptions: {
                position: 'topright',
                draw: {
                    polyline: false,
                    marker: false
                },
                edit: {
                    remove: true
                }
            },
            resetMapButtonOptions: {
                text: 'Reset Search',  // string
                iconUrl: '',  // string
                glyph: '', // bootstrap glyph icons
                hideText: true,  // bool
                maxWidth: 30,  // number
                doToggle: false,  // bool
                toggleStatus: false  // bool
            },
            layers: {
                layerWeight: 1,
                layerOpacity: 1,
                bindPopupClass: "search-popup",
                items: [
                    {
                        name: "basins",
                        featureProperties: "LAKEBASIN"
                    }
                ]
            },
            dateRangeSelector: {
                dateRangeSelector: {
                    id: '#searchtime',
                    format: 'YYYY-MM-DD HH:mm:ss'
                },
                downloadButtons: {
                    permalink: {
                        id: "#permalink",
                        downloadID: "#download-permalink",
                        template: "#permalink-template",
                        modalID: "#permalink-modal"
                    },
                    json: {
                        id: '#download-json'
                    },
                    csv: {
                        id: "#download-csv"
                    }


                }
            },
            about: {
                title: "About",
                link: "#about",
                templateSelector: "#about-template"
            },
            about: {
                title: "Welcome",
                link: "#welcome",
                templateSelector: "#welcome-template"
            },
            footer: {
                templateID: "#footer-default-template"
            }
        }
    },
    graphs: {
        timeSeries: {
            showUnits: {
                sideBar: false,
                graphTitle: true,
                graphXAxis: true,
                graphYAxis: true,
                graphTooltip: true,
                showZero: true
            },
            showSource: true
        },
        depthProfile: {
            showUnits: {
                sideBar: false,
                graphXAxis: true,
                graphYAxis: true

            }
        }
    }
}

// load the env file from /src/javascript/envs/{env}.js

app.config.areas.items = [];
app.config.parameters.items = [
    {
        "id": "total-phosphorus",
        "title": "Total Phosphorus",
        "categories": [ "nutrients" ],
        "description": "",
        "alternates": [ "TP, ug/l", "TP ug/l" ],
        "detailView" : "time", // depth, or ignore
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "chlorophyll-a",
        "title": "Chlorophyll <i>a</i>",
        "categories": [ "nutrients", "sensor-data" ],
        "description": "",
        "alternates": [ "Chlorophyll-a, ug/l", "Chlorophyll-a ug/l" ],
        "detailView" : "time",
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "temperature",
        "title": "Temperature",
        "categories": [ "ctd", "triaxus" ],
        "description": "",
        "alternates": [ "Temperature [ITS-90, deg C]", "temperature, IPTS-68 [deg C]", "temperature, ITS-90 [deg C]", "Temperature, C" ],
        "detailView" : "time",
        "searchView": "visible", // only checking if it is != ignore as of 2014-06
        "exploreView" : "visible" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "depth-code",
        "title": "Depth Code",
        "categories": [ ],
        "description": "Depth Code",
        "alternates": [ "DEPTH_CODE" ],
        "detailView" : "ignore",
        "searchView": "ignore", // only checking if it is != ignore as of 2014-06
        "exploreView" : "ignore" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "par",
        "title": "PAR",
        "categories": [ ],
        "description": "PAR",
        "alternates": [ "PAR" ],
        "detailView" : "depth",
        "searchView": "depth", // only checking if it is != ignore as of 2014-06
        "exploreView" : "depth" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "sample-id",
        "title": "Sample ID",
        "categories": [ "" ],
        "description": "Sample ID",
        "alternates": [ "SAMPLE_ID" ],
        "detailView" : "ignore",
        "searchView": "ignore", // only checking if it is != ignore as of 2014-06
        "exploreView" : "ignore" // only checking if it is != ignore as of 2014-06
    },
    {
        "id": "fluorescence",
        "title": "Fluorescence",
        "categories": [ "" ],
        "description": "Fluorescence",
        "alternates": [ "Fluorescence" ],
        "detailView" : "depth",
        "searchView": "depth", // only checking if it is != ignore as of 2014-06
        "exploreView" : "depth" // only checking if it is != ignore as of 2014-06
    }
];
app.config.sensors.items = [];
app.config.sources.items = [
    {
        "id": "epa",
        "title": "Water Quality",
        "prettyName": "Environmental Protection Agency",
        "acronym": "EPA / GLNPO",
        "description": "U.S. Environmental Protection Agency's (USEPA) Great Lakes National Program Office (GLNPO) has a water quality monitoring program in the offshore waters (water greater than 30 meters in depth or greater than 3 miles from shore) of the Laurentian Great Lakes. Samples are taken from set stations in each lake twice a year, once in the spring and once in the summer. Samples have been collected from some stations since the 1980s.",
        "parameters": {
            "description" : "The following parameters from GLNPO's long-term monitoring stations are available on this site:",
            "ids": [
                "total-phosphorus",
                "chlorophyll-a"
            ]
        },
        "color": "#0D71BA",
        "moreinfo" : {
            "text" : "",
            "link" : ""
        }
    },
    {
        "id": "glfmsp",
        "title": "Contaminants",
        "prettyName": "Great Lakes Fish Monitoring and Surveillance Program",
        "description": "Great Lakes Fish Monitoring and Surveillance Program (GLFMSP) has been monitoring contaminants in open waters since the 1970s. It is a partnership among the Great Lakes states, tribes and federal agencies, and has a complimentary program in Environment Canada. A particular sub-program--Open Lakes Trend Monitoring Program--looks at contaminant trends using fish (walleye in Lake Erie and lake trout in the other lakes) as biomonitors for toxic chemicals in the water. Fish samples are collected from each of the Great Lakes in the fall, alternating between nearshore sites in even years and offshore sites in odd years. To ensure that data are comparable across the time series, fish are collected in the same general nearshore and offshore areas and fish of similar size are collected to reduce the impact of size variation on contaminant trend data. Annual averages from whole fish samples are available on this site for the following contaminants: mercury, PCBs, PBDEs, DDT, toxaphene, chlordanes, dieldrin, and mirex.",
        "color": "#CC2128",
        "moreinfo" : {
            "text" : "More Information about GLFMSP",
            "link" : "http://www.epa.gov/glnpo/monitoring/fish/index.html"
        }
    },
    {
        "id": "iadn",
        "title": "Air Pollutants",
        "prettyName": "Integrated Atmospheric Deposition Network",
        "description": "Integrated Atmospheric Deposition Network (IADN) has been monitoring atmospheric deposition of pollutants (in air and precipitation) to the Great Lakes basin since the early 1990s. The program is run by USEPA, Environment Canada's (EC) Health Division of Ontario Region and EC's Meteorological Service of Canada. This website contains data from seven stations-- five Master Stations, one on each of the Great Lakes, and two Satellite Stations in the urban centers of Chicago, IL and Cleveland, OH. Annual averages from each station are provided for PCBs, endosulfan, chlordanes, DDTs, and dieldrin as measured in the gaseous phase as well as DDTs and dieldrin from precipitation.",
        "color": "#F7941E",
        "moreinfo" : {
            "text" : "More Information about IADN",
            "link" : "http://www.epa.gov/glnpo/monitoring/air2/index.html"
        }
    },
    {
        "id": "lec",
        "title": "Lake Erie Lower Trophic Data",
        "prettyName": "Lake Erie Committee Forage Task Group",
        "description": "Lake Erie Forage Task Group is part of the Lake Erie Committee (LEC), which is a member of the Great Lakes Fisheries Commission. LEC is a bi-national group with members from Ontario, Michigan, Ohio, Pennsylvania and New York who administer the fisheries of Lake Erie. In 1999, the Forage Task Group of the LEC initiated a lower trophic level assessment program within Lake Erie and Lake St. Clair. The program monitors total phosphorus and chlorophyll a, along with several other variables that characterize ecosystem health across Lake Erie. A total of 21 nearshore stations are sampled every two weeks from May through September each year.",
        "parameters": {
            "description": "The following parameters are available on this site from the LEC lower trophic level assessment program:",
            "ids": [
                "total-phosphorus",
                "chlorophyll-a"
            ]
        },
        "color": "#4D6363",
        "moreinfo" : {
            "text" : "More Information about LEC",
            "link" : "http://www.glfc.org/lakecom/lec/FTG.htm"
        }
    },
    {
        "id": "usgs",
        "title": "Water Discharge",
        "prettyName": "United States Geological Survey",
        "description": "U.S. Geological Survey conducts long-term monitoring of lakes and rivers nationwide. They have gauges throughout many rivers in the Great Lakes basin. We've chosen five principal rivers to focus on for this website, one on each lake: St. Louis River (Superior), St. Joseph River (Michigan), Saginaw River (Huron), Maumee River (Erie) and Oswego River (Ontario). In each river, we present data from gauges with the most complete long term data set. The data provided from each gauge is currently for discharge only. Measurements are taken daily. *Note that USGS posts both approved data (quality-assured) as well as more recent provisional data (accuracy unverified and subject to revision). Most current data are provisional. The data are released on the condition that neither the USGS nor the United States Government may be held liable for any damages resulting from its authorized or unauthorized use.",
        "color": "#39B54A",
        "moreinfo" : {
            "text" : "",
            "link" : ""
        }
    }

];
// your public api url. Prod and Dev use this read-only, so it is staticly set here.
app.config.clowderUrl = window.location.origin + "/clowder";

// loads jQuery and Backbone
(function($, Backbone, window, undefined) {
    "use strict";

app.graphs.boxwhisker = function() {
	var width = 185, height = 500, vMargin = 40, hMargin = 50;

	function boxwhisker(selection) {
		selection.each(function(d) {
			d = d.values[0];
      if (app.config.features.boxWhiskerRaw) {
        d = _.flatten(d.values.map(function(d) { return d.raw; })).sort(d3.ascending);
      } else {
        d = _.flatten(d.values.map(function(d) { return d.y; })).sort(d3.ascending);
      }

			var x = d3.scale.quantize().domain([0, 100]).range([0, hMargin, width / 2, width - hMargin, width]),
				y = d3.scale.linear().domain([Math.min(0, d[0]), d[d.length - 1]]).range([height - vMargin, vMargin]);

			var median = d3.quantile(d, 0.5),
				quartileRange = [d3.quantile(d, 0.25), d3.quantile(d, 0.75)],
				IQR = (quartileRange[1] - quartileRange[0]),
				extremefn = function(d) { return d < quartileRange[0] - 3 * IQR || d > quartileRange[1] + 3 * IQR; },
				outlierfn = function(d) { return !extremefn(d) && (d < quartileRange[0] - 1.5 * IQR || d > quartileRange[1] + 1.5 * IQR); },
				extremes = d.filter(function(d) { return extremefn(d); }),
				outliers = d.filter(function(d) { return outlierfn(d); }),
				mean = d3.mean(d),
				extent = d3.extent(d.filter(function(d) { return !(outlierfn(d) || extremefn(d)); }));

			var svg = d3.select(this).selectAll(".graphs-bw").data([0]);
			svg.enter().append("svg").attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1")
				.attr("preserveAspectRatio", "none").attr("class", "graphs-bw");
			svg.attr("viewBox", "0 0 " + width + " " + height);
			
			// Create lines which represent the upper and lower quartiles.
			var line = svg.selectAll(".graphs-quartile-range").data([x(25), x(75)]);
			line.enter().append("line").attr("class", "graphs-quartile-range");
			line.transition().attr("x1", function(d) { return d; }).attr("x2", function(d) { return d; })
				.attr("y1", y(quartileRange[0])).attr("y2", y(quartileRange[1]));
				
			// Create the circle that represents the mean.	
			var circle = svg.selectAll(".graphs-mean").data([mean]);
			circle.enter().append("circle").attr("class", "graphs-mean");
			circle.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/16)).attr("cx", x(50));
			
			
			// Create the lines that connect the quartiles to their extrema	
			var connection = svg.selectAll(".graphs-connection").data(extent);
			connection.enter().append("line").attr("class", "graphs-connection");
			connection.transition().attr("x1", x(50)).attr("y1", function(d) { return y(d); })
				.attr("x2", x(50)).attr("y2", function(d, i) { return y(quartileRange[i]); });
			
			// Create the circles for the outliers	
			var outlier = svg.selectAll(".graphs-outlier").data(outliers);
			outlier.enter().append("circle").attr("class", "graphs-outlier")
				.attr("cy", function(d) { return d < extent[0] ? height + 20 : -10; });
			outlier.exit().transition().style("opactiy", "0")
				.attr("cy", function(d) { return d3.select(this).attr("cy") >= height / 2 ? height + 20 : -10; }).remove();
			outlier.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/32)).attr("cx", x(50));

			// create the circles for the extreme outliers
			var extreme = svg.selectAll(".graphs-extreme").data(extremes);
			extreme.enter().append("circle").attr("class", "graphs-extreme")
				.attr("cy", function(d) { return d < extent[0] ? height + 20 : -10; });
			extreme.exit().transition().style("opactiy", "0")
				.attr("cy", function(d) { return d3.select(this).attr("cy") >= height / 2 ? height + 20 : -10; }).remove();
			extreme.transition().attr("cy", function(d) { return y(d); })
				.attr("r", ((x(75) - x(25))/32)).attr("cx", x(50));

			
			// Create the labels for all the interesting features, min, max, quartiles, median
			extent.splice(1, 0, quartileRange[0], median, quartileRange[1]);
			var feature = svg.selectAll(".graphs-feature").data(extent),
				featureEnter = feature.enter().append("g").attr("class", "graphs-feature");

			featureEnter.append("line");
			featureEnter.append("text");

			feature.select("line").transition().attr("x1", x(25)).attr("x2", x(75))
				.attr("y1", function(d) { return y(d); })
				.attr("y2", function(d) { return y(d); });

			feature.select("text").transition().attr("dy", ".32em")
				.attr("x", function(d, i)  { return i % 2 ? x(25) -5 : x(75) + 5; })
				.attr("y", function(d)  { return y(d); }).text(function(d) { return Math.round(d*100)/100; }) // use d3.format
				.style("text-anchor", function(d, i) { return i % 2 ? "end" : "start"; });
		});
	}
	
	boxwhisker.width = function(value) {
		if(!arguments.length) return width;
		width = value;
		return boxwhisker;
	};
	
	boxwhisker.height = function(value) {
		if(!arguments.length) return height;
		height = value;
		return boxwhisker;
	};
	
	boxwhisker.margin = function(vertical, horizontal) {
		if(!arguments.length) return [vMargin, hMargin];
		
		vMargin = vertical;
		
		if(horizontal === undefined) 
			hMargin = vertical;
		else hMargin = horizontal;
		
		return boxwhisker;
	};
	
	return boxwhisker;
};

app.graphs.depth = function() {
    var width = 510, height = 500, margin = 40,
        min = 0, max = 10, interpolate = "linear",
        dispatch = null;
    
    function depth(selection) {
		selection.each(function(d) {
			d = mapper(d);
			
			var ys = yScale(d.values);
			
			var y = d3.scale.linear().range([margin, height]).domain(ys);
            	
            var range = Array.range(+ys[0], +ys[1]);
            
            var graph = d3.select(this).selectAll("svg").data([0]);
            graph.enter()
                .append("svg")
                .attr("class", "graphs-depth-chart");
            graph.attr("width", width).attr("height", height + d.values.length * margin);
            
            // Y-Axis Label
            if(app.config.graphs.depthProfile.showUnits.graphYAxis)
            {
                var yAxisLabel = graph.selectAll(".graphs-yAxisLabel").data([0]);
                
                yAxisLabel.enter()
                    .append("text")
                    .attr("class", "graphs-yAxisLabel");
                
                yAxisLabel.transition()
                    .attr("transform", "rotate(-90)")
                    .style("text-anchor", "middle")
                    .attr("x", -215) // Moving up/down. + (up), - (down)
                    .attr("y", 10) // Moving left/right. + (right), - (left)
                    .text("Meters");
                
                yAxisLabel.exit().remove();
            }
                
            /* Y Axis */
            var yAxis = graph.selectAll(".graphs-y-axis").data([d.values]);
            yAxis.enter().append("g").attr("class", "graphs-y-axis")
                .attr("transform", "translate(" + margin + ", 0)");
            yAxis.exit().remove();
            yAxis.transition().call(d3.svg.axis().scale(y).orient("left"));
            
            if (false) {
              var depthBins = graph.selectAll(".graphs-depth-bin").data(range);
              depthBins.enter().append("rect").attr("class", "graphs-depth-bin").style("color", "white")
                  .style("opacity", "0").style("cursor", "pointer")
                  .on("mouseover.graphs", function(d, i) {
                      d3.select(this).transition().style("color", "grey").style("opacity", "0.6");
                  }).on("mouseout.graphs", function(d, i) {
                      d3.select(this).transition().style("color", "white").style("opacity", "0");
                  }).on("click.graphs", function(d, i) {
                      d3.selectAll(".graphs-selected-bin").classed("graphs-selected-bin", false);
                      d3.select(this).classed("graphs-selected-bin", true);
                      dispatch.changeDepth.call(this, i);
                  });
              depthBins.exit().remove();
              depthBins.transition().attr("x", margin).attr("y", function(d) { return y(d); }).attr("width", width - 2 * margin).attr("height", (height) / range.length);
            }            
            
            var chart = graph.selectAll(".graphs-chart").data(d.values, function(d) { return d.key; });
            chart.enter().append("g").attr("class", function(d) { return "graphs-chart " + d.key; })
                .style("color", function(f) { return f.color; })
                .style("stroke", function(f) { return f.color; })
                .style("fill", function(f) { return f.color; });
            chart.exit().transition().style("opacity", 0).remove();
            
            /* X Axis */
            var xAxis = chart.selectAll(".graphs-x-axis").data(function(d) { return [d]; }, function(d) { return d.key; });
            xAxis.enter().append("g").attr("class", "graphs-x-axis").attr("transform", function(d) { return "translate(0, " + (height + 5) + ")"; });
            xAxis.transition().call(function(selection) { selection.each(function(d) { d3.select(this).call(d3.svg.axis().scale(d.xScale).orient("bottom")); }); })
				.attr("transform", function(d, i, j) { return "translate(0, " + (height + margin * (j)) + ")"; });
            
            // Parameter + Unit
            if(app.config.graphs.depthProfile.showUnits.graphXAxis)
            {
                xAxis.append("text")
                    .attr("x", 40)
                    .attr("y", 27)
                    .text(function(d) {
                        // show the parameter's unit if it exists, otherwise show just the parameter
                        return (d.unit) ? (d.key).toUpperCase() + " ( " + d.unit + " )" : (d.key).toUpperCase()
                    });
            }
            
            var line = chart.selectAll(".graphs-line").data(function(d) { return [d]; }, function(d) { return d.key; });
            line.enter().append("path").attr("class", "graphs-line");
            line.transition().attr("d", function(d) { return d3line(d.xScale, y)(d.values); });
			
			var area = chart.selectAll(".graphs-area").data(function(d) { return [d]; }, function(d) { return d.key; });
			area.enter().append("path").attr("class", "graphs-area");
			area.transition().attr("d", function(d) { return d3area(d.xScale, y)(d.values); });
		});
    }
    
    function d3line(x, y) {
        return d3.svg.line()
			.x(function(d) { return x(d.x); })
			.y(function(d) { return y(d.y); })
			.interpolate(interpolate);
    }

    function d3area(x, y) {
        return d3.svg.area()
            .x0(function(d) { return x(d.x0); })
            .x1(function(d) { return x(d.x1); })
            .y(function(d) { return y(d.y); })
            .interpolate(interpolate);
    }

    function yScale(values) {
    	var yValues = values.reduce(function(a, b) { // Flatten the outer most array
    		return a.concat(b.values.reduce(function(a, b) { // Flatten the value objects
                return a.concat([b.y]); 
    		}, [])); 
    	}, []); // We should now have an array of all the y values in the heirarchy
        
        return d3.extent(yValues);
        // return d3.scale.linear().range([margin, height - values.length * margin])
        //     .domain(d3.extent(yValues));
    }

    function xScale(values) {
        var xValues = values.reduce(function(a, b) { return a.concat([b.x]); }, []);
        return d3.scale.linear().range([margin, width - margin])
            .domain([d3.min(xValues), d3.max(xValues)]);
    }
    
    // Precomputation on data, should happen before (or during) call to mapper
    function depthBin(values) {
        var depthSums = {}, minBoundSum = {}, maxBoundSum = {};
        var depthCounts = {};
        values.forEach(function(d) {
            var val = Math.floor(d.y);
            if(!depthCounts[val]) {
                depthCounts[val] = depthSums[val] = minBoundSum[val] = maxBoundSum[val] = 0;
            }
            
            depthCounts[val]++;
            depthSums[val] += d.x;
            minBoundSum[val] += d.x0;
            maxBoundSum[val] += d.x1;
        });
        
        return Object.keys(depthCounts).map(function(d) {
			return {
				x : depthSums[d] / depthCounts[d],
				y : +d,
				x0 : minBoundSum[d] / depthCounts[d],
				x1 : maxBoundSum[d] / depthCounts[d]
			};
        });
    }
        
    function mapper(data) {
        data.values.map(function(d) {
            d.values = depthBin(d.values);
            d.xScale = xScale(d.values);
            return d;
        });
        
        return data;
    }

    depth.remove = function(selection) {
        selection.each(function(d) {
            d3.select(this).selectAll("svg").transition()
                .attr("height", 0).attr("width", 0).remove();
        });
    };

    depth.width = function(value) {
        if (!arguments.length) return width;
        width = value;
        return depth;
    };

    depth.height = function(value) {
        if (!arguments.length) return height;
        height = value;
        return depth;
    };

    depth.margin = function(value) {
        if (!arguments.length) return margin;
        margin = value;
        return depth;
    };

    depth.min = function(value) {
        if (!arguments.length) return min;
        min = value;
        return depth;
    };

    depth.max = function(value) {
        if (!arguments.length) return max;
        max = value;
        return depth;
    };

    depth.interpolate = function(value) {
        if (!arguments.length) return interpolate;
        interpolate = value;
        return depth;
    };
    
    depth.dispatch = function(value) {
		if(!arguments.length) return dispatch;
		dispatch = value;
		return depth;
    };

    return depth;
};
// this doesn't assume multiple lines in a chart
app.graphs.sparkline = function() {
	var height = 30,
		width = 800,
		max = 100,
		min = 0,
		margin = 2,
		dateFormat = d3.time.format("%x"),
		xScale = null,
		yScale = null,
		interpolate = "linear";

	function sparkline(selection) {
		selection.each(function(d) {
			d = d.values;

			var x = xScale ? xScale : d3.time.scale().domain(d.map(function(d) {
					return d.x;
				})),
				y = yScale ? yScale : d3.scale.linear().domain(d3.extent(d, function(d) {
					return d.y;
				}));

			x.range([margin, width - margin]);
			y.range([height - margin, margin]);

            // only one line per sparkline graph
			var line = d3.svg.line().x(function(d) {
				return x(d.x);
			}).y(function(d) {
				return y(d.y);
			}).interpolate(interpolate);

			var container = d3.select(this);

			var svg = container.selectAll("svg").data([0]);
			svg.enter().append("svg").attr("width", width).attr("height", height);

			var g = svg.selectAll("g").data([0]);
			g.enter().append("g");

			var path = g.selectAll(".graphs-line").data([d]);
			path.enter().append("path").attr("class", "graphs-line").attr("fill", "none").attr("stroke-width", "1").attr("stroke", "#08C");
			path.transition().attr("d", function(d) {
				return line(d);
			});
			
			// nick had wanted to change to add circles at each datapoint
			// he's added to the timeseries to do the reverse lookup in geodashboard
			// in timeseries, ther eis a separate function that could be extended to this if we want it later
			var dataPoints = g.selectAll(".graphs-dataPoint").data(d);
			dataPoints.enter().append("circle").attr("class", "graphs-dataPoint")
				.attr("r", 5).style("opacity", 0);
			dataPoints.attr("cx", function(d) { return x(d.x); }).attr("cy", function(d) { return y(d.y); });
			
			var clearText = g.selectAll(".graphs-clearText").data([0]);
			clearText.enter().append("text").attr("class", "graphs-clearText").attr("x", margin).attr("y", margin + 12);
			
			dataPoints
				.on("mouseover", function(d) {
					d3.select(this).style("opacity", 1);
					g.selectAll(".graphs-clearText").text(dateFormat(d.x) + ", " + d.y);
				})
				.on("mouseout", function(d) {
					d3.select(this).style("opacity", 0);
					g.selectAll(".graphs-clearText").text("");
				});

			// sparkline.exts.forEach(function(d) {
			// 	d.call(sparkline, svg);
			// });
		});
	}

	sparkline.height = function(value) {
		if (!arguments.length) return height;
		height = value;
		return sparkline;
	};

	sparkline.width = function(value) {
		if (!arguments.length) return width;
		width = value;
		return sparkline;
	};

	sparkline.interpolate = function(value) {
		if (!arguments.length) return interpolate;
		interpolate = value;
		return sparkline;
	};

	sparkline.margin = function(value) {
		if (!arguments.length) return margin;
		margin = value;
		return sparkline;
	};

	sparkline.max = function(value) {
		if (!arguments.length) return max;
		max = value;
		return sparkline;
	};

	sparkline.min = function(value) {
		if (!arguments.length) return min;
		min = value;
		return sparkline;
	};

	sparkline.x = function(value) {
		if (!arguments.length) return xScale;
		xScale = value;
		return sparkline;
	};

	sparkline.y = function(value) {
		if (!arguments.length) return yScale;
		yScale = value;
		return sparkline;
	};

	sparkline.ext = function(ext) {
		if (!sparkline.exts) sparkline.exts = [];

		sparkline.exts.push(ext);
	};

	if(sparkline.extensions) {
		sparkline.extensions.forEach(function(d) {
			d.call(sparkline);
		});
	}

	return sparkline;
};

app.graphs.sparkline.extend = function(extension) {
	if (!graphs.sparkline.extensions) graphs.sparkline.extensions = [];

	graphs.sparkline.extensions.push(extension);
};
// similar to the depth graph
// this can do multiple lines on a single graph

// testing  

app.graphs.timeSeries = function() {
	var width = 100, height = 100, // default values overwritten by functions below
		margin = { top: 0, right: 0, bottom: 0, left: 0 }, // default values
		padding = { top: 30, right: 30, bottom: 30, left: 60 }, // default values
		x = null, y = null, interpolate = "linear",
		min = 0, max = 10, xFormat = function(x) { return x; },
		yFormat = function(y) { return y; },
		sharedXAxis = true, sharedYAxis = true;
	
	function timeSeries(selection) {
		selection.each(function(data) {
		    // sanity checks to make sure we are getting real Dates
			data.values = data.values.map(function(e) {
				e.values = e.values.map(function(f) { f.x = new Date(f.x); f.y = +f.y; return f;})
					.sort(function(a, b) { return a.x - b.x; });
				return e;	
			});

			x = d3.time.scale().range([padding.left, width - padding.right]);
			y = d3.scale.linear().range([height - padding.bottom, padding.top]);
			
			var viz = d3.select(this);
			
			var svg = viz.selectAll("svg").data([data], function(d) { return d.key; });
			svg.enter().append("svg")
			  .attr("xmlns", "http://www.w3.org/2000/svg").attr("version", "1.1")
			  .attr("preserveAspectRatio", "none")
			  .attr("class", function(e) { return "graphs-time-series " + e.key; })
			  .attr("viewBox", "0 0 " + width + " " + height);
			  

			/* Plot */
			// creates a different plot for each line you might have in a datapoint
			// allows you to hvae different lines on the same chart
			// different from depth in that it allows point highlighting for each graphed element
			var arr = data.key.split(" ");
			
			var plotG = svg.selectAll(".graphs-plotG").data(function(e) { return e.values; });
			plotG.enter().append("g");
			  
			plotG.transition()
				.attr("class", function(f) { return "graphs-plotG " + f.key; })
                .style("color", function(f) { return f.color; })
                .style("stroke", function(f) { return f.color; })
                .style("fill", function(f) { return f.color; })
				.call(plots, arr[arr.length - 1]);

			plotG.exit().remove();
		});
	}
	
	function plots(selection, units) {
		selection.each(function(data, i) {
			var bisectDate = d3.bisector(function(d) { return d.x; }).left;
			// only show two digits for the point on the graph
			var valueFormat = d3.format(".2f");
			var dateFormat = d3.time.format("%Y-%m-%d");
			
			var plot = d3.select(this);

			// main title 
			var title = plot.selectAll(".graphs-title").data([data]);
			
			title.enter()
				.append("text")
				.attr("class", "graphs-title");

			title.transition()
				.attr("x", 70)
				.attr("y", 15)
				.style("text-anchor", "left")
				.text(function(d) { return _.isUndefined(d.title) ? d.key : d.title; });

			title.exit().remove();
			
			// X-Axis Label
			if(app.config.graphs.timeSeries.showUnits.graphXAxis)
			{
				var xAxisLabel = plot.selectAll(".graphs-xAxisLabel").data([data]);
				
				xAxisLabel.enter()
					.append("text")
					.attr("class", "graphs-xAxisLabel");
				
				xAxisLabel.transition()
					.style("text-anchor", "middle")
					.attr("x", 275)
					.attr("y", 247)
					.text("Date");
				
				xAxisLabel.exit().remove();
			}
			
			// source Label
			if(app.config.graphs.timeSeries.showSource && data.sources && data.sources.length > 0)
			{
				var sourceLabel = plot.selectAll(".graphs-sourceLabel").data([data]);

				sourceLabel.enter()
					.append("text")
					.text("Sources: ")
					.attr("class", "graphs-sourceLabel");
				sourceLabel.transition()
					.style("text-anchor", "end")
					.attr("x", 510)
					.attr("y", 25);
				sourceLabel.exit().remove();

				// TODO make sure with many sources it doesn't fall off the graph. Anchor to right.
				var sources;
				if (data.sources.length < 5) {
					sources = data.sources;
				} else {
					sources = data.sources.splice(0, 5);
					sources.push("...");
				}
				var sourceLabelIndex = sourceLabel.selectAll(".graphs-sourceLabelIndex").data(sources, function(d) { return(d); });
				sourceLabelIndex.enter()
					.append("a")
					.attr("xlink:href", function(d) { return d; })
					.attr("target", "clowder")
					.attr("class", "graphs-sourceLabelIndex")
					.text(function(d, i) { if (d == "...") { return d; } else { return "[" + (i + 1) + "]"; } })
					.append("title")
					.text("Original data source");
				sourceLabelIndex.exit().remove();
			}

			// Y-Axis Label
			if(app.config.graphs.timeSeries.showUnits.graphYAxis)
			{
				var yAxisLabel = plot.selectAll(".graphs-yAxisLabel").data([data]);
				
				yAxisLabel.enter()
					.append("text")
					.attr("class", "graphs-yAxisLabel");
				
				yAxisLabel.transition()
					.attr("transform", "rotate(-90)")
					.style("text-anchor", "middle")
					.attr("x", -110) // Moving up/down. + (up), - (down)
					.attr("y", 25) // Moving left/right. + (right), - (left)
					.text(function(d) { return d.unit; }); // Catch the undefined text
				
				yAxisLabel.exit().remove();
			}
			
			var x = d3.time.scale()
				.domain(data.xrange)
				.range([padding.left, width - padding.right]);
					
			var y = d3.scale.linear()
				.domain(data.yrange)
				.range([height - padding.bottom, padding.top]);
								
		    // background color
		    // TODO make sure to use enter/exit/remove to clean up when redrawing
		    var background = plot.selectAll(".graphs-background-rect").data([data.values]);
			background.enter()
			  .append("g")
			  .attr("class", "graphs-background")
			  .append("rect")
			  .attr("class", "graphs-background-rect")
			  .attr("transform", "translate(" + padding.left + ", " + padding.top + ")")
			  .attr("width", width - padding.left - padding.right)
			  .attr("height", height - padding.top - padding.bottom);

			// y axis before x axis so that y tick lines don't cover x axis
			// TODO there must be a better way
			var yAxisG = plot.selectAll(".graphs-y-axis").data([data.values]);
			
			yAxisG.enter()
				.append("g")
				.attr("class", "graphs-y-axis")
				.attr("transform", "translate(" + padding.left + ", 0)");
				
			yAxisG
				.transition()
				.call(d3.svg.axis()
					.scale(y)
					.orient("left")
					.ticks(5)
                    .tickSize(-width + padding.right + padding.left, 0, 5)
					.tickFormat(valueFormat));

			yAxisG.selectAll("text")
	          .attr("transform", function(d) {
	             return "translate(" + -5 + "," + 0 + ")";
	         });

			var xAxisG = plot.selectAll(".graphs-x-axis").data([data.values]);
			
			xAxisG.enter()
				.append("g")
				.attr("class", "graphs-x-axis");
			
			xAxisG
				.transition()
				.attr("transform", function(d) { return "translate(0, " + (height - padding.bottom ) + ")"; })
				.call(d3.svg.axis()
					.scale(x)
					.orient("bottom")
					.ticks(10)
					);

			// rotate labels at a 45 degree angle so that they don't overlap
			xAxisG.selectAll("text")
	          .attr("transform", function(d) {
	             return "translate(" + (this.getBBox().height*-1.5) + "," + this.getBBox().height + ")rotate(-45)";
	         });
				
			if (_.isUndefined(app.config.graphs.timeSeries['showLine']) || app.config.graphs.timeSeries['showLine'] == true) {
				var linePath = plot.selectAll(".graphs-line").data([data.values]);

				linePath.enter()
					.append("path")
					.attr("class", "graphs-line");

				linePath
					.transition()
					.attr("d",
					function(g) {
						return (d3.svg.line()
							.x(function(d) { return x(d.x); })
							.y(function(d) { return y(d.y); })
							.interpolate(interpolate))(g);
					}
				);

			}

			// Vertical Line on mouseOver
			//if(app.config.graphs.timeSeries.mouseOver.verticalLine)
			var mouseOverVerticalLine = plot.selectAll("graphs-mouseOverVerticalLine").data([data.values]);
			
			mouseOverVerticalLine.enter()
				.append("line")
				.attr("class", "graphs-mouseOverVerticalLine")
				.attr({
					"x1": 0,
					"y1": padding.top,
					"x2": 0,
					"y2": height - padding.bottom
				});
			
			mouseOverVerticalLine.exit().remove();
			
			// div shown with popup when mouse hovers over dot
			var popupDiv = d3.select('.graphs-tooltip').data([0]);
			popupDiv.enter()
				.append("div")
				.attr("class", "graphs-tooltip")
				.style("opacity", "0");
			popupDiv.exit().remove();

			// add little dots to the graph
			var dots = plot.selectAll('.graphs-dots').data(data.values);
			dots.enter()
				.append("path")
				.attr("class", "graphs-dots")
			dots
				.transition()
				.attr("transform", function(d) { return "translate(" + x(d.x) + "," + y(d.y) + ")"; })
				.attr("d", d3.svg.symbol().size(16));
			dots.exit().remove(); 

			var focusDot = plot.selectAll(".graphs-focus").data([0]);
			focusDot.enter()
				.append("circle")
				.attr("class", "graphs-focus");
			focusDot.append("circle");
			focusDot.exit().remove();

			// Mouse hover handling
			var rect = plot.selectAll(".graphs-listener-rect").data([data.values]);
			
			rect.enter()
				.append("rect")
				.attr("class", "graphs-listener-rect")
				.attr("transform", "translate(" + padding.left + ", " + padding.top + ")")
				.attr("width", width - padding.left - padding.right)
				.attr("height", height - padding.top - padding.bottom)
				.style("opacity", 0);
			rect.on("mouseout", function() {
				focusDot.transition().attr("r", "0");
				popupDiv.transition().style("opacity", 0);
				mouseOverVerticalLine.transition().style("opacity", 0);
			});
			rect.on("mouseover", function() {
				focusDot.transition().attr("r", "4.5");
				popupDiv.transition().style("opacity", 0.9);
				mouseOverVerticalLine.transition().style("opacity", 0.9);
			});
			rect.on("mousemove", mousemove);
				
			// handles the hovering over the graph and switching which datapoint is drawn on the map
			// give me the mouse's position, invert it
			// padding.left handles issue where rectangle is offset from the left padding of the svg
			// mouse position + graph padding + adjustments to accommodate
			// bisectDate function defined above: for data values, give me the index of nearest datapoint
			function mousemove() {
				var x0 = x.invert(d3.mouse(this)[0] + padding.left);
                var i = bisectDate(data.values, x0, 1);
                var d;
                if (i >= data.values.length) {
                	d = data.values[i - 1];
                } else {
	                var d0 = data.values[i - 1];
	                var d1 = data.values[i];
	                d = x0 - d0.x > d1.x - x0 ? d1 : d0;
                }

				var label = (d.label === undefined) ? dateFormat(d.x) : d.label;
				var value = valueFormat(d.y);
				
				var unit = "";
				if(app.config.graphs.timeSeries.showUnits.graphTooltip && data.unit)
				{
					unit = " " + data.unit;
				}
        
				mouseOverVerticalLine.attr("transform", function () { return "translate(" + x(d.x) + ",0)"; });

			    popupDiv.html("<b>Date</b>  : " + label + "<br/>" +
												"<b>Value</b> : "  + value + unit + " Avg.<br />")
			        .style("left", (this.getBoundingClientRect().left - padding.left + x(d.x)) + "px")     
			        .style("top", (this.getBoundingClientRect().top - padding.top + y(d.y)) + "px");
			    focusDot.attr("cx", x(d.x)).attr("cy", y(d.y));
			}
		});
	}

	timeSeries.width = function(value) {
		if(!arguments.length) return width;
		width = value;
		return timeSeries;
	};
	
	timeSeries.height = function(value) {
		if(!arguments.length) return height;
		height = value;
		return timeSeries;
	};
	
	timeSeries.margin = function(value) {
		if(!arguments.length) return margin;
		applyPadding(value.split(" "), margin);
		
		return timeSeries;
	};
	
	timeSeries.padding = function(value) {
		if(!arguments.length) return padding;
		applyPadding(value.split(" "), padding);
		
		return timeSeries;
	};
	
	timeSeries.min = function(value) {
		if(!arguments.length) return min;
		min = value;
		return timeSeries;
	};
	
	timeSeries.max = function(value) {
		if(!arguments.length) return max;
		max = value;
		return timeSeries;
	};
	
	timeSeries.x = function(value) {
		if(!arguments.length) return x;
		x = value;
		return timeSeries;
	};
	
	timeSeries.xFormat = function(value) {
		if(!arguments.length) return xFormat;
		xFormat = value;
		return timeSeries;
	};
	
	timeSeries.y = function(value) {
		if(!arguments.length) return y;
		y = value;
		return timeSeries;
	};
	
	timeSeries.yFormat = function(value) {
		if(!arguments.length) return yFormat;
		yFormat = value;
		return timeSeries;
	};
	
	timeSeries.sharedXAxis = function(value) {
		if(!arguments.length) return sharedXAxis;
		sharedXAxis = value;
		return timeSeries;
	};
	
	timeSeries.sharedYAxis = function(value) {
		if(!arguments.length) return sharedYAxis;
		sharedYAxis = value;
		return timeSeries;
	};
	
	timeSeries.interpolate = function(value) {
		if(!arguments.length) return interpolate;
		interpolate = value;
		return timeSeries;
	};
	
	function applyPadding(values, obj) {
		if(values[0]) {
			obj.top = obj.right = obj.bottom = obj.left = +values[0];
		}
		
		if(values[1]) {
			obj.right = obj.left = +values[1];
		}
		
		if(values[2]) {
			obj.bottom = +values[2];
		}
		
		if(values[3]) {
			obj.left = +values[3];
		}
	}

	return timeSeries;
};

// attach the event object to the app to ensure the object exists during the build process
app.event = {};
app.event.EventBus = _({}).extend(Backbone.Events);
app.router = {};
app.router.AppRouter = Backbone.Router.extend({
    
    routes : {
        "" : app.config.landingPage,
        "explore" : "home",
        "search" : "search",
        "compare" : "compare",
        "welcome": "welcome",
        "detail/location/:location(/startTime/:startTime/endTime/:endTime)(/parameters/:parameters)(/bin/:bin)(/depth/:depth)(/depthYear/:depthYear)(/depthBin/:depthBin)(/biologyStartTime/:biologyStartTime)(/biologyEndTime/:biologyEndTime)(/biologyBin/:biologyBin)(/)" : "populateDetailView",
        "about" : "about"
    },

    initialize: function() {
        var self = this;
        // on route changes update google analytics
        this.bind('route', this._pageView);

        // dynamically create a route listener for any "child views" specified in app.config
        _.each(app.config.views, function(view) {
          if (view.children) {
            _.each(view.children, function(childView) {
              if (childView.title && childView.link && childView.template) {
                var routeString = new RegExp('^' + view.link.replace('#', '') + '\/' + childView.link.replace('#', ''));
                self.route(routeString, childView.title.toLowerCase());
              }
            });
          }
        });
    },
    
    // split backbone and send information
    _pageView: function() {
        if (app.config.googleAnalytics != "") {
            var path = Backbone.history.getFragment();
            if (path.match(/^detail/)) {
                path = path.replace(/\/([^/]*)\/([^/]*)/g, '&$1=$2').replace(/&/, "?").replace(/\/$/, "");
            }
            ga('send', 'pageview', {page: "/" + path});
        }
    }
});

// attach the model object to the app to ensure the object exists during the build process
app.model = {};
app.model.Sensor = Backbone.AssociatedModel.extend({
    keepId: true,
    defaults: {
        selectable: true,
        selected: false,
        checked: false,
        sourceSelected: false,
        sourceChecked: false,
        parameterSelected: false,
        parameterChecked: false,
        areaChecked: false,
        areaSelected: false
    },

    initialize: function() {
        this.on("change:sourceSelected", this.isSelected, this);
        this.on("change:sourceChecked", this.isChecked, this);
        this.on("change:areaChecked", this.isChecked, this);
        this.on("change:areaSelected", this.isSelected, this);
        this.on("change:parameterSelected", this.isSelected, this);
        this.on("change:parameterChecked", this.isChecked, this);
        this.on("change:checked", this.isSelected, this);
        this.on("change:selected", this.isSelected, this);
        this.on("change:parameters[*]", this.isParamSelected, this);
        this.on("change:parameters[*].checked", this.isParamChecked);
    },

    isParamSelected: function() {
        if (this.isParamSelectedProcessing) return;
        this.isParamSelectedProcessing = true;
        var params = this.get("parameters");
        var paramSelected = _.findWhere(params, {selected: true});
        if (paramSelected) {
            this.set("parameterSelected", true);
        }
        this.isParamSelectedProcessing = false;
    },

    isParamChecked: function(changed) {
        if (this.isParamCheckedProcessing) return;
        this.isParamCheckedProcessing = true;
        var params = this.get("parameters");
        var paramChecked = _.findWhere(params, {checked: true});
        if (paramChecked) {
            this.set("parameterChecked", true);
        }
        this.isParamCheckedProcessing = false;
    },

    isSelected: function() {
        if (this.isSelectedProcessing) return;
        this.isSelectedProcessing = true;
        if (this.get("sourceSelected") || this.get("parameterSelected") || this.get("areaSelected") || this.get("checked")) {
            this.set("selected", true);
        } else {
            this.set("selected", false);
        }
        this.isSelectedProcessing = false;
    },

    isChecked: function(changed) {
        if (this.isCheckedProcessing) return;
        this.isCheckedProcessing = true;
        // if a parameter gets checked, and a source is checked, but this sensor's source is not checked, uncheck it
        if ( (this.get("parameterChecked")) && ( (app.collection.sources.getChecked().length == 1) && (!this.get("sourceChecked")) )  ) {
            this.isCheckedProcessing = false;
            return this.set("checked", false);
        }
        if ( (app.collection.sources.getChecked().length == 1) && (this.get("sourceChecked")) ) {
            this.set("checked", true);
        } else if (this.get("sourceChecked") || this.get("parameterChecked") || this.get("areaChecked")) {
            this.set("checked", true);
        } else {
            this.set("checked", false);
        }
        this.isCheckedProcessing = false;
    },

    url: function() {
        if (_.isUndefined(this.id)) {
            return app.config.clowderUrl + '/api/geostreams/sensors';
        } else {
            return app.config.clowderUrl + '/api/geostreams/sensors/' + encodeURIComponent(this.id);
        }
    },

    // set is called after app.collection.sensors calls fetch, then calls parse.
    // set is given JSON objects for each of the sensors, and creates a Backbone model for each sensor.
    // We are checking the JSON object's "type" property (which is the Data Source, like "epa") and
    // ensuring that the Data Source already exists in app.collection.sources.
    set: function ( key, val, options ) {
        var self = this;
        var args, attrs, _attrs, geometry;
        args = arguments;

        // Handle both `"key", value` and `{key: value}` -style arguments.
        if ( typeof key === 'object' ) {
            attrs = key;
            options = val;
        }
        // Handle GeoJSON argument.
        if ( attrs && attrs['type'] && attrs['type'] === 'Feature' ) {
            _attrs = _.clone( attrs['properties'] ) || {};

            if (attrs.created) _attrs['created'] = attrs.created;
            if (attrs.name) _attrs['name'] = attrs.name;
            if (attrs.min_start_time) _attrs['min_start_time'] = attrs.min_start_time;
            if (attrs.max_end_time) _attrs['max_end_time'] = attrs.max_end_time;
            _attrs['selectable'] = true,
            _attrs['selected'] = false,
            _attrs['checked'] = false,
            _attrs['sourceSelected'] = false,
            _attrs['sourceChecked'] = false,
            _attrs['parameterSelected'] = false,
            _attrs['parameterChecked'] = false,
            _attrs['areaChecked'] = false,
            _attrs['areaSelected'] = false;

            if (_attrs['type'] && _attrs['type']['id']) _attrs['type']['id'] = _attrs['type']['id'].toLowerCase().replace(/ /g, '-');

            if (attrs.parameters) {
                _attrs['parameters'] = new Backbone.Collection;
                _.each(attrs.parameters, function(parameterID) {
                    if (parameterID == null) return;
                    // if this parameterID matches a known parameter, we'll link them, otherwise we'll create a new parameter and link this sensor to it
                    // "alternates" is an array of possible spellings of that parameter

                    // find a parameter model for this parameterID, or see if it is listed as an alternate in any other parameter
                    var parameterModel = app.collection.parameters.find(function(model) {
                        return model.id == parameterID || _.contains(model.get("alternates"), parameterID);
                    });

                    // TODO after all datapoints and sensors are updated, only enable th next if/else when in debug mode
                    if (parameterModel !== undefined) {

                        // here we're adding this sensor to an existing parameter model
                        // by telling it "merge: true" we'll add this sensor to any other sensors it already has
                        app.collection.parameters.add({id: parameterModel.id, sensors: self}, {merge: true});

                    } else {
                        console.log(_attrs['name'] + " has a new parameter that is not yet listed in config.js : " + parameterID);/*RemoveLogging:skip*/
                        // here we're creating a new parameter model since the parameterID didn't match the ID of an existing parameter model
                        // and wasn't in any other parameters list of "alternates"
                        // we'll sanitize the ID, and if the sanitized version is different, we'll store that as it's alternate value so it can be found
                        parameterModel = app.collection.parameters.add({
                            id: parameterID.replace(/[^a-zA-Z0-9-_]/g, ''),
                            title: parameterID,
                            alternates: (parameterID == parameterID.replace(/[^a-zA-Z0-9-_]/g, '')) ? [] : [ parameterID ],
                            sensors: self
                        }, {merge: true});
                    }

                    // by pushing this parameterModel back onto "parameters", our sensor will have a reference to each parameter's model
                    _attrs.parameters.add(parameterModel);


                });
            }
            // Clone the geometry attribute.
            geometry = _.clone( attrs['geometry'] ) || null;
            if ( geometry ) {
                geometry.coordinates = geometry['coordinates'].slice();
            }
            _attrs['geometry'] = geometry;

            if ( attrs[this.idAttribute] ) {
                _attrs[this.idAttribute] = attrs[this.idAttribute];
            }
            args = [_attrs, options];
        }

        // this "finalizes" the Backbone model for this sensor
        var result = Backbone.Model.prototype.set.apply( this, args );

        // This next if statement associates a sensor's "region" (ie. "SU" for Lake Superior)
        // to a Backbone Model called an Area. We already use Area Models to define our Lakes,
        // and we're adding a field to app.config.areas.items called "region" to make the connection

//        TODO: sensors and areas don't have any knowledge of one another because we can't reliably link an area to a sensor's region yet. see next TODO...
        // verify that the sensor model has a region field (ie: "SU")...
//        if (_attrs && _attrs.region) {
            // check to see if there is a model for "SU" yet...
            // TODO: the findWhere works because we have defined all of the known regions in app.config.areas.items. This will not work in the future though, because we are assigning new id's to subregions (ie: Lake Erie Center). The region property from the API is simply "ER", which we assign to the Region. But we assign to the region ER for all subregions: Lake Erie Center, Lake Erie East and Lake Erie West. The findWhere function will find the first area that matches the same region, but since there are now multiple areas with that region, we have no guarantee that we have the right one. Here is some data to help in the future:
            // TODO: (cont.) API Data: type.id = ER, type.title = Lake Erie
//            var areaModel = app.collection.areas.where({region: _attrs.region});

//            if (areaModel != undefined) {
//                // handle different results from the collection response:
//                switch (areaModel.length) {
//                    case 1:
////                        var sensors = areaModel[0].get("sensors");
////                        // add this sensor model to the area's array of sensors
////                        if (sensors) sensors.push(result);
////                        // merge this new array of sensors back into the area model
////                        areaModel = app.collection.areas.add({ id: areaModel[0].id, region: _attrs.region, sensors: sensors}, {merge: true});
////                        // we'll also replace the text "SU" in the sensor model with the area model found previously in app.collection.areas
////                        result.set("region", areaModel);
//                        break;
//                    default:
//                        
//                        
//                }
//            }

            // if it is already an area model...
//            if (areaModel instanceof app.collection.Areas) {
//                // 
//                // ...get the area's array of sensors
//                var sensors = areaModel.get("sensors");
//                // add this sensor model to the area's array of sensors
//                if (sensors) sensors.push(result);
//                // merge this new array of sensors back into the area model
//                areaModel = app.collection.areas.add({ id: areaModel.id, region: _attrs.region, sensors: sensors}, {merge: true});
//                // we'll also replace the text "SU" in the sensor model with the area model found previously in app.collection.areas
//                result.set("region", areaModel);
//            } else {
//                // we didn't find an area model, so we're creating a new one
//                // since this will now show up on the Search page as a new Area, we want to pass in an id, region and a formatted title
//                // this won't be necessary if the area that matches _attrs.type is already defined in app.config.areas.items
//                // 
////                var title = _attrs.type.title.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
//                areaModel = app.collection.areas.add({
//                    id: _attrs.region.toLowerCase(),
//                    title: _attrs.region,
//                    region: _attrs.region,
//                    sensors: [this]
//                }, {merge: true});
//            }
//        }

        // This next if statement associates a sensor's "source" (ie. "epa" for Environmental Protection Agency)
        // to a Backbone Model called a Source. We already use Source Models,
        // so we'll add this sensor to the source model to make the connection

        if (_attrs && _attrs.type && _attrs.type.id) {
            // ensure that a "source" model exists
            // since _attrs.type includes id and name, the add method can be used. "add" won't duplicate for the same id
            var source = app.collection.sources.add(_attrs.type);

            // assign the source's color to this sensor
            if (source.get('color')) {
              this.set('color', source.get('color'));
            }

            // the add function on the following line is possible because we're using backbone-associations.js
            // it allows us to associate nested attributes and is a different function from the add used above with collections
            source.get("sensors").add(result);
            if (_attrs.region) source.get("regions").add({id: _attrs.region});

//          TODO: if we are able to connect areas to models, this should be implemented again
            // if an area model exists (as in, we replaced the region of "SU" with a Backbone Model, add it to this source
            // this allows us to do this: EPA --> Lake Superior --> Sensors
//            if (areaModel) {
//                source.get("areas").add(areaModel);
//            }
        }

      // This next if statement associates a sensor's "region" (ie. "Illinois")
      // to a Backbone Model called a Region. We already use Region Models,
      // so we'll add this sensor to the region model to make the connection

      if (_attrs && _attrs.region) {
        // ensure that a "region" model exists
        // add the region to the regions collection (or get the model if it already exists)
        var region = app.collection.regions.add({id: _attrs.region});
        // each region model maintains a Backbone Collection of sensors
        // associate this sensor with the region model
        //region.get("sensors").add(result);

        // each region model maintains a Backbone Collection of all of the parameters in all of its sensors
        // storing them here prevents having to calculate them dynamically later
        // loop through this sensor's parameters and add to the region model
        //result.get('parameters').each(function(param) {
        //    region.get("parameters").add(param);
        //});

        // each region model maintains a Backbone Collection of sources
        if (! region.get('sources')) {
          region.set('sources', new Backbone.Collection);
        }

        region.get("sources").add(this.get('type'));
      }

      if (_attrs && _attrs['huc']) {
        _.each(_.keys(_attrs['huc']), function(hucCode) {
          app.collection.hucs.add({                           // adds huc2, huc4 as models in the collection
            id: hucCode,
            title: hucCode.toUpperCase()
          });
            //.get('sensors').add(result);

        });
        _.each(_.keys(_attrs['huc']), function(hucCode) {               // hucCode is a string like "huc2"
          var hucN = app.collection.hucs.findWhere({id: hucCode});  // gets the huc2 or huc4 model
          if (hucN) {
            var hucModel = hucN.get('items').add({                      // "items" is a backbone collection in the huc2 model
              id: _attrs['huc'][hucCode]['code'],                       // should set the code from this sensor's huc2 object as id
              title: _attrs['huc'][hucCode]['name']                     // should set the name from this sensor's huc2 object as title
            });
            hucModel.get('sensors').add(result);                        // add this sensor to the hucModel
          }

        });

      }

      return result;

    },

      // The GeoJSON representation of a `Feature`.
      // http://www.geojson.org/geojson-spec.html#feature-objects
      // whenever the collection or model calls the toJSON() function, we manipulate the output
      // to suit our purposes. This allows us to completely alter what is stored in the "model" for a sensor
      // while still ensuring that a true GeoJSON object can be created from the model when needed
      toJSON: function ( options ) {
            var attrs, props, geometry;
            options = options || {};
            attrs = _.clone( this.attributes );
            props = _.omit( attrs, ['geometry', 'created', 'id']);
            // Add model cid to internal use.
            if ( options.cid ) {
              props.cid = this.cid;
            }
            // Clone the geometry attribute.
            geometry = _.clone( attrs['geometry'] ) || null;
            if ( geometry ) {
              geometry.coordinates = geometry['coordinates'].slice();
            }
            var json = {
              type: 'Feature',
              name: attrs.name,
              created: attrs.created,
              geometry: geometry,
              properties: props
            };
            if ( this.keepId ) {
              json[ this.idAttribute ] = this.id;
            }
            return json;
      }
});

// the GeoStreamService model interfaces with '/datapoints' on the API

// if we have already collected the datapoints from the API, the 'geostream' argument will not be empty, and subsequent page loads will not hit the API.
app.model.GeoStreamService = function(geostream, options) {
    this.geostream = geostream ? geostream : {};
    this.options = {};
    // set the API endpoint for this model
    this.options.url = app.config.clowderUrl + "/api/geostreams/datapoints?";
    this.options.urlTrend = app.config.clowderUrl + "/api/geostreams/datapoints/trends?";
    this.options.urlBins = app.config.clowderUrl + "/api/geostreams/datapoints/bin/";

    // lookup a datapoint by location
    this.getByLocation = function(location, callback) {
        // if we have this location in our cached results, return it
        if(this.geostream[location.id]) {
            callback(this.geostream[location.id]);
            return;
        } else {
          // this location is not in our cached results, so fetch from the API using id
          var query = this.options.url + $.param({sensor_id: location.id });
          $.ajax(query, {
              dataType: "jsonp",
              success: (function(geostream) {
                  return function(data) {
                      var cleanedData = cleanupData(data, location.id);

                      // keep a cached copy of the cleaned data
                      geostream[location.id] = cleanedData;
                      callback(cleanedData);
                  };
              })(this.geostream)
          });
        }
    };

    this.getTrend = function(parameter, season, callback){
        //We might need to add cached results here to help site responsiveness

        var binString;
        switch(app.config.binning.time) {
            case "decade":
                binString = "&decade=";
                break;
            case "lustrum":
                binString = "&lustrum=";
                break;
            case "year":
                binString = "&year=";
                break;
            case "semi":
            case "season":
                binString = "&semi=";
                break;
            case "month":
                binString = "&month=";
                break;
            default:
                binString = "";
        }

        season = season || '';

        var query = this.options.urlTrend + parameter  + binString + season;
        console.log(query);
        $.ajax(query, {
            dataType: "jsonp",
            success: function(geostream) {
                // return function(data) {
                var sensorResults = [];
                var parameterTitle = "";
                _.each(geostream, function(trend) {
                    var trendsObject = {};
                    var statsObject = {};
                    _.each(trend.properties, function(value, param) {                      
                      if (param == 'source') { return }
                      if (param.split(/_(.+)?/)[1]) statsObject[param.split(/_(.+)?/)[1]] = value;
                      var parameterModel = app.collection.parameters.find(function(model) {
                        return model.get("id") == param.split('_')[0];
                      });
                      if (parameterModel) {
                        parameterTitle = parameterModel.get('title').replace(/<(.*?)>/g, '');
                        if (parameterTitle) trendsObject[parameterTitle] = {'last_time': trend.last_time};
                      }
                    });
                    var sensor = app.collection.sensors.findWhere({id: +trend.sensor_id});
                    if (!_.isEmpty(statsObject)) {
                        $.extend( trendsObject[parameterTitle], statsObject );
                        var trends = sensor.get('trends');
                        if (trends) {
                            _.extend(trends, trendsObject);
                        } else {
                            sensor.set("trends", trendsObject);
                        }
                    }
                    sensorResults.push(sensor);

                });

                // keep a cached copy of the cleaned data
                //geostream[location.properties.name] = cleanedData;
                //callback(cleanedData);
                // };
                //app.event.eventBus.trigger("change:trends", sensorResults, parameterTitle);
                callback(sensorResults, parameterTitle);

            }

        });


    };
  // retrive binned datapoints
  this.getBinnedData = function(time, depth, sensor_id, cache, callback) {
    var query = this.options.urlBins + time + "/" + depth + "?" + $.param({sensor_id: sensor_id});
    console.log("Query for binned datapoints");
    // Check the cache for results for a sensor / bin
    if(this.geostream[sensor_id + time]) {
      callback(this.geostream[sensor_id + time]);
      return;
    } else {
      $.ajax(query, {
        dataType: "jsonp",
        success: (function(geostream) {
          return function(data) {
            if(cache) {
              geostream[sensor_id + time] = data;
            }
            callback(data);
          };
        })(this.geostream)
      });
    }
  };
};

// Given the data returned by clowder geostreams api this will
// reorganize the data. This will create a single object that
// represents a single reading.
// This can also bin multiple readings time wise, for example for
// SeaGrant this will combine multiple readings per year/season.
// [ {
//      year: 1999,
//      season: 'summer',
//      parameter: 'Chlorophyll a ug/l',
//      values: [ {
//         raw: [ 19.5, 20.1, 18.0, 19.8 ]
//         value: 77.4,
//         count: 4,
//         average: 19.35
//         depth: 0,
//         source: [ 'http://localhost:9001/clowder/XYZ' ]
//      }],
//      raw:
// } ]
//
// value is the sum of data
//
// This function asssumes that start/end date is the same and will
// only use the startTime returned!!!
function cleanupData(data, sensorID) {
    var cleanedData = [];

    // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
    // are averaged before being drawn in the graphs.
    // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
    // as an argument. From there, you can return 'smart', 'semi', or some other binning value
    // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
    // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
    var binningTime = "";
    if (app.config.binning.filterFunction) {
      binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: sensorID}));
    } else if (app.config.binning.time) {
      binningTime = app.config.binning.time;
    } else {
      binningTime = 'smart';
    }

    data.forEach(function(d) {
        // convert returned date to javascript
        var start_time = new Date(d.start_time);

        // check if there is a source in the parameter list
        var source = d.properties.source;

        // check if there is a depth code in parameter list
        var depthcode = d.properties.DEPTH_CODE;
        if (_.isUndefined(depthcode)) depthcode="NA";
        
        // map all these to EP
        if (["SRF", "LEP", "TRM"].indexOf(depthcode) > -1) {
          depthcode = "EP";
        }

        // find the depth binning requirements
        var depth = app.config.binning.depth * Math.ceil(d.geometry.coordinates[2] / app.config.binning.depth);

        // extract the information for each parameter
        for (var k in d.properties) {
            // source is handled differently
            if (k == 'source') continue;

            // depth code is handled differently
            if (k == 'DEPTH_CODE') continue;

            // check the value and make sure it is a number
            var v = parseFloat(d.properties[k]);
            if (_.isNaN(v)) continue;

            // filter to be used to find entry
            var filter = {};

            // find the parameter
            var parr = app.collection.parameters.filter(function(x) {
                if (_.isUndefined(x.get("alternates"))) {
                    
                }
                return (x.id == k) || x.get("alternates").indexOf(k) >= 0;
            });
            if (parr.length == 0) {
                // Format the id of this parameter: "Chlorophyll-a, ug/l" becomes "chlorophyll-a-ug-l"
                var p = k.replace(/[^a-zA-Z0-9-]{1,2}/g, '-').toLowerCase();
                app.collection.parameters.add({
                    id: p,
                    title: k,
                    alternates: [k],
                    detailView: app.config.parameters.defaults.detailView
                });
                filter.parameter = k;
                filter.detailView = app.config.parameters.defaults.detailView;
            } else {
                filter.parameter = parr[0].id;
                filter.detailView = parr[0].get('detailView');
            }

            // find the time binning requirements
            filter.year = start_time.getUTCFullYear();
            filter.label = filter.year.toString();
            var date = new Date(filter.year, 7, 1, 0, 0, 0);
            if (binningTime == 'semi') {
                if (start_time.getUTCMonth() < 6) {
                    filter.bin = "spring";
                    date.setUTCMonth(4);
                } else {
                    filter.bin = "summer";
                    date.setUTCMonth(7);
                }
                filter.label = filter.label + " " + filter.bin;
            } else if (binningTime == 'season') {
                filter.bin = getSeason(start_time);
                filter.label = filter.label + " " + filter.bin;
                if (filter.bin == "winter") {
                    date.setUTCMonth(1);
                } else if (filter.bin == "spring") {
                    date.setUTCMonth(4);
                } else if (filter.bin == "summer") {
                    date.setUTCMonth(7);
                } else if (filter.bin == "fall") {
                    date.setUTCMonth(10);
                }
            } else {
                if (_.contains(['month', 'day', 'hour', 'minute', 'none'], binningTime)) {
                    filter.month = start_time.getUTCMonth();
                    filter.label = filter.label + "-" + (filter.month + 1).toString();
                    date.setUTCMonth(start_time.getUTCMonth());
                }
                // if binningTime is "none" we won't bin beyond the day level
                if (_.contains(['day', 'hour', 'minute', 'none'], binningTime)) {
                    filter.day = start_time.getUTCDate();
                    filter.label = filter.label + "-" + filter.day.toString();
                    date.setUTCDate(filter.day);

                }
                if (_.contains(['hour', 'minute', 'none'], binningTime)) {
                    filter.hour = start_time.getUTCHours();
                    filter.label = filter.label + " " + filter.hour.toString();
                    date.setUTCHours(filter.hour);
                }
                if (_.contains(['minute', 'none'], binningTime)) {
                    filter.minute = start_time.getUTCMinutes();
                    filter.label = filter.label + ":" + filter.minute.toString();
                    date.setUTCMinutes(filter.minute);
                }

            }

            // find the entry in cleanedData, if not found add it.
            var entry;
            if (filter.parameter in cleanedData) {
                entry = _.findWhere(cleanedData[filter.parameter], filter);
                if (_.isUndefined(entry)) {
                    entry = filter;
                    entry.values = [];
                    entry.date = date;
                    cleanedData[filter.parameter].push(entry);
                }
            } else {
                entry = filter;
                entry.values = [];
                entry.date = date;
                cleanedData[filter.parameter] = [entry];
            }

            // find the appropriate depth value, if not found add it.
            var depthval = _.findWhere(entry.values, {depth: depth, depthcode: depthcode});
            if (_.isUndefined(depthval)) {
                depthval = {
                    average: 0,
                    depth: depth,
                    depthcode: depthcode,
                    count: 0,
                    value: 0,
                    raw: [ ],
                    source: [ ]
                };
                entry.values.push(depthval);
            }

            // update the depth value and compute average
            depthval.count++;
            depthval.value += v;
            depthval.raw.push(v);
            depthval.average = depthval.value / depthval.count;

            // add source to depth value if not there already
            if (!_.isUndefined(source) && depthval.source.indexOf(source) == -1) {
                depthval.source.push(source);
            }
        }
    });

    return _.flatten(_.values(cleanedData));
}

// given a date this will return the associated season.
function getSeason(date) {
    if ((date.getUTCMonth() < 2) || (date.getUTCMonth() == 2 && date.getUTCDate() < 21)) {
        return 'winter';
    }
    if ((date.getUTCMonth() < 5) || (date.getUTCMonth() == 5 && date.getUTCDate() < 21)) {
        return 'spring';
    }
    if ((date.getUTCMonth() < 8) || (date.getUTCMonth() == 8 && date.getUTCDate() < 21)) {
        return 'summer';
    }
    if ((date.getUTCMonth() < 11) || (date.getUTCMonth() == 11 && date.getUTCDate() < 21)) {
        return 'fall';
    }
    return 'winter';
}



app.model.Area = Backbone.AssociatedModel.extend({
    relations: [
        {
            type: Backbone.Many,
            key: 'sensors',
            relatedModel: app.model.Sensor
        }
    ],
    defaults: {
        sensors: new Backbone.Collection,
        markers: {},
        checked: false,
        selected: false
    },

    set: function ( key, val, options ) {
        var args, attrs, _attrs, geometry;
        args = arguments;

        // Handle both `"key", value` and `{key: value}` -style arguments.
        if ( typeof key === 'object' ) {
            attrs = key;
            options = val;
        }
        // Handle GeoJSON argument.
        if ( attrs && attrs['type'] && attrs['type'] === 'Feature' ) {
            _attrs = _.clone( attrs['properties'] ) || {};

            if (attrs.created) _attrs['created'] = attrs.created;
            if (attrs.name) _attrs['name'] = attrs.name;
            if (attrs.checked) _attrs['checked'] = attrs.checked;

            _attrs['selected'] = false;
            _attrs['sensors'] = new Backbone.Collection;

            // Clone the geometry attribute.
            geometry = _.clone( attrs['geometry'] ) || null;
            if ( geometry ) {
                geometry.coordinates = geometry['coordinates'].slice();
            }
            _attrs['geometry'] = geometry;

            if ( attrs[this.idAttribute] ) {
                _attrs[this.idAttribute] = attrs[this.idAttribute];
            }
            args = [_attrs, options];
        }

        return Backbone.Model.prototype.set.apply( this, args );
    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }


});
app.model.HucItem = Backbone.Model.extend({
    defaults: {
        title: false,
        checked: false,
        selected: false
    },

    initialize: function() {
        this.set('sensors', new Backbone.Collection);

    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }


});
app.model.Huc = Backbone.Model.extend({
    defaults: {
        title: false, // should be in the format of "huc2", "huc4", etc
        checked: false,
        selected: false
    },

    initialize: function() {
        this.set('items', new app.collection.HucItems);

    },

    // app.collection.hucs.findWhere({id: "huc2"}).groupByHUCSourcesSensors()
    // this creates a response that groups all huc regions in a group, ie, all the HUC2 regions
    // then in each HUC2 region (like Ohio 05), groups by source, then sensors
    groupByHUCSourcesSensors: function() {
        var response = new Backbone.Collection;
        this.get('items').each(function(huc) {
          var responseHucModel = response.add({id: huc.get('id'), title: huc.get('title')});
          responseHucModel.set('sources', new Backbone.Collection);
          huc.get('sensors').each(function(sensor) {
            var responseSource = responseHucModel.get('sources').add(sensor.get('type'));
            if (!responseSource.get('sensors')) {
              responseSource.set('sensors', new Backbone.Collection);
            }
            responseSource.get('sensors').add(sensor);
          });
        });
        return response;
    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; }


});
app.model.Region = Backbone.Model.extend({
  defaults: {
    checked: false,
    selected: false,
    title: null
  },

  // since we are not using the api, we must override any api-type functions
  // or else backbone will try to hit the api and complain because no url exists for this model
  sync : function() { return null; },
  fetch : function() { return null; }


});
app.model.Parameter = Backbone.AssociatedModel.extend({
    relations: [
        {
            type: Backbone.Many,
            key: 'sensors',
            relatedModel: app.model.Sensor
        }
    ],
    defaults: {
        sensors: [],
        markers: {},
        checked: false,
        selected: false,
        enabled: true,
        sourceChecked: false,
        detailView: app.config.parameters.defaults.detailView,
        searchView: app.config.parameters.defaults.searchView,
        exploreView: app.config.parameters.defaults.exploreView
    },
    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; },


    initialize: function() {
        var self = this;
        if (this.get('categories')) {
          _.each(this.get('categories'), function(cat) {
            var categoryModel = app.collection.parameterCategories.add({id: cat});
            if (!categoryModel.get('parameters')) {
              categoryModel.set('parameters', new Backbone.Collection);
            }
            categoryModel.get('parameters').add(self);
          });
        }

        this.on("change:sensors[*].selected", this.isSelected, this);
        this.on("change:sensors[*].sourceChecked", this.isSourceChecked, this);
    },

    isSelected: function() {
        var sensors = this.get("sensors");

        var isSensorSelected = _.any(sensors.models, function(sensor) {
            return sensor.get("selected");
        });
        this.set("selected", isSensorSelected);

    },

    isSourceChecked: function() {
        // The scenario: on the search page, click an area, then click a source
        // The problem this fixes: the parameters that show up in the ResultsView should update to only show the parameters for the checked source
        // How we solve this:
        // 1. get all the sensors for this parameter
        // 2. ask the sensor if it's source is checked - because we set this value on the model
        // 3. Using underscore's "some" function, we will return true if any of this parameter's sensors have a checked source
        // 4. If true, we will set "sourceChecked" true on this parameter. The ResultsView will then update the list of parameters
        var sensors = this.get('sensors');
        var sensorsSourceChecked = _.some(sensors.models, function(sensor) {
            return sensor.get("sourceChecked");
        });

        if (sensorsSourceChecked) {
            this.set("sourceChecked", true);
        } else {
            this.set("sourceChecked", false);
        }
    },

    getSelectedSensors: function() {
        return this.get("sensors").where({ selected: true });
    },

    getSelectableSensors: function() {
        return this.get("sensors").where({ selectable: true });
    },

    getCheckedSensors: function() {
        return this.get("sensors").where({ checked: true });
    }
});
app.model.Source = Backbone.AssociatedModel.extend({
    relations: [
        {
            type: Backbone.Many,
            key: 'sensors',
            relatedModel: app.model.Sensor
        },
        {
            type: Backbone.Many,
            key: 'areas',
            relatedModel: app.model.Area
        }
    ],
    defaults: {
        sensors: [],
        regions: new Backbone.Collection,
        areas: [],
        markers: {},
        checked: false,
        selected: false
    },

    initialize: function() {
        if (this.id) {
          this.set('id', this.id.toLowerCase().replace(' ', '-'));
        }

        this.on("change:sensors[*].selected", this.isSelected, this);

        // if prettyName isn't defined, this is a new Data Source and it isn't listed in sources.json
        // convert the source's title to Proper Case, and set it as the prettyName.
        if (this.get('prettyName') == undefined) {
            var prettyName = this.get('title');
            prettyName = prettyName.replace(/\w\S*/g, function(txt){return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase();});
            this.set("prettyName", prettyName);
        }
    },

    checkCategories: function() {
        var categories = {};
        var parameters = this.get('parameters');
        if (parameters == undefined || parameters.ids == undefined) {
          return;
        }
        _.each(parameters.ids, function(id) {
          var parr = app.collection.parameters.filter(function(x) {
            return (x.id == id) || x.get("alternates").indexOf(id) >= 0;
          });
          if (parr.length == 0) {
            if (categories['unknown'] == undefined) {
              categories['unknown'] = [];
            }
            categories['unknown'].push(id);
          } else {
            _.each(parr[0].get('categories'), function(category) {
              if (categories[category] == undefined) {
                categories[category] = [];
              }
              categories[category].push(parr[0].get('title'));
            });
          }
        });
        parameters.categories = categories;
    },

    isSelected: function() {
        var sensors = this.get("sensors");

        var isSensorSelected = _.any(sensors.models, function(sensor) {
            return sensor.get("selected");
        });
        this.set("selected", isSensorSelected);

    },

    // since we are not using the api, we must override any api-type functions
    // or else backbone will try to hit the api and complain because no url exists for this model
    sync : function() { return null; },
    fetch : function() { return null; },

    // returns "selected" sensors for this source
    getSelectedSensors: function() {
        return this.get("sensors").where({ selected: true });
    },

    // returns "selectable" sensors for this source
    getSelectableSensors: function() {
        return this.get("sensors").where({ selectable: true });
    },

    getCheckedSensors: function() {
        return this.get("sensors").where({ checked: true });
    },

    getColor: function() {
      if (!this.get('color')) {
        return "black";
      } else {
        return this.get('color');
      }
    }
});
app.model.Layer = Backbone.Model.extend({
    defaults: {
        markers: {},
        checked: false,
        initialized: false
    },

    initialize: function() {
      var self = this;
      _.each(this.get('properties'), function(value, key) {
          self.set(key, value);
      });
      this.set("layer", new L.TileLayer.WMS(this.get("wms"), {
          layers: this.get("id"),
          format: 'image/png',
          transparent: true,
          opacity: this.get("opacity")
      }));
      if (this.get("default") == "true") {
          this.set("checked", true);
      }
    }

});

app.collection.Sensors = Backbone.Collection.extend({
	url: app.config.clowderUrl + "/api/geostreams/sensors",
    model: app.model.Sensor,
    
    visible: function() {
        return this.where({ visible: true });
    },
    
    selected: function() {
        return this.where({ selected: true });
    },
    
    getSelectableSensors: function() {
        return this.where({ selectable: true });
    },

    getChecked: function() {
        return this.where({ checked: true });
    },

    getOrFetchStations: function(callback) {
        if (!this.models.length > 0) {
            this.fetch({
                success: function(collection) {
                    callback(collection);
                }
            });
        } else {
            // return "this" which refers to app.collection.sensors
            callback(this);
        }
    },
    
    getStationByName: function(name, callback) {
        this.getOrFetchStations((function(self) {
            return function(sensorsCollection) {
                // we could also use the following command, because we know that app.collection.sensors is populated now:
                // var station = app.collection.sensors.findWhere({name: name})

                var station = sensorsCollection.findWhere({name: name});
                if (station) {
                    callback(station.toJSON());
                } else {
                    callback(undefined);
                }
            }
        })(this));
    },

    uncheckAll: function() {
        _.each(this.models, function(sensor) {
            sensor.set({
                "sourceSelected" : false,
                "sourceChecked" : false,
                "areaChecked" : false,
                "areaSelected" : false,
                "parameterSelected" : false,
                "parameterChecked" : false,
                "checked" : false,
                "selected" : false,
                "selectable" : true
            });
        });
    },

    // parse isn't necessary here at all
    // parse happens after a collection fetch, and before saving each JSON object as a sensor model
    // but I left it in as a reminder in case we would ever want to "filter" our sensors
    // before they become sensor "models". ie: manipulate their structure, etc.
    parse: function(response) {
        return response;
    },
    
    // "select" is used by the draw tools to select sensors on the map.
    // if we use "fetch", all the sensors in the collection get reset, so "select" uses ajax 
    // and doesn't alter the collection
    select: function(params, callback) {
        $.ajax({
            url: this.url,
            dataType: "jsonp",
            data: $.param(params),
            success: function (sensors) {
                
                // anytime someone selects something on the map, we want to designate the sensors as "selectable" and all others as "unselectable"
                // here we are simply getting all known id's...
                var all = _.pluck(app.collection.sensors.toJSON(), 'id');
                // here we get the id's for the sensors selected on the map...
                var found = _.pluck(sensors, 'id');
                // and here we indicate which sensors were NOT selected on the map
                var disabledSensors = _.difference(all, found);
                
                // set "selectable" to false for all the sensors we didn't just select (since they are true by default)
                _.each(disabledSensors, function(disabledSensor) {
                    app.collection.sensors.findWhere({id: disabledSensor}).set({selectable: false, selected: false, areaChecked: false, sourceChecked: false, parameterChecked: false });
                });

                // at this point, the variable 'sensors' is the raw JSON of all our sensors returned from the API
                // however, when fetching this selection from the API, we only specified the coordinates.
                // If the user has already checked a Data Source or a Parameter, we should filter those out of what was returned from the API

                // let's check to see if a Data Source checkbox was already selected.
                // If so, we'll only include those and filter out the rest

                // this next function should return an array of Source models
                var selectedSources = app.collection.sources.getSourcesWithSensors().where({checked: true});
                // if the array contains more than one checked item, we'll proceed...
                if ( (selectedSources instanceof Array) && (selectedSources.length > 0) ) {
                    // we'll use the 'map' function to create an array of just the source ID's
                    selectedSources = _.map(selectedSources, function(source) {
                        return source.get('id');
                    });
                    // now we will update the sensors variable (remember that sensors was the raw data from the API)
                    // we'll use filter to only include the sensors that have one of the checked Data Sources in properties.type.id
                    sensors = _.filter(sensors, function(sensor) {
                        return (selectedSources.indexOf(sensor.properties.type.id) > -1);
                    });
                    // at this point, the sensors variable has been updated, removing any sensors where the Data Source was unchecked
                }

                // let's check to see if a parameter checkbox was already selected.
                // If so, we'll update our sensors variable that was returned from the API to only include those and filter out the rest

                // this next function should return an array of Parameter models
                var selectedParameters = app.collection.parameters.where({checked: true});
                // if the array contains more than one checked item, we'll proceed...
                if ( (selectedParameters instanceof Array) && (selectedParameters.length > 0) ) {
                    // we'll use 'map' to essentially reduce the parameter models to just the list of alternates
                    // and we wrap the map function in the 'flatten' function. This will give us one big array of all possible alternates
                    selectedParameters = _.flatten(_.map(selectedParameters, function(parameter) {
                        return parameter.get('alternates');
                    }));
                    // now we will update the sensors variable (remember that sensors was the raw data from the API)
                    // we'll use filter to only include the sensors that have one of the checked Parameters
                    sensors = _.filter(sensors, function(sensor) {
                        // since sensor.parameters is an array, and selectedParameters is an array, we'll use the "intersection" function.
                        // This will check if this sensor.parameters contains any of the parameters in selectedParameters,
                        // and if true, that sensor will remain in the "sensors" variable. All others will be removed.
                        return _.intersection(sensor.parameters, selectedParameters).length > 0;
                    });
                }

                // any sensor still in the sensors variable is going to be a "checked sensor"
                // we'll grab the sensor model for each, then assign the proper attributes
                // make sure that each sensor is both selectable and areaSelected
                _.each(sensors, function(sensor) {
                    var sensorModel = app.collection.sensors.findWhere({id: sensor.id});
                    sensorModel.set({
                        areaSelected: true,
                        selectable: true,
                        areaChecked: true
                    });
                });
                
                // we check if a callback function was passed so that we can perform some action after this complete
                // see SearchView.js
                if (typeof(callback) == 'function') {
                    callback(sensors);
                } else {
                    return sensors;
                }
                
            }
        });
        
    },

    groupByHUCcodeSourcesSensors: function(options) {
      var hucKey;
      var results = new Backbone.Collection;
      results.comparator = 'code';

      if (options.hucCode && ($.inArray(options.hucCode, [2,4,6,8]) > -1)) {
        hucKey = "huc" + options.hucCode;
      } else {
        console.warn("app.collection.sensors.groupByHUCcodeSourcesSensors(hucCode): hucCode argument is required.");
        return;
      }

      if (hucKey) {
        results['title'] = hucKey;
        // this.each refers to the collection. this.each will iterate all sensors
        this.each(function(sensor) {
          var huc = sensor.get('huc');
          if (huc[hucKey]) {
              var hucModel = results.add({id: huc[hucKey]['code'], code: huc[hucKey]['code'], title: huc[hucKey]['name']});
              hucModel.set('checked', true);
              if (! hucModel.get('sources')) {
                hucModel.set('sources', new Backbone.Collection);
              }
              var source = hucModel.get('sources').add(sensor.get('type'));
              if (! source.get('sensors')) {
                source.set('sensors', new Backbone.Collection);
              }
              source.get('sensors').add(sensor);
          }
        });
      }

      if (options.includedCodes) {
          return results.filter(function(sensor) {
              return _.contains(options.includedCodes, sensor.get('code'));
          });
      } else {
          return results;
      }

    }
});
// currently using this in the SearchView.js, but may further explore collections with other models
var Datapoints = Backbone.Collection.extend({
	url: app.config.clowderUrl + "/api/geostreams/datapoints"
});

app.datapoints = new Datapoints();
app.collection.Parameters = Backbone.Collection.extend({
    model: app.model.Parameter,
    comparator: 'title',

    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/parameters"

    getChecked: function() {
        return this.where({checked: true});
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },
    groupByParametersSourcesSensors: function(collection) {
      // clone the entire parameters collection (or a passed collection)
      // into a new Backbone Collection called "response"
      // filter the collection only for visible parameters
      // we'll make modifications and return them, but won't alter the original collection
      collection = collection || this;
      var response = new Backbone.Collection(collection.clone().where({exploreView: 'visible'}));

      // for each parameter in the collection
      response.each(function(parameter) {
        // each parameter will look through it's sensors and keep a list of their sources
        // the "HucItems" collection uses a Collection -> Item -> Sensors structure that is
        // reusable here
        var sources = new app.collection.HucItems;

        // loop through each sensor for this parameter
        parameter.get('sensors').each(function(sensor) {

          // if we call "add", it won't create a duplicate model for the source
          var source = sources.add(sensor.get('type'));

          // add this sensor to the source collection
          source.get('sensors').add(sensor);

        });

        // after looping through each parameter, add the sources collection as a property
        parameter.set('sources', sources);

      });

      return response;
    }
});
app.collection.Areas = Backbone.Collection.extend({
    model: app.model.Area,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/areas"

    clearSelection: function() {
        _.each(this.models, function(area) {
            if (area.get('items')) {
                area.get('items').each(function(areaInGroup) {
                    areaInGroup.set({
                        selected: false,
                        checked: false
                    });
                });
            } else {
                area.set({
                    selected: false,
                    checked: false
                });
            }
        });
    },

    getChecked: function() {
        return this.where({checked: true});
    },

    groupByAreaSourcesSensors: function(title) {
      title = title || "River Reaches";
      var results = this.findWhere({title: title}).get('items').clone();

      results.each(function(riverReach) {
        if (riverReach.get('sensors').length) {
          riverReach.set('checked', true);
          riverReach.set('sources', new Backbone.Collection);
          riverReach.get('sensors').each(function(sensor) {
            var source = riverReach.get('sources').add(sensor.get('type'));
            if (!source.get('sensors')) {
              source.set('sensors', new Backbone.Collection);
            }
            source.get('sensors').add(sensor);

          });
        }
      });

      return new Backbone.Collection(results.filter(function(area) {
        return area.get('sensors').length;
      }));
    }

});


app.collection.HucItems = Backbone.Collection.extend({
    model: app.model.HucItem,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/hucs"

    getChecked: function() {
        return this.where({checked: true});
    }

});


app.collection.Hucs = Backbone.Collection.extend({
    model: app.model.Huc,
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/hucs"

    getChecked: function() {
        return this.where({checked: true});
    }

});


app.collection.Regions = Backbone.Collection.extend({
  model: app.model.Region,
  // a url function is not defined since we are not using the api
  // url: app.config.clowderUrl + "/api/geostream/regions"

  getChecked: function() {
    return this.where({checked: true});
  },

  groupByRegionsParametersSensors: function() {
    var results = new Backbone.Collection;

    this.each(function(region) {
      var newRegion = results.add(region);
      var regionSensors = app.collection.sensors.where({region: region.id});
      _.each(regionSensors, function(sensor) {

        if (! newRegion.get('parameters')) {
          newRegion.set('parameters', new Backbone.Collection);
        }

        sensor.get('parameters').each(function(parameter){
          var newParameter = parameter.clone();
          newParameter.unset('sensors');

          var parameterSensors = parameter.get('sensors');
          parameterSensors = parameterSensors.where({region: region.id});

          newParameter.set('sensors', parameterSensors);
          newRegion.get('parameters').add(newParameter);
        });

      });
    });
    return results;
  },

  groupByRegionsSourcesSensors: function() {
    var results = new Backbone.Collection;

    this.each(function(region) {
      var newRegion = region.clone();
      var newRegionModel = results.add(newRegion);
      newRegionModel.get('sources').each(function(source) {
        source.set('sensors', new Backbone.Collection);
        var sourceSensors = app.collection.sensors.filter(function(sensor) {
          return sensor.get('type').id == source.id && sensor.get('region') == region.id;
        });
        source.get('sensors').add(sourceSensors);
      });
    });
    return results;
  }

});


app.collection.Sources = Backbone.Collection.extend({
    model: app.model.Source,
    comparator: 'id',
    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/sources"

    getChecked: function() {
        return this.where({ checked: true });
    },

    // calling 'app.collection.sources.getColor(id)' will return a consistent color for each source
    getColor : function(source) {
        return source == "epa" ? "#0D71BA" :
            source == "usgs" ? "#39B54A" :
            source == "lec" ? "#4D6363" :
            source == "gsfmp" ? "#CC2128" :
            source == "iadn" ? "#F7941E" :
            "white";
    },

  // used by the Explore page. This goes through each source and returns the following:
  // Sources grouped by Areas, then Sensors. ie: [ epa: [ lake-superior: [ sensors: [ sensor1, sensor2...]]
    groupBySourcesAreasSensors: function(callback) {

        // we'll return an array of objects, one for each source
        var response = [];

        // "this" refers to the collection. this.each iterates over each source in app.collection.sources.
        this.each(function(source) {
            // build this source's object
            var sourceResponse = {};
            sourceResponse.id = source.get('id');
            sourceResponse.title = source.get('title');
            sourceResponse.prettyName = source.get('prettyName');
            sourceResponse.acronym = source.get('acronym');
            sourceResponse.description = source.get('description');
            sourceResponse.color = source.get('color');
            sourceResponse.areas = new Array(app.collection.sources.getSourcesWithSensors().length);

            // in order to group by areas, we have to iterate each of the sensors and create the areas array
            // as we find unique areas
            // note that if there are no areas defined in app.config.areas, these won't exist yet, so we will need to create them
            var sourceSensors = source.get("sensors");
            // if the source doesn't have any sensors, it was likely added in the config,
            // but no data or sensors exist yet so we won't inlude it in the response
            if (!sourceSensors.length) return true;
            sourceSensors.each(function(sensor) {
                var sensorRegion = sensor.get('region');
                if (sensorRegion) {
                    sensorRegion = sensorRegion.toLowerCase();
                    // get the Backbone model for this area
                    var areaModel = app.collection.areas.findWhere({ id: sensorRegion });
                    // if no Backbone model exists for this area (aka: the sensor's "region" property), create it
                    if (!areaModel) areaModel = app.collection.areas.add({id: sensorRegion, region: sensorRegion.toUpperCase(), title: sensorRegion.toUpperCase() });
                    var areaTitle = areaModel.get("title");
                    // foundArea must check to see if this area has already been found
                    // otherwise you'd get a huge array where each sensor recreated the area it was in
                    var foundArea = _.findWhere(sourceResponse.areas, { id: sensorRegion });

                    if (!foundArea) {
                        foundArea = { id: sensorRegion, title: areaTitle, sensors: [sensor.toJSON()]};
                        sourceResponse.areas.splice(areaModel.get('sortOrder'), 0, foundArea);
                    } else {
                        foundArea.sensors.push(sensor.toJSON());
                    }

                }

            });

            response.push(sourceResponse);
        });
        if (typeof callback == 'function') {
            callback(response);
        } else {
            return response;
        }
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },

  // get a unique list of all (non-ignored) parameters for the passed in source
  getParameters: function(source) {
    var parameters = [];
    if (source.get('sensors')) {
      source.get('sensors').each(function(sensor) {
        _.each(sensor.get('parameters'), function(parameter) {
          if (parameter.get('id') != 'ignored') {
            parameters.push(parameter);
          }
        });
      });
    }
    return _.unique(parameters);
  },

    // maintain backwards compatibility of the original getAreas function
    getAreas: function(options) {
      return this.groupBySourcesAreasSensors(options);
    },

    getSourcesWithSensors: function(options) {
      var response = this.filter(function(source){
        return source.get('sensors') && source.get('sensors').length;
      });
      return new Backbone.Collection(response);
    }


});
app.collection.ParameterCategories = Backbone.Collection.extend({
    comparator: 'title',

    // a url function is not defined since we are not using the api
    // url: app.config.clowderUrl + "/api/geostream/parameterCategories"

    getChecked: function() {
        return this.where({checked: true});
    },

    uncheckAll: function() {
        this.each(function(model) {
            model.set("checked", false);
        });
    },

  groupByParameterCategoriesSourcesSensors: function() {
    // clone the entire parameters collection into a new Backbone Collection called "response"
    // filter the collection only for visible parameters
    // we'll make modifications and return them, but won't alter the original collection
    var response = this.clone();

    response.each(function(parameterCategory) {
      if (parameterCategory.get('parameters')) {
        parameterCategory.set('checked', true);
        var catResponse = app.collection.parameters.groupByParametersSourcesSensors(parameterCategory.get('parameters'));
        if (catResponse.get('sources') && catResponse.get('sources').length) {
          parameterCategory.set('parameters', catResponse);
        }

      }
    });

    response = response.filter(function(category) {
      return category.get('parameters') && category.get('parameters').find(function(param) {
          return param.get('sources') && param.get('sources').length && param.get('sources').find(function(source) {
              return source.get('sensors') && source.get('sensors').length;
            });
        });
    });

    return response;
  }
});
app.collection.Layers = Backbone.Collection.extend({
    model: app.model.Layer,
    // a url function is not defined since we are not using the api
    // url: app.clowderUrl + "/api/geostream/areas"


    clearSelection: function() {
        _.each(this.models, function(layer) {
            layer.set("checked", false);
        });
    }


});
app.view = app.view || {};

Backbone.View.prototype.close = function(){
  this.remove();
  this.unbind();
  if (this.onClose){
    this.onClose();
  }
}

function getBinningDesignators(value) {
    switch(value) {
        case "season":
            return {
                title: "Season",
                items: [
                    { id: 'spring', name: 'Spring' },
                    { id: 'summer', name: 'Summer' },
                    { id: 'fall', name: 'Fall' },
                    { id: 'winter', name: 'Winter' }
                 ]
            };
            break;
        case "semi":
        case "semi-year":
        case "semi-annually":
            return {
                title: "Season",
                items: [
                    { id: 'spring', name: 'Spring' },
                    { id: 'summer', name: 'Summer' }
                ]
            };
            break;
        case "monthly":
            return {
                title: "Month",
                items: [
                    {id: 'jan', name: 'January'},
                    {id: 'feb', name: 'February'},
                    {id: 'mar', name: 'March'},
                    {id: 'apr', name: 'April'},
                    {id: 'may', name: 'May'},
                    {id: 'jun', name: 'June'},
                    {id: 'jul', name: 'July'},
                    {id: 'aug', name: 'August'},
                    {id: 'sep', name: 'September'},
                    {id: 'oct', name: 'October'},
                    {id: 'nov', name: 'November'},
                    {id: 'dec', name: 'December'}
                ]
            };
            break;
        default:
            return {title: "Bin", items: []};
            break;
    }
}
app.view.AppView = Backbone.View.extend({
    el : "#appContainer",

    events : {
      "click #header-go-home" : "goHome"
    },

    initialize : function() {
        var self = this;
        this.router = app.router.router = new app.router.AppRouter();

        this.eventBus = app.event.eventBus = _({}).extend(Backbone.Events);

        // compile the map pin, used in HomeView and DetailView
        this.mapPin = _.template($('#map-pin-template').html())();

        this.geoStreamService = app.model.geoStreamService = new app.model.GeoStreamService();

        this.currentView = null;

        $('meta[name="keywords"]').attr('content', app.config.metadata.keywords);
        $('meta[name="description"]').attr('content', app.config.metadata.description);
        $('meta[property="og:title"]').attr('content', app.config.metadata.title);
        $('meta[property="og:description"]').attr('content', app.config.metadata.description);
        $('meta[property="og:url"]').attr('content', app.config.metadata.url);
        $('meta[name="twitter:title"]').attr('content', app.config.metadata.title);
        $('meta[name="twitter:description"]').attr('content', app.config.metadata.description);
        $('meta[name="twitter:url"]').attr('content', app.config.metadata.url);


        // certain views need to update when the window size changes
        $(window).resize((function(self) {
            return function(event) {
                self.render();
            };
        })(this));

        if (app.config.areas) {
            if (app.config.areas.groups) {
                app.collection.areas = new app.collection.Areas();
                _.each(app.config.areas.groups, function(areaGroup) {
                    app.collection.areas.add(
                        {
                            id: areaGroup.title.toLowerCase().replace(/ /g, '-'),
                            title: areaGroup.title,
                            items: new app.collection.Areas(areaGroup.items)
                        }
                    );
                });
            } else if (app.config.areas.items) {
                app.collection.areas = new app.collection.Areas(app.config.areas.items);
            }

        }

        app.collection.regions = new app.collection.Regions();
        app.collection.hucs = new app.collection.Hucs();

        if (app.config.sources && app.config.sources.items) {
            app.collection.sources = new app.collection.Sources(app.config.sources.items);
        }

        if (app.config.parameters.categories) {
          app.collection.parameterCategories = new app.collection.ParameterCategories(app.config.parameters.categories);
        } else {
          app.collection.parameterCategories = new app.collection.ParameterCategories;
        }

        if (app.config.parameters && app.config.parameters.items) {
            app.collection.parameters = new app.collection.Parameters(app.config.parameters.items);
        } else {
          app.collection.parameters = new app.collection.Parameters;
        }

        if (app.config.sensors && app.config.sensors.items) {
            app.collection.sensors = new app.collection.Sensors(app.config.sensors.items);
        }

        if (app.config.layers && app.config.layers.items) {
            app.collection.layers = new app.collection.Layers(app.config.layers.items);
        }

        if (app.config.glossary && app.config.glossary.items) {
            app.collection.glossary = new Backbone.Collection(app.config.glossary.items);
        }

        // update all views when new data gets in
        app.collection.sensors.fetch({
            dataType: "jsonp",
            success: function(data) {
              app.event.eventBus.trigger("sensors:refresh");
              return data;
            }
        });

        // if a view in app.config contains an array of "children" pages, setup route listeners
        _.each(app.config.views, function(view) {
          if (view.children) {
            _.each(view.children, function(childView) {
              if (childView.title && childView.link && childView.template) {
                if (!childView.view) childView.view = 'PageView';
                self.router.on("route:" + childView.title.toLowerCase(), (function(self) {
                  return function() {
                    document.title = app.config.title + " - " + childView.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + view.link + "']").parent().addClass("active");
                    if(!app.view[childView.title.toLowerCase() + "View"]) {
                      app.view[childView.title.toLowerCase() + "View"] = new app.view[childView.view](childView);
                    }

                    self.setView(app.view[childView.title.toLowerCase() + "View"]);
                    self.render();
                  }
                })(self));
              }
            });
          }
        });

        if (app.config.views.about && app.config.views.about.title && app.config.views.about.link) {
            this.router.on("route:about", (function (self) {
                return function () {
                    document.title = app.config.title + " - " + app.config.views.about.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.about.link + "']").parent().addClass("active");
                    self.initializeAboutView();
                    self.render();
                };
            })(this));
        }

        if (app.config.views.welcome && app.config.views.welcome.title && app.config.views.welcome.link) {
            this.router.on("route:welcome", (function (self) {
                return function () {
                    document.title = app.config.title + " - " + app.config.views.welcome.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.welcome.link + "']").parent().addClass("active");
                    self.initializeWelcomeView();
                    self.render();
                };
            })(this));
        }

        if (app.config.views.compare && app.config.views.compare.title && app.config.views.compare.link) {
            this.router.on("route:compare", (function(self) {
                return function() {
                    document.title = app.config.title + " - " + app.config.views.compare.title;
                    $("ul.navbar > li").removeClass("active");
                    $("a[href='" + app.config.views.compare.link + "']").parent().addClass("active");
                    self.initializeCompareView();
                    self.render();
                };
            })(this));

            app.collection.parameterBins = new Backbone.Collection();

            // always sort parameter alphabetically in the dropdowns
            app.collection.compareSelectedParameters = new Backbone.Collection();
            app.collection.compareSelectedParameters.comparator = 'title';

            // sort dates by id so they are in order by time
            app.collection.compareSelectedDates = new Backbone.Collection();
            app.collection.compareSelectedDates.comparator = 'id';
        }


        // on route:home, set currentView to the HomeView
        this.router.on("route:home", (function(self) {
            return function() {
                document.title = app.config.title;
                $("ul.navbar > li").removeClass("active");
                $("a[href='" + (app.config.views.home.link || "#explore") + "']").parent().addClass("active");
                self.initializeHomeView();
                self.render();
            };
        })(this));

        this.router.on("route:populateDetailView", (function(self) {
          return function(location, startTime, endTime, parameters, season, depth, depthYear, depthSeason, biologyStartTime, biologyEndTime, biologyBin) {
            document.title = app.config.title + " - Details";
            self.initializeDetailView(location, startTime, endTime, parameters, season, depth, depthYear, depthSeason, biologyStartTime, biologyEndTime, biologyBin);
          };
        })(this));

        this.router.on("route:search", (function(self) {
            return function() {
                document.title = app.config.title + " - " + (app.config.views.search.title || "Search");
                $("ul.navbar > li").removeClass("active");
                $("a[href='" + (app.config.views.search.link || "#search") + "']").parent().addClass("active");
                self.initializeSearchView();
                self.render();
            };
        })(this));

        // catch event fired by explore-pill or map-pin clicks
        // use this to fire the detail page route.
        this.eventBus.bind("change:location", (function(self) {
            return function(location) {
                var loc = "location/" + location + "/";
                self.router.navigate("detail/" + loc, {trigger : true, replace : false});
            };
        })(this));

        // listens for the event fired to grab station stats for a station.
        // this whole block might be better suited in the ExploreMapView.
        // I wanted to avoid the MapView needing to know about the StationService.
        this.eventBus.bind("station:fetch", (function(self) {
          return function(stationName) {
              var sensor = app.collection.sensors.findWhere({name: stationName});
              if (sensor) {
                  app.event.eventBus.trigger("station:update", sensor);
              }
          };
        })(this));

        this.footerView = new app.view.FooterView({config: app.config.views.footer});
        this.navBarView = new app.view.NavBarView().render();

        // should really happen on refresh of sources/paramters
        app.collection.sources.forEach(function(source) {
            source.checkCategories();
        });

      // this block loops through areas that contain "items" (as is done for River Reaches in GLTG)
      // and uses GeoJson-Utils to associate sensors to the sub-area where they reside
      if (app.collection.areas && app.collection.areas.length) {
        app.event.eventBus.once("sensors:refresh", function() {
          app.collection.areas.each(function(area) {
            if (area.get('items')) {
              var items = area.get('items');
              app.collection.sensors.each(function (sensor) {
                items.find(function (riverReach) {
                  if (sensor.get('geometry') &&
                    sensor.get('geometry')['coordinates'] &&
                    riverReach.get('geometry') &&
                    riverReach.get('geometry')['coordinates']) {
                    if (gju.pointInPolygon(sensor.get('geometry'), riverReach.get('geometry'))) {
                      riverReach.get('sensors').add(sensor);
                      sensor.set(area.get('id'), riverReach.get('id'));
                      return true;
                    }
                  }
                });
              });
            }
          });
        });
      }

      // get the config
      $.get(app.config.clowderUrl + "/api/geostreams/config", function(data) {
        _.extend(app.config, data);
      });

      Backbone.history.start();
    },

    render : function() {
        if(this.currentView) this.currentView.render();
    },


    goHome : function() {
      this.router.navigate("", {trigger : true, replace : true});
    },

    // handles changing the current view.
    setView : function(view) {
      // if the new view is not the current view.
      if(this.currentView !== view) {
        // detach (preserves event listeners) the current view from the DOM
        if(this.currentView) this.currentView.$el.detach();
        this.currentView = view;
        this.$("#content-container")
          .append(this.currentView.el); // append the new view.
      }
//      this.render();
    },

    initializeHomeView : function() {
      if(!app.view.homeView) {
        app.view.homeView = new app.view.HomeView({
          parentView: this,
          mapPin: this.mapPin,
          router: this.router,
          geoStreamService : this.geoStreamService,
          stationService: this.stationService,
          attributes: app.config.views.home.attributes,
          config: app.config.views.home
        });
      }

      this.setView(app.view.homeView);
    },

    initializeDetailView : function() {
      // create the DetailView if is doesn't exist,
      // or if the existing detailView doesn't match the ID of the location we want
      if(!this.detailView || this.detailView.locationID != arguments[0]) {
        this.detailView = new app.view.DetailView({
          parentView : this,
          eventBus : this.eventBus,
          mapPin : this.mapPin,
          router : this.router,
          stationService : this.stationService,
          geostreamService : this.geoStreamService,
          arguments: arguments,
          attributes: app.config.views.detail.attributes,
          config: app.config.views.detail
        });
      }

      // call the function to set up the view.
      this.setView(this.detailView);
      this.detailView.initiate.apply(this.detailView, arguments);
    },

    initializeSearchView : function() {
      if (!app.view.searchView) {
          app.view.searchView = new app.view.SearchView({
            parentView : this,
            eventBus : this.eventBus,
            router : this.router,
            stationService : this.stationService,
            geostreamService : this.geoStreamService,
            attributes: app.config.views.search.attributes,
            config: app.config.views.search
        });
      }

      this.setView(app.view.searchView);
  },

  initializeAboutView : function() {
    if(!app.view.aboutView) {
      app.view.aboutView = new app.view.AboutView(app.config.views.about);
    }

    this.setView(app.view.aboutView);
   },

    initializeWelcomeView : function() {
        if(!app.view.welcomeView) {
            app.view.welcomeView = new app.view.WelcomeView(app.config.views.welcomeView);
        }

        this.setView(app.view.welcomeView);
    },

    initializeCompareView : function() {
     if(!app.view.compareView) {
       app.view.compareView = new app.view.CompareView({
       });
     }

     this.setView(app.view.compareView);
   }
});

/* 
 * Handles the controls on the detail page.
 * Specifically covers the parameters checkboxes and the date slider.
 * Does not properly map to a single div within the DetailView because
 * of the way the tabs are setup in the page.
 */
app.view.ControlsView = Backbone.View.extend({
    attributes: function() {
        return {
            tagName: this.options.attributes.tagName,
            id: this.options.attributes.id
        }
    },

	events : {
		'change #times' : 'changeTime'
	},
	
	initialize : function(options) {
        this.config = options;
        _.bindAll(this, "renderSeasonSelect");
	    this.eventBus = options.eventBus || app.event.eventBus;
        this.eventBus.on("locationUpdated", this.location(arguments));
        this.eventBus.on("renderSeasonSelect", this.renderSeasonSelect);
	  
        this.parametersID = options.parametersID || null;
        this.binDropdown = options.binDropdown || null;
        this.sliderID = options.sliderID || null;
        this.dateRangeID = options.dateRangeID || null;
        this.yearFormID = options.yearFormID || null;
        this.yearSelectID = options.yearSelectID || null;
        this.name = options.name || null;

        this.render = _.bind(this.render, this);
	},
	
	/*
	 * This blog: http://ianstormtaylor.com/break-apart-your-backbonejs-render-methods/
	 * recommended making the render functions return 'this' to make them chainable.
	 * I only got around to applying it here.
	 */
	render : function() {
		this.renderParameters(this.parameters)
			.renderSlider(this.times)
			.renderSeasonSelect();
	},

	// uses a d3 paradigm to avoid reconstructing the slider every time a
	// dimension changes. Allows the same function to handle a change to the
	// dimensions and also create the slider from scratch if it hasn't been
	// initialized yet.
	renderSlider : function(times, location) {
        if (!this.sliderID) return;
		var $slider = d3.select("#" + this.sliderID);

		var data = { items: times, location: location };

        var instance = $slider.selectAll(".instance").data([data], function(d) { return d.location; });
        instance.enter().append("div").attr("class", "instance")
        .call((function (self) {
            return function(selection) {
                selection.each(function(d) {
                    $(this).slider({
                        range : true,
                        step : 1,
                        values : [0, d.items.length - 1],
                        min : 0,
                        max : d.items.length - 1,
                        change : function(event, ui) {
                            var values = ui.values;
                            self.changeTimes(values[0], values[1]);
                        },
                        create : function(event, ui) {
                            self.changeTimes(0, d.items.length - 1);
                        },
                        slide: function(event, ui) {
                            var values = ui.values;
                            self.sliderTimes(self.times[values[0]], self.times[values[1]]);
                        }
                    });
                });
            };
        })(this));

        instance.exit().remove();
		
		return this;
	},
	
	// uses a d3 paradigm to avoid reconstructing the parameter checkboxes every
  // time a dimension changes. Allows the same function to handle a change to
  // the dimensions and also create the checkboxes from scratch if they
  // haven't been initialized yet.
	renderParameters : function(parameters, selected, location, colors) {
        if (!this.parametersID) return;
        var parametersID = this.parametersID;
        var $parameters = d3.selectAll("#" + parametersID);

        var data = parameters.map(function(d) {
            var parameterModel = app.collection.parameters.findWhere({id: d});
            if (parameterModel) {
                return { parameter: parameterModel, location: location, color: colors(d) };
            }
        });
        
        var controlGroup = $parameters.selectAll(".control-group").data(data, function(d) { return d.location + "." + d.parameter.id; });
        controlGroup.enter().append("div").attr("class", "control-group");
        controlGroup.exit().remove();

        var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
        controls.enter().append("div").attr("class", "controls");

        var input = controls.selectAll("input").data(function(d) { return [d]; });

        var controlLabel = controlGroup.selectAll(".control-label-parameters").data(function(d) { return [d]; });
        controlLabel.enter().append("label");
        controlLabel.attr("class", function(d) { return "control-label-parameters " + d.parameter.id; })
            .style("color", function(d) { return d.color; })
            .attr("for", function(d) { return d.parameter.id; }).html(function(d) {
              var parameterWithUnit = d.parameter.get('title');
              if (d.parameter.get('unit') && d.parameter.get('unit').length) {
                  parameterWithUnit += " <small>(";
                  parameterWithUnit += d.parameter.get('unit');
                  parameterWithUnit += ")</small>";
              }
              return parameterWithUnit;
          }); //TODO: hack to display Chl a with an italic a. fix this with parameter name mapping
            
        input.enter().append("input").attr("type", "checkbox").attr("name", "parameter")
            .property("checked", function(d, i) { if(selected) return selected.filter(function(e) { return e === d.parameter.id; } ).length === 0 ? null : true; })
            .attr("id", function(d) { return d.parameter.id; }).attr("value", function(d) { return d.parameter.get('title'); })
            .on("click", (function(self) {
                return function() {
                    self.changeParameters();
                }
            })(this));
		return this;
	},
	
	renderSeasonSelect : function() {
    if (!this.binDropdown) return;
    if (this.binDropdown && this.binDropdown.filterFunction && this.binDropdown.sensor) {
      if (!this.binDropdown.filterFunction(this.binDropdown.sensor)) {
        return;
      }
    }
    var data = getBinningDesignators(app.config.binning.time) || { title: "Season", items: [{ id: "spring", name: "Spring"}, { id: "summer", name: "Summer" }]};

      var seasonForm = d3.select("#" + this.binDropdown.seasonFormID);

	  var controlGroup = seasonForm.selectAll(".control-group").data([data.items]);
	  controlGroup.enter().append("div").attr("class", "control-group");
	  controlGroup.exit().remove();
	  
	  var label = controlGroup.selectAll(".control-label-season").data(function(d) { return [d]; });
	  label.enter().append("label").attr("class", "control-label-season").attr("for", this.binDropdown.seasonSelectID).html(data.title);
	  label.exit().remove();
	  
	  var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
	  controls.enter().append("div").attr("class", "controls");
	  controls.exit().remove();

	  var select = controls.selectAll("#" + this.binDropdown.seasonSelectID).data(function(d) { return [d]; });
	  select.enter().append("select").attr("id", this.binDropdown.seasonSelectID).attr("class", "input-season-select")
	    .on("change", (function(self) {
	      return function() {
	        self.changeSeason();
	      }
	    })(this));
	  select.exit().remove();
	  
	  var options = select.selectAll("option").data(function(d) { return d; });
	  options.enter().append("option").attr("value", function(d) { return d.id; }).html(function(d) { return d.name; })
	  options.exit().remove();
	  
	  return this;
	},

    renderYearSelect : function(times, that) {
        if (!that.yearFormID) return;
        // get the unique years. It gives us a list like: 1998, 1999, 2000...
        var data = _.uniq(_.map(times, function(time) {
                var year = new Date(time);
                return year.getUTCFullYear();
        }));
        var yearForm = d3.select("#" + this.yearFormID);

        var controlGroup = yearForm.selectAll(".control-group").data([data]);
        controlGroup.enter().append("div").attr("class", "control-group");
        controlGroup.exit().remove();

        var label = controlGroup.selectAll(".control-label-year").data(function(d) { return [d]; });
        label.enter().append("label").attr("class", "control-label-year").attr("for", this.yearSelectID).html("Year");
        label.exit().remove();

        var controls = controlGroup.selectAll(".controls").data(function(d) { return [d]; });
        controls.enter().append("div").attr("class", "controls");
        controls.exit().remove();

        var select = controls.selectAll("#" + this.yearSelectID).data(function(d) { return [d]; });
        select.enter().append("select").attr("id", this.yearSelectID).attr("class", "input-year-select")
            .on("change", (function(self) {
                return function() {
                    self.changeYear();
                }
            })(this));
        select.exit().remove();

        var options = select.selectAll("option").data(function(d) { return d; }, function(d) { return d; });
        options.enter().append("option").attr("value", function(d) { return d; }).html(function(d) { return d; });
        options.exit().remove();
        options.order();

        this.year = $("#" + this.yearSelectID).val();

        return this;
    },

    setParameters : function(parameters, selected, colors) {
        this.parameters = parameters;
        var selectedFound = selected || this.getSelectedParameters();
        this.renderParameters(parameters, selectedFound, this.location(), colors);
    },

    setTimes : function(times) {
        this.startTime = times[0];
        this.endTime = times[times.length - 1];
        this.times = times;
        this.renderSlider(times, this.location());
        this.renderSeasonSelect();
    },

    setYears: function(years) {
        this.renderYearSelect(years, this);
    },

    // the location is used by the d3 selections to determine if the controls
    // need to be repopulated.
    location : function(location, season) {
        if(!location) return this.loc;

        this.loc = location + (season || "");
    },


    // sets the time range next to the slider.
    sliderTimes : function(start, end) {
      var prettyStart = new Date(start);
      prettyStart = (prettyStart.getUTCMonth() + 1) + "/" + prettyStart.getUTCDate() + "/" + prettyStart.getUTCFullYear();

      var prettyEnd = new Date(end);
      prettyEnd = (prettyEnd.getUTCMonth() + 1) + "/" + prettyEnd.getUTCDate() + "/" + prettyEnd.getUTCFullYear();

      $('#' + this.dateRangeID).html(prettyStart + ' - ' + prettyEnd);
    },

    // catches changes to the date slider and fires an event to change the
    // route.
    changeTimes : function(start, end) {
      this.startTime = this.times[start];
      this.endTime = this.times[end];

      this.sliderTimes(this.startTime, this.endTime);

      if(this.name == 'biology') {
          app.event.eventBus.trigger("change:biologyDates", this.startTime, this.endTime);
      } else {
          app.event.eventBus.trigger("change:times", this.startTime, this.endTime);
      }

    },

  // catches changes to the parameters and fires an event to change the route.
	changeParameters: function() {
		app.event.eventBus.trigger("change:parameters", this.getSelectedParameters());
	},

	changeSeason: function() {
	    if(this.name == 'biology') {
            app.event.eventBus.trigger("change:biologyBin", $("#" + this.binDropdown.seasonSelectID).val(), this.name);
        } else {
            app.event.eventBus.trigger("change:bin", $("#" + this.binDropdown.seasonSelectID).val(), this.name);
        }

	},

    changeYear: function() {
        this.year = $("#" + this.yearSelectID).val();
        app.event.eventBus.trigger("change:year", this.year);
    },

	  // used by the DetailView to determine which parameters are already selected.
	  // mostly useful when going back and forth between the same DetailView,
	  // so that the selected parameters aren't reset.
    getSelectedParameters : function() {
        var ret = [];
        
        $('.parameters input[type="checkbox"]:checked').each(function() {
            ret.push(encodeURIComponent($(this).attr('id')));
        });
        
        return ret;
    }
});
app.view.DetailView = Backbone.View.extend({
    events : {
        "click #detail-go-back" : "goBack",
        "click #showDistribution" : "showDistribution",
        "click .download:not(.disabled)": app.utils.interceptDownload
      },

      initialize: function(options) {
        var that = this;
        if(app.collection.sensors.length > 0) {
          that.initializeAfterFetch(options);
        } else {
            app.collection.sensors.fetch({
                success: function(collection) {
                    that.initializeAfterFetch(options);
                }
            });

        }

      },

      initializeAfterFetch : function(options) {
        var self = this;
        this.config = options.config;
        _.bindAll(this, 'render', 'setTabTitles', 'updateDownloadLink');
        this.timeSeriesModal = _.template($("#time-series-modal-template").html())();
        this.boxWhiskerModal = _.template($("#box-whisker-modal-template").html())();
        this.dataAggregateModal = _.template($("#data-aggregate-modal-template").html())();

        this.infoIcon = _.template($("#info-icon-template").html());
        this.sourceModals = _.template($("#source-modals-template").html())();
        this.stationService = options.stationService;
        this.geostreamService = options.geostreamService;
        this.router = options.router;
        this.eventBus = options.eventBus;
        this.mapPin = options.mapPin;
        // locationID is used in AppView when rendering the DetailView.
        // if the locationID doesn't match the DetailView we are rendering,
        // we'll initialize a new DetailView. Resolves GEOD-309
        this.locationID = options.arguments[0] || null;

        // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
        // are averaged before being drawn in the graphs.
        // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
        // as an argument. From there, you can return 'smart', 'semi', or some other binning value
        // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
        // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
        this.binningTime = "";

        this.sensor = app.collection.sensors.findWhere({name: this.locationID});
        if (app.config.binning.filterFunction) {

            this.binningTime = app.config.binning.filterFunction(this.sensor);
        } else if (app.config.binning.time) {
          this.binningTime = app.config.binning.time;
        } else {
          this.binningTime = 'smart';
        }

        this.startTime = null;
        this.endTime = null;
        this.parameters = null;

        this.$el.append(_.template($(this.config.templateSelector).html())());
        var that = this;
        this.miniMap = []
        this.config.miniMap.forEach(function(map) {
            map.config["parentView"] = that;
            map.config["mapPin"] = that;
            that.miniMap.push(new app.view.MiniMapView(map.config));
            that.$(map.id).append(that.miniMap[that.miniMap.length-1].el);
        });

        if (this.config.controlsViews && this.config.controlsViews.length) {
            _.each(this.config.controlsViews, function(controlView) {
                controlView['parentView'] = self;
                controlView['eventBus'] = self.eventBus;
                if (controlView.config.binDropdown && controlView.config.binDropdown.filterFunction) {
                  controlView.config.binDropdown.sensor = app.collection.sensors.findWhere({name: self.locationID});
                }
                self[controlView.name] = new app.view.ControlsView(controlView.config);
            })
        }

//        if (this.config.timeSeriesViews && this.config.timeSeriesViews.length) {
//            _.each(this.config.timeSeriesViews, function(timeSeriesView) {
//                timeSeriesView['parentView'] = self;
//                timeSeriesView['eventBus'] = self.eventBus;
//                self[timeSeriesView.name] = new app.view.TimeSeriesView(timeSeriesView.config);
//            });
//        }

          this.aggregateTimeSeriesView = new app.view.TimeSeriesView({
          parentView : this,
        });

        this.$("#aggregate-timeseries").append(this.aggregateTimeSeriesView.el);

        this.limnologyView = new app.view.LimnologyView({
          parentView : this,
        });

        this.$("#limnology-chart").append(this.limnologyView.el);

        this.limnologyView.changeDepth((function(self) {
          return function(i) {
            self.eventBus.trigger("change:depth", i);
          };
        })(this));

        this.timeSeriesView = new app.view.TimeSeriesView({
          parentView : this,
        });
        this.stackedBarViews  = [];

        //this.$("#timeseries-at-depth").append(this.timeSeriesView.el);

        this.$('a[data-toggle="tab"]').on("shown", function(self) {
          return function(event) {
            self.render();
          };
        }(this));

        this.$el.append(this.timeSeriesModal);
        this.$el.append(this.boxWhiskerModal);
        this.$el.append(this.dataAggregateModal);

          // setup a loading spinner we can turn on and off
          var spinner = app.config.views.detail.spinnerOptions || {
                lines: 13 // The number of lines to draw
                , length: 28 // The length of each line
                , width: 14 // The line thickness
                , radius: 42 // The radius of the inner circle
                , scale: 1 // Scales overall size of the spinner
                , corners: 1 // Corner roundness (0..1)
                , color: '#000' // #rgb or #rrggbb or array of colors
                , opacity: 0.25 // Opacity of the lines
                , rotate: 0 // The rotation offset
                , direction: 1 // 1: clockwise, -1: counterclockwise
                , speed: 1 // Rounds per second
                , trail: 60 // Afterglow percentage
                , fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
                , zIndex: 2e9 // The z-index (defaults to 2000000000)
                , className: 'spinner' // The CSS class to assign to the spinner
                , top: '50%' // Top position relative to parent
                , left: '50%' // Left position relative to parent
                , shadow: false // Whether to render a shadow
                , hwaccel: false // Whether to use hardware acceleration
                , position: 'absolute'
            };
          this.spinner = new Spinner(spinner);

          // the spinner isn't appended to the DOM until the first time it gets called.
          app.event.eventBus.on("detail:spinner:start", function() {
              self.$("#detail-panel").append(self.spinner.spin().el);
          });
          // use the eventBus to stop the spinner from anywhere
          app.event.eventBus.on("detail:spinner:stop", function() {
              self.spinner.stop();
          });



        // This function will return a url string in the order required by the view.
        // ie: location/time/parameters/depth/bin
        // Each individual change event listener in this view only needs to check to see if
        // its attribute (like parameter, bin, depth or time) is already set -
        // if so, it will replace it with the new values.
        // Otherwise it will append the attributes to the end of the url
        // The urlReOrder function does the job of making sure the order
        // matches what is required by this view.
        this.urlReOrder = function(newUrl) {
            // find "detail/location/ID/
            var locationRegex = /detail\/location\/.+?\//g;
            var locationFragment = locationRegex.exec(newUrl) || "";

            // find "startTime/DATE/endTime/DATE/
            var timeRegex = /startTime\/.*\/endTime\/.*?\//g;
            var timeFragment = timeRegex.exec(newUrl) || "";

            // find "parameters/PARAMETERS
            var parametersRegex = /parameters\/.*?\//g;
            var parametersFragment = parametersRegex.exec(newUrl) || "";

            // find "bin/SEASON"
            var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
            var binRegexString = binIDs.join("|");
            var binRegex = new RegExp("bin\/(" + binRegexString + ")\/");
            var binFragment = binRegex.exec(newUrl);
            // since we are using parenthesis to match spring or summer, regex returns
            // an array of matches. We only need the first one, which is the entire url, so we use [0] below.
            binFragment = (binFragment ? binFragment[0] : "");

            // find "depth/NUMBER"
            var depthRegex = /depth\/\d+\//g;
            var depthFragment = depthRegex.exec(newUrl) || "";

            // find "depthYear/YEAR"
            var depthYearRegex = /depthYear\/.*?\//g;
            var depthYearFragment = depthYearRegex.exec(newUrl) || "";
            depthYearFragment = (depthYearFragment ? depthYearFragment[0] : "");

            // find "depthBin/SEASON"
            var depthBinRegex = new RegExp("depthBin\/(" + binRegexString + ")\/");
            var depthBinFragment = depthBinRegex.exec(newUrl);
            // since we are using parenthesis to match spring or summer, regex returns
            // an array of matches. We only need the first one, which is the entire url, so we use [0] below.
            depthBinFragment = (depthBinFragment ? depthBinFragment[0] : "");

            // find "biologyStartTime/DATE/biologyEndTime/DATE/
            var biologyDateRegex = /biologyStartTime\/.*\/biologyEndTime\/.*?\//g;
            var biologyFragment = biologyDateRegex.exec(newUrl) || "";


            // find "biologyBin/SEASON"
            var biologyBinRegex = /biologyBin\/.*?\//g;
            var biologyBinFragment = biologyBinRegex.exec(newUrl) || "";
            biologyBinFragment = (biologyBinFragment ? biologyBinFragment[0] : "");

            return locationFragment + timeFragment + parametersFragment + binFragment + depthFragment + depthYearFragment + depthBinFragment + biologyFragment + biologyBinFragment;

          };

          // fire the route for a change to the selected time range in the detail view - times tab
          this.eventBus.bind("change:times", (function(self) {
              return function(start, end) {
                  var timeFragment = "startTime/" + encodeURIComponent(start) + "/endTime/" + encodeURIComponent(end) + "/";
                  var newUrl = Backbone.history.fragment;

                  if(Backbone.history.fragment.indexOf("startTime") >= 0) {
                      newUrl = newUrl.replace(/startTime\/[^\/]*\/endTime\/[^\/]*\//, timeFragment);
                  } else {
                      newUrl = newUrl + timeFragment;
                  }
                  self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
              };
          })(this));

          // fire the route for a change to the selected time range in the detail view - biology tab
          this.eventBus.bind("change:biologyDates", (function(self) {
              return function(start, end) {
                  var timeFragment = "biologyStartTime/" + encodeURIComponent(start) + "/biologyEndTime/" + encodeURIComponent(end) + "/";
                  var newUrl = Backbone.history.fragment;

                  if(Backbone.history.fragment.indexOf("biologyStartTime") >= 0) {
                      newUrl = newUrl.replace(/biologyStartTime\/[^\/]*\/biologyEndTime\/[^\/]*\//, timeFragment);
                  } else {
                      newUrl = newUrl + timeFragment;
                  }
                  self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
              };
          })(this));

          // fire the route for a change to the selected parameters in the detail view
          this.eventBus.bind("change:parameters", (function(self) {
              return function(parameters) {
                  self.parameters = parameters;
                  var parametersFragment =  parameters.length === 0 ? "" : "parameters/" + parameters.join() + "/";
                  var newUrl = Backbone.history.fragment;

                  if(Backbone.history.fragment.indexOf("parameters") >= 0) {
                      newUrl = newUrl.replace(/parameters\/.*?\//g, parametersFragment);
                  } else {
                      newUrl = Backbone.history.fragment + parametersFragment;
                  }
                  self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
              };
          })(this));

          // fires the route for a change to the bin value
          this.eventBus.bind("change:bin", (function(self) {
              return function(bin, view) {
                  if (view == "depth") {
                      self.eventBus.trigger("change:depthBin", bin, view);
                      return;
                  } else if (view == "biology") {
                    self.eventBus.trigger("change:biologyBin", bin, view);
                    return;
                  }
                  var binFragment = "";
                  var newUrl = Backbone.history.fragment;
                  var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                  var binRegex = binIDs.join("|");
                  var binRegexObject = new RegExp("bin\/("+ binRegex +")?\/");

                  if (bin != "") {
                      binFragment = "bin/" + bin + "/";
                  }

                  if (Backbone.history.fragment.indexOf("bin") >= 0) {
                      newUrl = newUrl.replace(binRegexObject, binFragment);
                  } else {
                      newUrl = newUrl + binFragment;
                  }

                  self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

              };
          })(this));

          // fires the route for a change to the depth value in the limnology chart.
          this.eventBus.bind("change:depth", (function(self) {
              return function(depth) {
                  var depthFragment = "depth/" + depth + "/";
                  var newUrl = Backbone.history.fragment;

                  if(Backbone.history.fragment.indexOf("depth") >= 0) {
                      newUrl = newUrl.replace(/depth\/[^\/]*\/?/, depthFragment);
                  } else {
                      newUrl = newUrl + depthFragment;
                  }
                  self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
              };
          })(this));

          // fires the route for a change to the depth value in the limnology chart.
          this.eventBus.bind("change:year", (function(self) {
              return function(depthYear) {
                  var depthYearFragment = "depthYear/" + depthYear + "/";
                  var newUrl = Backbone.history.fragment;

                  if(Backbone.history.fragment.indexOf("depthYear") >= 0) {
                      newUrl = newUrl.replace(/depthYear\/[^\/]*\/?/, depthYearFragment);
                  } else {
                      newUrl = newUrl + depthYearFragment;
                  }
                  self.router.navigate(self.urlReOrder(newUrl), {trigger : true, replace : true});
              };
          })(this));

          // fires the route for a change to the bin value
          this.eventBus.bind("change:depthBin", (function(self) {
              return function(depthBin) {
                  var depthBinFragment = "";
                  var newUrl = Backbone.history.fragment;
                  var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                  var binRegex = binIDs.join("|");

                  if (depthBin != "") {
                      depthBinFragment = "depthBin/" + depthBin + "/";
                  }

                  if (Backbone.history.fragment.indexOf("depthBin") >= 0) {
                      newUrl = newUrl.replace(new RegExp("depthBin\\/("+ binRegex +")?\\/"), depthBinFragment);
                  } else {
                      newUrl = newUrl + depthBinFragment;
                  }

                  self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

              };
          })(this));

          // fires the route for a change to the bin value
          this.eventBus.bind("change:biologyBin", (function(self) {
              return function(biologyBin) {
                  var biologyBinFragment = "";
                  var newUrl = Backbone.history.fragment;
                  var binIDs = _.map(getBinningDesignators(self.binningTime).items, function(bin) { return bin.id} );
                  var binRegex = binIDs.join("|");

                  if (biologyBin != "") {
                      biologyBinFragment = "biologyBin/" + biologyBin + "/";
                  }

                  if (Backbone.history.fragment.indexOf("biologyBin") >= 0) {
                      newUrl = newUrl.replace(new RegExp("biologyBin\\/("+ binRegex +")?\\/"), biologyBinFragment);
                  } else {
                      newUrl = newUrl + biologyBinFragment;
                  }

                  self.router.navigate(self.urlReOrder(newUrl), { trigger : true, replace : true });

              };
          })(this));



      },

    initiate : function(location, startTime, endTime, parameters, bin, depth, depthYear, depthBin,  biologyStartTime, biologyEndTime, biologyBin) {
        this.locationID = location;
        this.sensor = app.collection.sensors.findWhere({name: this.locationID});

        app.collection.sensors.getStationByName(location, (function(self) {
          return function(station) {
              self.eventBus.trigger("locationUpdated", location, bin);
              self.miniMap.forEach(function(map) {
                  map.setGeoJsonFeature([ station ]);
                  map.render();
              });

              self.go(station, startTime, endTime, parameters, bin, depth, depthYear, depthBin, biologyStartTime, biologyEndTime, biologyBin);
            };
        })(this));
    },

    go : function(station, startTime, endTime, parameters, bin, depth, depthYear, depthBin,  biologyStartTime, biologyEndTime, biologyBin) {
        var self = this;
        self.sensorID = station.id;
        app.event.eventBus.trigger("detail:spinner:start");

        // If start / end time are null, we need to initialize the range slider with time data
        if(startTime == null && endTime == null || !sessionStorage.getItem(station.id)) {
            // Generate the dates for the sensor min / max
            var sensor = app.collection.sensors.findWhere({id: station.id});
            var dates = getDates(new Date(sensor.get('min_start_time')), new Date(sensor.get('max_end_time')));

            // save dates in session so we don't regenerate for the same sensor
            sessionStorage.setItem(station.id, JSON.stringify(dates));

            // here is how I understand the following:
            // the setTimes function sets the times for the controlView, which calls renderSlider,
            // which calls changeTimes, which triggers a "change:times" event which re-renders this view
            self.controlsView.setTimes(dates);

            // the return in the following case is heavily dependent on the setTimes function called above.
            // The first time running the DetailView go function, you won't have a startTime or endTime
            // so this returns and waits for setTimes to finish. See notes just above...
            return;
        }

        if(biologyStartTime == null && biologyEndTime == null && self.controlsBiologyView != null) {
                biologyStartTime = startTime;
                biologyEndTime = endTime;

        }

        var time = self.binningTime;
        if (_.isUndefined(self.binningTime)) {
            time = 'none';
        }

        // If self.binningTime is 'smart', then determine aggregation by temporal range
        if (self.binningTime == 'smart') {
            time = getAggregationBin(new Date(startTime), new Date(endTime));
            $("#bin-duration").text("Averaged by " + time).append(' <a href="#data-aggregate-modal" role="button" data-toggle="modal"><i class="icon-info-sign"></i></a>');
        } 

        // Use default depth if no depth value provided
        var sensorDepth = depth;
        if(sensorDepth === undefined || sensorDepth === null) {
            sensorDepth = app.config.binning.depth;
        }

        // Fetch and Display sensor data for a particular time bin
        this.geostreamService.getBinnedData(time, sensorDepth, station.id, true, function(data) {
            
            self.startTime = startTime = new Date(startTime);
            self.endTime = endTime = new Date(endTime);
            biologyStartTime = new Date(biologyStartTime);
            biologyEndTime = new Date(biologyEndTime);

            // If a list of selected parameters are provided, then only display selected 
            parameters = parameters == null ? [] : parameters.split(",");

            // Retrieve generated dates for this station
            var dates = JSON.parse(sessionStorage.getItem(station.id));
            self.controlsView.setTimes(dates);
            
            // Used to filter data outside of the range of start / end time
            var filterDatesByRange = function(x, startTime, endTime) {
                var date = new Date(x.date);
                var datapoint = {year: date.getUTCFullYear(), month: date.getUTCMonth(), day: date.getUTCDate(), hour: date.getUTCHours(), minute: date.getUTCMinutes() };

                if (startTime.getUTCFullYear() > datapoint.year) {
                    return false;
                }
                if (startTime.getUTCFullYear() == datapoint.year) {
                    if (!_.isUndefined(datapoint.month)) {
                        if (startTime.getUTCMonth() > datapoint.month) {
                            return false;
                        }
                        if (startTime.getUTCMonth() == datapoint.month) {
                            if (!_.isUndefined(datapoint.day)) {
                                if (startTime.getUTCDate() > datapoint.day) {
                                    return false
                                }
                                // I'm leaving this here in case we decide to support filter down to hour/minutes 
                                // Since the slider does not filter to a smaller unit than days, I'm removing this unnecessary check
                                // The slider would need be smarter to change it's increment when the date range is small enough 
                                // in order to support showing hourly / minute data
                                // if (startTime.getUTCDate() == datapoint.day) {
                                //   if (!_.isUndefined(datapoint.hour)) {
                                //     if (startTime.getUTCHours() > datapoint.hour) {
                                //       return false;
                                //     }
                                //     if (startTime.getUTCHours() == datapoint.hour) {
                                //       if (!_.isUndefined(datapoint.minute) && startTime.getUTCMinutes() > datapoint.minute) {
                                //         return false;
                                //       }
                                //     }
                                //   }
                                // }
                            }
                        }
                    }
                }
                if (endTime.getUTCFullYear() < datapoint.year) {
                    return false;
                }
                if (endTime.getUTCFullYear() == datapoint.year) {
                    if (!_.isUndefined(datapoint.month)) {
                        if (endTime.getUTCMonth() < datapoint.month) {
                            return false;
                        }
                        if (endTime.getUTCMonth() == datapoint.month) {
                            if (!_.isUndefined(datapoint.day)) {
                                if (endTime.getUTCDate() < datapoint.day) {
                                    return false
                                }
                                // I'm leaving this here in case we decide to support filter down to hour/minutes 
                                // Since the slider does not filter to a smaller unit than days, I'm removing this unnecessary check
                                // The slider would need be smarter to change it's increment when the date range is small enough 
                                // in order to support showing hourly / minute data
                                // if (endTime.getUTCDate() == datapoint.day) {
                                //   if (!_.isUndefined(datapoint.hour)) {
                                //     if (endTime.getUTCHours() < datapoint.hour) {
                                //       return false;
                                //     }
                                //     if (endTime.getUTCHours() == datapoint.hour) {
                                //       if (!_.isUndefined(datapoint.minute) && endTime.getUTCMinutes() < datapoint.minute) {
                                //         return false;
                                //       }
                                //     }
                                //   }
                                // }
                            }
                        }
                    }
                }
                return true;
            };

            // time series parameters found in binned data
            var time_series_params = [];

            // depth parameters found in binned data
            var depth_profile_params = [];

            // biology parameters found in binned data
            var biology_params = [];

            // We need both keys and ids to support parameters that were not found, but added at runtime
            // The returned data from the API matches the key, but the app.collection.parameters are stored by id and needed later
            var timeSeriesMap = {};
            var depthKeyMap = {};
            var biologyMap = {};

            // properties in the binned data
            var properties = data[0].properties;

            // Break out depth and time data and store display name for graphs
            for(var key in properties) {
                var parr = app.collection.parameters.filter(function(x) {
                    return (x.id == key) || x.get("alternates").indexOf(key) >= 0;
                });

              if (parr[0] && parr[0]['id'] ) {
                if(parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'time') {
                  timeSeriesMap[parr[0].id] = key;
                  time_series_params.push(parr[0].id);
                } else if (parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'depth') {
                  depthKeyMap[parr[0].id] = key;
                  depth_profile_params.push(parr[0].id);
                } else if (parr[0].id.indexOf('ignored') == -1 && parr[0].get('detailView') === 'biology') {
                  biologyMap[parr[0].id] = key;
                  biology_params.push(parr[0].id);
                }
              }
            }

            if (time_series_params.length > 0) {
                $("#time-tab").toggle(true);

                time_series_params.sort();

                // For time series data, if applicable, use bin selected in UI or default value if none specified 
                // This is used when selecting a subset of the semi bin (e.g spring or summer data)
                bin = bin || $('#bin-select-time').val();
                if (bin === undefined) {
                    if ( self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            bin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }
                    } 
                }

                // Use a unique color for each parameter
                var time_series_params_colors = Object.keys(properties).length > 10 ? d3.scale.category20().domain(time_series_params) : d3.scale.category10().domain(time_series_params);

                // keep a list of data sources, pointing back to clowder dataset
                var sources = [];

                var alldata = [], summerEPdata = [];
                var foundEP = false;
                // iterate over each of the unique parameters
                for (var i = 0; i < time_series_params.length; i++) {
                    // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                    // if there are parameters in the URL, but the param we are on isn't one of them, skip it
                    if (!_.isEmpty(parameters) && (_.indexOf(parameters, time_series_params[i]) < 0)) {
                        continue;
                    }

                    var propKey = timeSeriesMap[time_series_params[i]];

                    var valuesAll = [], valuesEPsummer = [];

                    // Gather date ranges to set time series X range
                    var dateRange = [];

                    // Since server side binning separates by depth code, we use this to see if a bin has been processed
                    var currentBin = null;
                    _.each(properties[propKey], function(entry) {

                        // Skip entry if we have already processed it
                        if(currentBin != entry.label) {
                            currentBin = entry.label;

                            // Check if data is in date range and matches the selected bin (e.g. summer/spring), if applicable
                            if((_.isUndefined(bin) || (entry.label.indexOf(bin) != -1)) && filterDatesByRange(entry, startTime, endTime)) {
                                // Server side binning separates by depth code so we must locate all values for a bin
                                var obj = _.select(properties[propKey], function(item) {
                                    return item.label === entry.label;
                                });

                                var countAll = 0, countEPdata = 0;
                                var sumAll = 0, sumEPdata = 0;
                                var rawAll = [], rawEP = [];
                                var entryDate;

                                _.each(obj, function(x) {
                                    var depthCode = x.depth_code;

                                    // Added this because client side filtering mapped these depth codes to EP
                                    if(["SRF", "LEP", "TRM"].indexOf(depthCode) > -1) {
                                        depthCode = "EP";
                                    }

                                    if(x.label.indexOf("summer") != -1 && !_.isUndefined(depthCode) && (depthCode == "EP")) {
                                        foundEP = true;
                                    }
                                    
                                    if(x.label.indexOf("summer") == -1 || _.isUndefined(depthCode) || (depthCode == "EP") || (depthCode == "NA")) {
                                        countEPdata += x.count;
                                        sumEPdata += x.average * x.count;
                                        rawEP.push(x.average * x.count);
                                    }

                                    countAll += x.count;
                                    sumAll += x.average * x.count;
                                    rawAll.push(x.average);
                                    entryDate = x.date;
                                    _.each(x.sources, function(source) {
                                        sources.push(source);
                                    });
                                });

                                valuesAll.push({ x: entryDate, y: sumAll / countAll, label: currentBin, raw: _.flatten(rawAll) });
                                dateRange.push(entryDate);

                                if(countEPdata != 0) {
                                    valuesEPsummer.push({ x: entryDate, y: sumEPdata / countEPdata, label: currentBin, raw: _.flatten(rawEP) })
                                }
                            }
                        } 
                    });

                    var yrange = d3.extent(valuesAll.concat(valuesEPsummer), function(d) { return d.y; });
                    if (app.config.graphs.timeSeries.showUnits.showZero != false && yrange[0] > 0) {
                        yrange[0] = 0;
                    }
                    var xrange = [new Date(dateRange[0]), new Date(dateRange[dateRange.length-1])];

                    if(valuesAll.length > 0) {
                        alldata.push({
                            key: i.toString(),
                            values: [ {
                                color: time_series_params_colors(time_series_params[i]),
                                key: time_series_params[i],
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesAll,
                                sources: _.uniq(_.flatten(sources)),
                                unit: (app.collection.parameters.findWhere({id: time_series_params[i]})).get("unit")
                            }]
                        });
                    }
                     
                    if(!_.isEmpty(valuesEPsummer)) {
                        summerEPdata.push({
                            key: i.toString(),
                            values: [ {
                                color: time_series_params_colors(time_series_params[i]),
                                key: time_series_params[i],
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesEPsummer,
                                unit: (app.collection.parameters.findWhere({id: time_series_params[i]})).get("unit")
                            }]
                        })
                    }
                }
                if((bin == "spring") || !foundEP) {
                    self.aggregateTimeSeriesView.setData(alldata, undefined);
                } else {
                    self.aggregateTimeSeriesView.setData(alldata, summerEPdata);
                }
                self.controlsView.setParameters(time_series_params, _.isEmpty(parameters) ? time_series_params : parameters, time_series_params_colors);
            } else {
                $("#time-tab").toggle(false);
            }    

            // Setup depth profile tab
            var dates = JSON.parse(sessionStorage.getItem(station.id));
            self.controlsDepthView.setTimes(dates);

            if (depth_profile_params.length > 0) {
                $("#depth-tab").toggle(true);

                // If depth year is set in the UI, use that as the depth year filter
                var depthDateFilter = null;
                if(depthYear) {
                    depthDateFilter = new Date(depthYear, 0, 1);
                }

                depth_profile_params.sort();

                // Used to filter depth data outside of the selected year
                var filterByYear = function(x, depthDateYear) {
                    var date = new Date(x.date);
                    var datapoint = {year: date.getUTCFullYear() };

                    if(depthDateYear.getUTCFullYear() == datapoint.year) {
                        return true;
                    } else {
                        return false;
                    }
                };

                // For depth data, if applicable, use bin selected in UI or default value if none specified
                // This is used when selecting a subset of the semi bin (e.g spring or summer data)
                depthBin = depthBin || $('#bin-select-depth').val();
                if (depthBin === undefined) {
                    if (self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            depthBin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }  
                    }
                }

                // Use d3 to create colors for each of the unique parameters
                var depth_profile_params_colors = depth_profile_params.length > 10 ? d3.scale.category20().domain(depth_profile_params) : d3.scale.category10().domain(depth_profile_params);

                // List of available years for depth data
                var depthDates = [];

                // We're creating a display_data object for this date
                var display_data = [{
                    key: new Date(startTime),
                    values: []
                }];

                for(var i = 0; i < depth_profile_params.length; i++) {
                    // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                    // if there are parameters in the URL, but the param we are on isn't one of them, skip it
                    if (!_.isEmpty(parameters) && (_.indexOf(parameters, depth_profile_params[i]) < 0)) {
                      continue;
                    }

                    // keep a list of data sources, pointing back to clowder dataset
                    var sources = [];

                    var property = depth_profile_params[i];
                    var propKey = depthKeyMap[property];

                    var valuesAll = [];
                    _.each(properties[propKey], function(x) {
                        // Check if data matches the selected bin (e.g. summer/spring), if applicable
                        if(_.isUndefined(depthBin) || x.label.indexOf(depthBin) > -1) {

                            // If no depth year is set in the UI, then use the first date as the filter
                            if(depthDateFilter == null) {
                                depthDateFilter = new Date(x.date);  
                            } 

                            // Create a list of unique dates to build the depth date year selection widget
                            if(_.isEmpty(_.where(depthDates, x.date))) {
                                depthDates.push(x.date);
                            }

                            // Only add values for the selected depth year
                            if(filterByYear(x, depthDateFilter)) {
                                for(var j = 0; j < x.sources.length; j++) {
                                    if(_.isEmpty(_.where(sources, x.sources[j]))) {
                                        sources.push(x.sources[j]);
                                    }  
                                }
                                valuesAll.push({ 
                                    x: x.average,
                                    x0: 0,
                                    x1: 0,
                                    y: x.depth
                                });
                            }
                        }
                    });
                    
                    // Only add parameter if it has values for the depth year
                    if(valuesAll.length > 0) {
                        display_data[0].values.push({
                             color: depth_profile_params_colors(property),
                             key: property, 
                             sources: sources, 
                             values: valuesAll,
                             unit: (app.collection.parameters.findWhere({id: property})).get("unit")    
                        });
                    } 
                }

                depthDates.sort();

                self.controlsDepthView.setYears(depthDates);
                self.limnologyView.setData(display_data);
                self.controlsDepthView.setParameters(depth_profile_params, _.isEmpty(parameters) ? depth_profile_params : parameters, depth_profile_params_colors);
            } else {
                $("#depth-tab").toggle(false);
            }


            if (biology_params.length > 0) {
                $("#biology-tab").toggle(true);
                $('#biology-graphs').html("");
                var biology_params_colors = [];

                biologyBin = biologyBin || $('#bin-select-biology').val();

                if(biologyBin === undefined) {
                    if( self.binningTime && !(self.binningTime == 'none' || self.binningTime == 'smart')) {
                        if(!_.isEmpty(getBinningDesignators(self.binningTime).items)) {
                            biologyBin = getBinningDesignators(self.binningTime).items[0]['id'];
                        }
                    }
                }
                self.stackedBarViews = [];
                // Iterate over all of the unique parameters
                for(var i =0; i < biology_params.length; i++) {
                    if (!_.isEmpty(parameters) && _.indexOf(parameters, biology_params[i]) < 0) {
                        continue;
                    }
                    var stackedGraphData = [];
                    var propKey = biologyMap[biology_params[i]];
                    var biologyDates = [];

                    _.each(properties[propKey], function (entry) {
                        biologyDates.push(new Date(entry.date));
                        //Check if data is in date range and is part of the season selected on the bin
                        if((_.isUndefined(biologyBin) || (entry.label.indexOf(biologyBin) != -1)) && filterDatesByRange(entry, biologyStartTime, biologyEndTime)) {
                            for (var k = 0; k < entry.array.length; k++) {
                                var biology_row = JSON.parse(entry.array[k]);
                                biology_row["date"] = entry.date;
                                stackedGraphData.push(biology_row);
                            }
                        }

                    });

                    //If the start and end date are the ones in the 'times' we want to update them to be a 'smaller' range
                    // of dates just to cover the range of the biology dates.
                    var biologyMinDate = new Date(Math.min.apply(null, biologyDates));
                    var biologyMaxDate =  new Date(Math.max.apply(null, biologyDates));

                    //This should only be called the first time. That is why the comparison between the biology and the biology start times.
                    if(biologyStartTime.getTime() == startTime.getTime() && biologyMinDate.getTime() > biologyStartTime.getTime()) {
                        biologyStartTime = biologyMinDate;
                        if(biologyEndTime.getTime() == endTime.getTime() && biologyMaxDate.getTime() < biologyEndTime.getTime()) {
                            biologyEndTime = biologyMaxDate;
                        }
                        var dates = getDates(biologyStartTime, biologyEndTime);
                        self.controlsBiologyView.setTimes(dates);
                    } else if (biologyStartTime.getTime() >= biologyMinDate.getTime() || biologyEndTime.getTime() <= biologyMaxDate.getTime()) {
                        //This should be called when you refresh the details page.
                        var dates = getDates(biologyMinDate, biologyMaxDate);
                        self.controlsBiologyView.setTimes(dates);
                        self.controlsBiologyView.sliderTimes(biologyStartTime, biologyEndTime);
                    }

                    var parameterProps = app.collection.parameters.findWhere({id: propKey});
                    var newGraphId = "stacked-biology-graph" + propKey;
                    $("#"+newGraphId).remove();
                    $('#biology-graphs').append('<svg id="' + newGraphId + '" width="960" height="500"></svg>');
                    var biologyZ = d3.scale.category10();
                    var scaleNames = [];

                    if(parameterProps.get("scaleColors") != undefined && parameterProps.get("scaleNames") != undefined) {
                        scaleNames = parameterProps.get("scaleNames");
                        biologyZ = d3.scale.ordinal().domain(scaleNames).range(parameterProps.get("scaleColors"));

                    }
                    var areaView = new app.view.StackedBarView({
                        id: newGraphId,
                        data: stackedGraphData,
                        scale: d3.time.years,
                        title: parameterProps.get("title") + " (" + parameterProps.get("unit") + ")",
                        scaleNames: scaleNames,
                        unit: parameterProps.get("unit"),
                        z: biologyZ
                    });
                    self.stackedBarViews.push(areaView);

                    biology_params_colors.push("#000000");
                }
                var biology_colors = d3.scale.ordinal().range(biology_params_colors);
                self.controlsBiologyView.setParameters(biology_params, _.isEmpty(parameters) ? biology_params : parameters, biology_colors);
            } else {
              $("#biology-tab").toggle(false);
            }

            // render everything
            if (self.sensor) self.updateDownloadLink();
            self.render();

        });
    },

      render : function() {
        app.event.eventBus.trigger("detail:spinner:start");
        if (!this.sensor) {
          this.sensor = app.collection.sensors.findWhere({name: this.locationID});
        }
        this.source = app.collection.sources.findWhere({id: this.sensor.get('type').id});

        var header_text
        if (this.sensor) {
          if (this.sensor.get('online_status')){
            if(this.sensor.get('popupContent')){
              header_text = this.sensor.get('popupContent') + ' <font color="red">' +  this.sensor.get('online_status') + '</font>';                       
            } else {
              header_text = this.locationID;
            }
            $("#detail-view-header").html(header_text)
          } else {
            if(this.sensor.get('popupContent')){
              header_text = this.sensor.get('popupContent');
            } else {
              header_text = this.locationID;
            }
          }
          $("#detail-view-header").html(header_text)
        }

        $("#time-series.tab-pane").each((function(self) {
          return function() {
            self.aggregateTimeSeriesView.render();
          };
        })(this));
        $("#depth-profile.tab-pane").each((function(self) {
          return function() {
            self.limnologyView.render();
            self.timeSeriesView.render();
            self.stackedBarViews.forEach(function(view) {
                view.render();
            });
          };
        })(this));
        // using true here makes the map pan into position each time. With false, it stays in place
        this.miniMap.forEach(function(mapElement){
            mapElement.map.invalidateSize(false);
          });
        this.eventBus.trigger("renderBinSelect");
        this.setTabTitles();
        this.$el.append(this.sourceModals);
        if (this.source) {
          if (this.source.get('calculation') && this.source.get('calculation')['time-series']) {
            var timeTitle = $("#time-series-title");
            timeTitle.find('a').remove();
            timeTitle.find('i').remove();
            timeTitle.append('<a href="#' + this.source.id + '-time-series-modal" role="button" ' +
              'data-toggle="modal"><i class="icon-info-sign bw-info"></i></a>');
          }
          if (this.source.get('calculation') && this.source.get('calculation')['depth-profile']) {
            var depthTitle = $("#depth-profile-title");
            depthTitle.find('a').remove();
            depthTitle.find('i').remove();
            depthTitle.append('<a href="#' + this.source.id + '-depth-profile-modal" role="button" ' +
              'data-toggle="modal"><i class="icon-info-sign bw-info"></i></a>');
          }
          $("#detail-source-icon").html(this.infoIcon({model: this.source.toJSON()}));
        }
        app.event.eventBus.trigger("detail:spinner:stop");

      },

      showDistribution : function() {
        this.$("#showDistribution").button("toggle");
        this.aggregateTimeSeriesView.render();
        this.timeSeriesView.render();
      },

      goBack : function(event) {
        if (this.sensor && this.sensor.id) {
          this.sensorID = this.sensor.id;
        }
        sessionStorage.removeItem(this.sensorID);
        this.router.navigate("#explore", {
          trigger : true,
          replace : false
        });
      },

      setTabTitles: function() {
          var source = app.collection.sources.findWhere({id: this.sensor.attributes.type.id});
          if (source && source.get('tabs')) {
              if (source.get('tabs').timeseries) {
                  $('#times-tab').find('a').text(source.get('tabs').timeseries);
              }
              if (source.get('tabs').depthprofile) {
                  $('#depth-tab').find('a').text(source.get('tabs').depthprofile);
              }
              if (source.get('tabs').biology) {
                  $('#biology-tab').find('a').text(source.get('tabs').biology);
              }
          }
      },

      updateDownloadLink : function() {
        var format = "YYYY-MM-DD HH:mm:ss";
        var params = {};

        params['geocode'] = [
          this.sensor.get('geometry')['coordinates'][1],
          this.sensor.get('geometry')['coordinates'][0],
          this.sensor.get('geometry')['coordinates'][2]
        ].toString();

        if (this.startTime) params["since"] = moment(this.startTime).format(format);
        if (this.endTime) params["until"] = moment(this.endTime).format(format);

        if (this.parameters) params["attributes"] = this.parameters;

        // create the url
        var url = app.datapoints.url + '?' + $.param(params, true);

        // update the DOM
        var downloadAsJsonButton = $("#detail-download-json");
        downloadAsJsonButton.attr('href', url + "&format=json");

        var downloadAsCsvButton = $("#detail-download-csv");
        downloadAsCsvButton.attr('href', url + "&format=csv");

      }
});

// Determine the bin based on the temporal range of the start / end date
function getAggregationBin(startTime, endTime) {
    var time = endTime.getUTCFullYear() - startTime.getUTCFullYear();
    var timeBin = 'day';

    // This should probably be configurable by project to select the binning
    // that makes the most sense for the expected temporal range of the data
    if(time > 100) {
        timeBin = 'decade';
    } else if(time > 50) {
        timeBin = 'lustrum';
    } else if(time > 25) {
        timeBin = 'year';
    } else if(time > 10) {
        timeBin = 'season';
    } else if(time > 1) {
        timeBin = 'month';
    } else {
        // Convert to days
        time = (endTime - startTime) / 86400000;
        if(time > 14) {
            timeBin = 'day';
        } else if(time > 3) {
            timeBin = 'hour';
        } else {
            timeBin = 'minute';
        }
    }
    return timeBin;
}

// Generate dates between start/end times with 1 day time steps
function getDates(startTime, endTime) {
    var dates = [];
    var currentDate = startTime;
    while(currentDate <= endTime) {
        dates.push(new Date(currentDate).toUTCString());
        currentDate = new Date(currentDate);
        currentDate.setDate(currentDate.getDate() + 1);
    }
    dates.push(new Date(endTime).toUTCString());

    return dates;
}

app.view.SelectItemView = Backbone.View.extend({
    tagName: "option",
    attributes: function() {
        return {
            value: this.model.id || ''
        }
    },

    initialize : function(options) {
        this.model = options.model;
        this.selected = options.selected;
        var content = this.model.get('name') || this.model.get('title') || '';
        this.template = _.template(content.replace(/<(?:.|\n)*?>/gm, ''));
        this.render();
    },

    render : function() {
        this.$el.empty();
        this.$el.append(this.template());
        if (this.selected) this.$el.attr('selected', 'selected');
        return this;
    }
});

app.view.SelectCollectionView = Backbone.View.extend({
    tagName : "select",
    className : "dropdown",

    events: {
        'change': 'changeSelect'
    },

    initialize: function(options) {
        this.trigger = options.trigger;
        if (options.placeholder) this.placeholder = new Backbone.Model({name: options.placeholder});

        _.bindAll(this, "render", "changeSelect", "append", "setSelected");
        this.collection.on('all', this.render);

        this.render();
    },

    render : function() {
        var self = this;
        this.$el.empty();

        // if there are models in this collection, we will populate them as select options
        if (this.collection.length) {
            // self.placeholder allows you to insert a default field like "None" or "Select a Value"
            if (self.placeholder) {
              self.$el.append(new app.view.SelectItemView({ model: self.placeholder, selected: true }).render().el);
            }

            // add each of the options to the select element and mark the selected item
            this.collection.each(function(item) {
                var selected = false;
                if (self.selectedItem && (item.id == self.selectedItem)) {
                  selected = true;
                }
                self.$el.append(new app.view.SelectItemView({ model: item, selected: selected }).render().el);
            });
        }
        return this;
    },

    append: function(item) {
        this.$el.append(new app.view.SelectItemView({model: item}).render().el);
    },

    changeSelect: function(event) {
        var self = this;
        self.setSelected(event.target.value);
        event.cancleBubble = true;
        if (this.trigger) {
            var parameterElement = $('.parameter-trend-active');
            if (parameterElement) {
              var parameterID = parameterElement.data('id');
              if (parameterID) {
                app.event.eventBus.trigger(this.trigger + ":" + parameterID, event.target.value);
              }
            }

        }
    },

    setSelected: function(value) {
        this.selectedItem = value;
    }


});

app.view.ExploreSourcesView = Backbone.View.extend({

    initialize : function(options) {
       var self = this;
       this.config = options.config;
       _.bindAll(this, "render");

       this.template = _.template($(this.config.templateID).html());
       
       this.modals = _.template($(this.config.modalsID).html())();

       this.listenTo(app.collection.sources, 'change', this.render);
       app.event.eventBus.on("sensors:refresh", this.render);

      // This event is fired when someone opens/closes an accordion in the
      // ExploreView.
      app.event.eventBus.on("change:sources", function(sources) {
        if ($('#sources').find('.data-nav-header-active').length) {
          if (!sources) {
            sources = [];
            $('input[name="source"]:checked').each(function() {
              sources.push($(this).attr("value"));
            });
          }
          var sensorModels = app.collection.sensors.filter(function(sensor) {
            return _.contains(sources, sensor.get('type').id);
          });
          var sensors = _.map(sensorModels, function(sensor) {
            return sensor.toJSON();
          });
          app.view.exploreMapView.setGeoJsonFeature(sensors, self.config.map);
          app.view.exploreMapView.render(self.config);
        }
      });

   },

   render : function() {
       var self = this;


       if (app.collection.sources.getSourcesWithSensors().length > 0) {
           var checkedSources = [];

           if ($('input[name="source"]').length == 0) {
               checkedSources = app.collection.sources.getSourcesWithSensors().map(function(source) { return source.get('id'); });
           } else {
               $('input[name="source"]:checked').each(function() {
                   checkedSources.push($(this).attr("value"));
               });
           }

           this.$el.empty();

           app.collection.sources.getAreas().forEach(function(source) {
               // if sources.json hasn't been filled out, the default will be an empty template that we shouldn't render
               if (source.id) {
                   if (_.contains(checkedSources, source.id)) source.checked = "checked";
                   self.$el.append(self.template({ model: source }));
               }
           });
           app.event.eventBus.trigger("change:sources", checkedSources);

       } else {
           // display placeholder text if no sources are found
           this.$el.html(app.config.sources.noneFound);
       }


       this.$el.append(this.modals);

       var $explorePills = this.$(this.config.pills.className);
//       $explorePills.tooltip(this.config.pills.tooltipOptions); // enable bootstrap tooltips

       // fires events to highlight or unhighlight map pins
       $explorePills.hover(function(self) {
           return function(event) {
               app.event.eventBus.trigger("marker:mouseover", event.currentTarget.value.replace(/\W/g, "-"));
           };
       }(this), function(self) {
           return function(event) {
               app.event.eventBus.trigger("marker:mouseout", event.currentTarget.value.replace(/\W/g, "-"));
           };
       }(this));

       $explorePills.on("click", (function(self) {
           return function(event) {
               app.event.eventBus.trigger("marker:click", event.currentTarget.value.replace(/\W/g, "-"));
           };
       })(this));

       /* Keep the checkboxes in sync with the collapsed accordion
        * checked - accordion open
        * unchecked - accordion closed
        * not happy with this solution.
        * In the future the EPA wants the checkboxes to be displayed,
        * the checkboxes should determine which pins are on the map.
        * TODO : see GEO-258
        */
       var $collapse = this.$(this.config.collapse.className);

       $collapse.on("hide", (function(self) {
           return function(event) {
               if(event.target !== this) return;
               $(this).parent().find(self.config.collapse.inputSelector).prop("checked", false);

               var sources = [];
               $(self.config.collapse.inputSelector + ':checked').each(function() {
                   sources.push($(this).attr("value"));
               });
               app.event.eventBus.trigger("change:sources", sources);
           };
       })(this));

       $collapse.on("show", (function(self) {
           return function(event) {
               if(event.target !== this) return;
               $(this).parent().find(self.config.collapse.inputSelector).prop("checked", true);

               var sources = [];
               $(self.config.collapse.inputSelector + ':checked').each(function() {
                   sources.push($(this).attr("value"));
               });
               app.event.eventBus.trigger("change:sources", sources);
           };
       })(this));

       self.$('.showHideExplore').on('click', this.toggleButton);

       return this;

   },

   toggleButton: function() {
        if(this.innerHTML.indexOf("more") >=0) {
            this.innerHTML = "Show less";
        } else {
            this.innerHTML = "Show more (" + this.dataset.sensors + " more sensors)";
        }

   }
});
app.view.ExploreByCustomView = Backbone.View.extend({
    initialize: function (options) {
        var self = this;
        this.config = options.config;
        this.elementID = options.elementID;
        _.bindAll(this, "render");

        this.template = _.template($(this.config.templateID).html());

        if (_.isArray(this.config.modalsID)) {
            this.modals = _.map(this.config.modalsID, function (template) {
                return _.template($(template).html())();
            });
        } else if (this.config.modalsID) {
            // setting showModal to true will enable the info icon in the explore_custom template
            this.showModal = true;
            this.modals = _.template($(this.config.modalsID).html())();
        }


        this.firstGroup = this.config.groupingOrder[0];
        this.lastGroup = this.config.groupingOrder[this.config.groupingOrder.length - 1];
        this.primaryCollection = this.config.primaryCollection || this.firstGroup;
        this.config.collectionGroupingFunctionArguments = this.config.collectionGroupingFunctionArguments || '';

        //this.listenTo(app.collection[this.config.listenTo], 'change', this.render);
        app.event.eventBus.on('sensors:refresh', this.render);

        app.event.eventBus.on("change:" + self.elementID, function (models) {
            if (models && models.length) {
                if ($('#' + self.elementID).find('.data-nav-header-active').length) {
                    var filteredModels = app.collection[self.lastGroup].filter(function (model) {
                        var huc;
                        switch (self.firstGroup) {
                            case "categories":
                                var cats = new Backbone.Collection;
                                model.get('parameters').each(function (p) {
                                    _.each(p.get('categories'), function (cat) {
                                        cats.add(cat)
                                    })
                                });
                                return _.intersection(cats.pluck('id'), models).length;
                                break;
                            case "parameterCategories":
                                return _.intersection(model.get('parameters').pluck('id'), models).length;
                                break;
                            case "parameters":
                                return _.intersection(model.get(self.firstGroup).pluck('id'), models).length;
                                break;
                            case "sources":
                                return _.contains(models, model.get('type')['id']);
                                break;
                            case "regions":
                            case "areas":
                                return _.contains(models, model.get('region'));
                                break;
                            case "river-reaches":
                                return _.contains(models, model.get('river-reaches'));
                                break;
                            case "huc2":
                                huc = model.get('huc');
                                if (huc && huc['huc2'] && _.contains(models, huc['huc2']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc4":
                                huc = model.get('huc');
                                if (huc && huc['huc4'] && _.contains(models, huc['huc4']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc6":
                                huc = model.get('huc');
                                if (huc && huc['huc6'] && _.contains(models, huc['huc6']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            case "huc8":
                                huc = model.get('huc');
                                if (huc && huc['huc8'] && _.contains(models, huc['huc8']['code'])) {
                                    return model;
                                } else {
                                    return false;
                                }
                            default:
                                return false;
                        }

                    });
                    if (filteredModels) {
                        var filteredModelsToJSON = _.map(filteredModels, function (model) {
                            return model.toJSON();
                        });
                        app.view.exploreMapView.setGeoJsonFeature(filteredModelsToJSON, self.config.map);
                        app.view.exploreMapView.render(self.config);
                    }

                }
            }
        });

    },

    render: function () {
        var self = this;

        if (app.collection[self.primaryCollection] && app.collection[self.primaryCollection].length > 0) {
            var checkedCollectionItems = new Backbone.Collection;

            if ($('input[name="collection"]').length == 0) {
                app.collection[self.primaryCollection].each(function (model) {
                    checkedCollectionItems.add({id: model.get('id')});
                });
            } else {
                $('input[name="collection"]:checked').each(function () {
                    checkedCollectionItems.add({id: $(this).attr("value")});
                });
            }

            this.$el.empty();

            // this approach doesn't really work because you end up with colors for the pills that aren't mapped to the icons on the map
            // and since the icons on the map will belong to several parameters, you won't be able to really make this work.

            //var groupedResponse = app.collection[self.primaryCollection][self.config.collectionGroupingFunction]();
            //for (var i = 0; i < groupedResponse.length; i++) {
            //   if (groupedResponse.models[i].id) {
            //     if (_.contains(checkedCollectionItems, groupedResponse.models[i].id)) groupedResponse.models[i].checked = "checked";
            //     self.$el.append(self.template({ model: groupedResponse.models[i].toJSON(), color: app.config.colors[i] }));
            //   }
            //}

            app.collection[self.primaryCollection][self.config.collectionGroupingFunction](self.config.collectionGroupingFunctionArguments).forEach(function (model) {
                // if models.json hasn't been filled out, the default will be an empty template that we shouldn't render
                if (model.id) {
                    if (checkedCollectionItems.contains(model.id)) model.set('checked', "checked");
                    self.$el.append(self.template({
                        model: model.toJSON(),
                        grouping: self.config.groupingOrder,
                        showModal: (self.showModal ? self.showModal : false)
                    }));
                }
            });
            app.event.eventBus.trigger("change:" + self.elementID, checkedCollectionItems.pluck('id'));

        } else {
            // display placeholder text if no sources are found
            if (app.config[self.primaryCollection]) {
                this.$el.html(app.config[self.primaryCollection].noneFound);
            } else {
                this.$el.html("No models found.");
            }

        }

        if (this.modals) {
            this.$el.append(this.modals);
        }


        var $explorePills = this.$(".explore-pill");
//       $explorePills.tooltip(this.config.pills.tooltipOptions); // enable bootstrap tooltips

        // fires events to highlight or unhighlight map pins
        $explorePills.hover(function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:mouseover", event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this), function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:mouseout", event.currentTarget.value.replace(/\W/g, "-"));
            };
        }(this));

        $explorePills.on("click", (function (self) {
            return function (event) {
                app.event.eventBus.trigger("marker:click", event.currentTarget.value.replace(/\W/g, "-"));
            };
        })(this));

        /* Keep the checkboxes in sync with the collapsed accordion
         * checked - accordion open
         * unchecked - accordion closed
         * not happy with this solution.
         * In the future the EPA wants the checkboxes to be displayed,
         * the checkboxes should determine which pins are on the map.
         * TODO : see GEO-258
         */
        var $collapse = this.$('.collapse');

        $collapse.on("hide", (function (self) {
            return function (event) {
                if (event.target !== this) return;
                $(this).parent().find('input[name="collection"]').prop("checked", false);

                var collectionItems = new Backbone.Collection;
                self.$('input[name="collection"]:checked').each(function () {
                    collectionItems.add({id: $(this).attr("value")});
                });
                app.event.eventBus.trigger("change:" + self.elementID, collectionItems.pluck('id'));
            };
        })(this));

        $collapse.on("show", (function (self) {
            return function (event) {
                if (event.target !== this) return;
                $(this).parent().find('input[name="collection"]').prop("checked", true);

                var collectionItems = new Backbone.Collection;
                self.$('input[name="collection"]:checked').each(function () {
                    collectionItems.add({id: $(this).attr("value")});
                });
                app.event.eventBus.trigger("change:" + self.elementID, collectionItems.pluck('id'));
            };
        })(this));

        self.$('.showHideExplore').on('click', this.toggleButton);
        return this;
    },

    toggleButton: function () {
        if (this.innerHTML.indexOf("more") >= 0) {
            this.innerHTML = "Show less";
        } else {
            this.innerHTML = "Show more (" + this.dataset.sensors + " more sensors)";
        }

    }
});
app.view.ExploreAccordionItemView = Backbone.View.extend({
    className: 'accordion-group',

    initialize: function(options) {
        var self = this;
        _.bindAll(this, "render", "renderAccordionContents");
        // these items are used to construct the template
        this.template = _.template($(options.template).html());

        // these items are passed into the template
        this.options = options;
        this.id = options.attributes.id;
        this.title = options.title;
        this.defaultOpen = options.defaultOpen;
        this.showHeader = (_.isBoolean(options.showHeader)) ? options.showHeader : true;
        this.subviewID = (options.contents && options.contents.containerID) ? options.contents.containerID : "";

        this.render();
        if (this.showHeader) {
          this.$el.on('show', function(event) {
            if (event.target.id == 'accordion-nav-collapse-' + self.id){
              $(this).find('.accordion-heading.data-nav-header').addClass('data-nav-header-active');
            }
          });
          this.$el.on('hide', function(event) {
            if (event.target.id == 'accordion-nav-collapse-' + self.id) {
              $(this).find('.accordion-heading.data-nav-header').removeClass('data-nav-header-active');
            }
          });
        }
    },

    render: function() {
        var self = this;
        // all model properties are directly available to this.template (defined above)
        this.$el.empty();
        this.$el.html( this.template({
                id: this.id,
                title: this.title,
                defaultOpen: this.defaultOpen,
                subviewID: this.subviewID,
                showHeader: this.showHeader
        }));

        this.renderAccordionContents();
        this.$('#accordion-nav-collapse-' + this.id).on('shown', function() {
            app.event.eventBus.trigger("change:" + self.id);
        });

        if (this.defaultOpen) {
          app.event.eventBus.trigger('change:' + this.id);
        }

        return this;
    },

    renderAccordionContents: function() {
        var config = this.options.contents;
        var elementID = this.options.attributes.id;

        if (config && config.viewType && config.viewConfig && app.view[config.viewType]) {
            var view = new app.view[config.viewType]({config: config.viewConfig, elementID: elementID});
            this.$("#" + config.containerID).append(view.render().el);
        }

        return this;
    }
});
app.view.ExploreTrendsCollectionView = Backbone.View.extend({
//    tagName : "div",
//    className : "accordian",

    initialize: function(options) {
        this.config = options.config;
        this.selected = this.config.selected;
        this.collection = [];
        this.render();
    },

    render : function() {
        var self = this;

        if (app.collection.parameters.length > 0) {

            this.$el.empty();
            this.collection.forEach(function(subView) {
              subView.remove();
              subView.unbind();
              if (subView.close) subView.close();
            });

            //Removed the depth parameters until Medici can handle depth parameter calculations
            app.collection.parameters.where({exploreView: "visible", detailView: "time"}).forEach(function(parameter) {
              if (self.config.filterFunction && typeof self.config.filterFunction == 'function') {
                if (!self.config.filterFunction(parameter)) {
                  return;
                }
              }
              // don't show parameters in the list if they don't have any sensors
                if (parameter.id && parameter.get('sensors') && parameter.get('sensors').length) {
                    var selected = false;
                    if (self.selected && (parameter.get('title') == self.selected)) {
                      selected = true;
                    }
                    var itemView = new app.view.ExploreTrendsItemView({ model: parameter, selected: selected });
                    self.collection.push(itemView);
                    self.$el.append(itemView.render().el);
                }
            });
        } else {
            // display placeholder text if no sources are found
            this.$el.html(app.config.parameters.noneFound);
        }
        return this;
    }


});

app.view.ExploreTrendsItemView = Backbone.View.extend({

    attributes: function() {
        return {
          class: 'btn parameter-trend',
          'data-id': this.model.id,
          'data-title': this.model.get('title').replace(/<(.*?)>/g, '')
        }
    },
    events: {
        'click': 'clickItem'
    },

    initialize : function(options) {
        var self = this;
        _.bindAll(this, "redrawMapArrows");
        this.model = options.model;
        this.selected = options.selected;
        this.geostreamService = app.model.geoStreamService;
        this.template = _.template(this.model.get('title').replace(/<(.*?)>/g, ''));

        app.event.eventBus.on("trends:redrawMapArrows:" + self.model.get('id'), function(season) {
            var activeParameterModel = self.getActiveParameterModel();
            if (activeParameterModel.get('id') == self.model.get('id')) {
              self.redrawMapArrows(null, season);
            }

        });

        app.event.eventBus.on('trends:changeLegendTitle', function(parameter) {
            if (parameter) {
              self.getActiveParameterModel(parameter, function(parameterModel) {
                  self.changeLegendTitle(parameterModel.get('title'));
              });
            }
        });

        this.render();
    },

    close: function() {
        app.event.eventBus.off('trends:redrawMapArrows');
    },

    render : function() {
        var sensors = this.model.get('sensors');
        if (sensors.length > 0) {
            this.$el.empty();
            this.$el.html(this.template);
        }
//        if (this.selected) {
//            this.$el.trigger('click');
//        }
        return this;
    },

    clickItem: function(event) {
        event.cancelBubble = true;
        var self = this;
        if (event) {
          $('.parameter-trend').removeClass("parameter-trend-active");
          $(event.target).addClass('parameter-trend-active');
          var parameterID = $(event.target).data('id');
          if (self.model.get('id') != parameterID) return;

          var parameterModel = app.collection.parameters.findWhere({id: parameterID});

          self.redrawMapArrows(parameterID);

          app.event.eventBus.trigger("change:trends", self.model.get('sensors'), parameterModel.get('title'));
        }
    },

    redrawMapArrows : function(parameter, season) {
        app.event.eventBus.trigger("explore:map:spinner:start");

        var self = this;

        parameter = self.getActiveParameterModel(parameter);

        if (self.model.get('id') != parameter.get('id')) {
            return;
        }

        self.changeLegendTitle(parameter.get('title'));

        season = self.getActiveSeason(season);

        var attr_str = $.param({attributes: this.model.get('id') }, true);
        this.geostreamService.getTrend(attr_str, season, function(sensors, parameterTitle){
            app.event.eventBus.trigger("change:trends", sensors, parameterTitle);
            app.event.eventBus.trigger("explore:map:spinner:stop");
        });



    },

    getActiveParameterModel: function(parameterID, callback) {
      var parameterElement = $('.parameter-trend-active');
      if (parameterElement) {
        parameterID = parameterID || ((parameterElement.data('id')) ? parameterElement.data('id') : null);
        if (parameterID) {
          var parameter = app.collection.parameters.findWhere({id: parameterID});
        }
      }
      if (typeof callback == 'function') {
          callback(parameter);
      } else {
          return parameter;
      }

    },

    getActiveSeason: function(season) {
        // TODO this should not be using app.config.binning.time since the dropdown isn't being used to 'bin'
        // like the TODO in the ExploreMapView, this should probably be moved to app.config.views.trendMap or similar
        season = season || $('#trends-map-season-control').find('select').val() ||
            (getBinningDesignators(app.config.binning.time).items && getBinningDesignators(app.config.binning.time).items[0]) ? getBinningDesignators(app.config.binning.time).items[0].id : 'spring';
        return season;
    },

    changeLegendTitle: function(title) {
      if (title) {
        $("#trends-map-legend-title-info").html(title);
      }
    }


});

app.view.LayerItemView = Backbone.View.extend({
    tagName: 'div',
    className: "toggleLayerView",
    attributes: function() {
        return {
            id : "layer-" + this.model.get("id")
        }
    },

    events: {
        //"click input": 'check',
        "click .layer-radio": 'check',
        "change .layer-opacity" : "updateOpacity"
    },


    initialize: function(options) {
        // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
        this.template = _.template($('#explore-layers-template').html());

        this.layers = options.layers;
//    	layers: 'sf:USRB_30m_DEM',
//        format: 'image/png',
//        transparent: true,
//        opacity: 0.75
//
        this.shapeOptions = {
            stroke: true,
            color: '#f06eaa',
            weight: 4,
            opacity: 0.5,
            fill: true,
            fillColor: null, //same as color by default
            fillOpacity: 0.2,
            clickable: true
        };

        var self = this;
        return this;
    },

    render: function() {
        // all model properties are directly available to this.template (defined above)
        var self = this;
        this.$el.empty();
        this.$el.html(
            this.template({
                checked: (this.model.get('properties').checked) ? "checked" : '',
                id: this.model.get('properties').id,
                title: this.model.get('properties').title,
                opacity: this.model.get('properties').opacity || 0.5,
                legend: this.model.get('properties').legend
            })
        );
        return this;
    },

    // this function handles the action of actually clicking the div for this Area - like "checking" an input box
    check: function(event) {
        var self = this;

        var checkedState = this.model.get("checked");
        this.model.set("checked", !(checkedState));
        app.event.eventBus.trigger("change:layer", this.model);
    },

    updateOpacity: function(event) {
        this.model.get("properties").opacity = $(event.currentTarget).val();
        app.event.eventBus.trigger("change:layer:opacity", this.model);
    },

    selectLayer : function(event) {
        //var self = this;
        //var geometry = self.model.get("geometry");
        //var points = geometry.coordinates[0].map(function(x) { return L.latLng(x[1], x[0]); });
        //app.event.eventBus.trigger("map:polygon:select", {'points': points});
        //var polygon = L.polygon(points, self.shapeOptions);
        //self.model.set("polygon", polygon);
        //polygon.addTo(app.view.advancedSearchView.drawnItems);
    }
});

app.view.LayersCollectionView = Backbone.View.extend({
    initialize: function() {
        this.collection = app.collection.layers;
        this.listenTo(this.collection, 'add', this.append, this);
        this.listenTo(this.collection, 'remove', this.render, this);
    },


    render: function() {
        var self = this;
        this.$el.empty();


        // append each of the areas to the view (like adding "li" elements to a "ul")...
        _.each(this.collection.models, function(model) {
            self.append(model);
        });


        return this;
    },


    append: function(model) {
        this.$el.append(new app.view.LayerItemView({model: model}).render().el);
    }
});
// this "view" controls how we list one specific Area on the Search page (like "Lake Michigan")
// each Area is a model, and this view ties that model to the UI
// AreaListView (below) does the job of inserting all of these views
// think of AreaView as an "li" and AreaListView is the "ul"
app.view.AreaItemView = Backbone.View.extend({
    tagName: 'div',
    className: "areaView",
    attributes: function() {
        return {
            id : "area-" + this.model.get("id")
        }
    },

    events: {
        "click": 'check'
    },

    initialize: function() {

        // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
        this.template = _.template($('#search-areas-template').html());

        this.shapeOptions = app.config.views.search.shapeOptions || {
            stroke: true,
            color: (app.config.areas && app.config.areas.shapeOptions && app.config.areas.shapeOptions.color) ? app.config.areas.shapeOptions.color : '#7eb4fc',
            weight: 4,
            opacity: 0.5,
            fill: true,
            fillColor: null, //same as color by default
            fillOpacity: 0.2,
            clickable: true
        };

        this.model.on("change", this.render, this);

        return this;
    },

    render: function() {
        // all model properties are directly available to this.template (defined above)
        var self = this;
        // if the area does not contain a geometry property, it won't be able to draw a region, so don't show it as an option to check
        if (this.model.get('geometry')) {
            this.$el.empty();
            this.$el.html( this.template( { area: this.model.toJSON() } ) );
        }


        return this;
    },

    // this function handles the action of actually clicking the div for this Area - like "checking" an input box
    check: function(event) {

        // since we are using a radio input, "uncheck" all other areas
        app.collection.areas.clearSelection();
        this.model.set({
            checked: true,
            selected: true
        });
        this.selectLake(event);

        // always update the buttons when something gets checked
        app.view.searchView.updateDownloadLink();
    },

    selectLake : function(event) {
        var self = this;
        var geometry = self.model.get("geometry");
        if (!geometry) {
            return;
        }
        var points = geometry.coordinates[0].map(function(x) { return L.latLng(x[1], x[0]); });
        app.event.eventBus.trigger("map:polygon:select", {'points': points});
        var polygon = L.polygon(points, self.shapeOptions);
        self.model.set("polygon", polygon);
        polygon.addTo(app.view.searchView.drawnItems);

        // zoom into area if zoomInOnClick is set to true
        if (app.config.areas.zoomInOnClick) {
          app.view.searchView.map.fitBounds(points, {padding: [50, 50]});
        }

    }
});

app.view.AreasCollectionView = Backbone.View.extend({

    initialize: function(options) {
        if (this.collection) {
            this.listenTo(this.collection, 'add', this.append, this);
            this.listenTo(this.collection, 'remove', this.render, this);
        }
        this.collapsibleGroupTemplate = _.template($('#search-area-group-template').html());
    },

    render: function() {
        var self = this;
        // if there are areas, create a list of them.
        // if there is only one area, make sure it isn't an empty one from the default app.config.areas.items
        if (this.collection && this.collection.length > 0) {
            this.$el.empty();

            // append each of the areas to the view (like adding "li" elements to a "ul")...
            this.collection.each(function(model) {

                // if this view was created from a app.config.areas.groups, it will contain items
                // and should nest them in a collapsible element
                if (model.get('items') && model.get('items').length > 0) {
                    self.$el.append('<div class="accordion searchAreaGroup" id="searchAreaGroup-' + model.get('title').replace(/ /g, '-') + '">');

                    // create an array of the item views so that we can attach them to the DOM
                    // if you try to attach them inside of the template function, their events won't fire
                    var itemElements = model.get('items').map(function(item) {
                        return new app.view.AreaItemView({model: item});
                    });

                    self.$('#searchAreaGroup-' + model.get('title').replace(/\W/g, '-')).append(self.collapsibleGroupTemplate({
                        title: model.get('title'),
                        parent: "searchAreaGroup-" + model.get('title').replace(/\W/g, '-')
                    }));

                    // now attach items to the dom, preserving events
                    _.each(itemElements, function(item) {
                        self.$('#accordion-inner-' + model.get('title').toLowerCase().replace(/ /g, '-')).append(item.render().el);
                    });

                    self.$el.append('</div>');
                } else {
                    self.append(model);
                }
            });

        } else {
            // if this is seen, you need to define a FeatureCollection in /assets/layers/layers.json
            this.$el.html("<p>" + app.config.areas.noneFound + "</p>");
        }

        return this;
    },

    append: function(model) {
        this.$el.append(new app.view.AreaItemView({model: model}).render().el);
        return this;
    }
});

// this "view" controls how we list one specific Parameter on the Search page (like "Chlorophyl a")
// each Parameter is a model, and this view ties that model to the UI
// ParameterListView (below) does the job of inserting all of these views
app.view.ParameterItemView = Backbone.View.extend({
    events: {
        "click :checkbox": 'check'
    },

    render: function() {
        // all model properties are directly available to this.template (defined above)
        this.$el.empty();
        this.$el.html( this.template( {
            model: this.model.toJSON(),
            view: this.view
        }) );

        return this;
    },

    initialize: function(options) {
        // the whole model gets passed to this template, so we can directly use any values we need
        this.template = _.template($('#search-parameters-template').html());

        this.view = options.view;

        _.bindAll(this, "check");
        this.model.on("change", this.render, this);

        return this;
    },

    check: function() {
        app.event.eventBus.trigger("check:parameter", {view: this.view, model: this.model});
    }

});
app.view.ParametersCollectionView = Backbone.View.extend({

    initialize: function(options) {
        this.config = options;
        this.view = options.view;
        this.listenTo(this.collection, 'add', this.append, this);
        this.listenTo(this.collection, 'remove', this.render, this);

    },

    append: function(model) {
        if (model.id) {
            this.tagName = (this.config.itemOptions && this.config.itemOptions.tagName) ? this.config.itemOptions.tagName : "div";
            this.className = (this.config.itemOptions && this.config.itemOptions.className) ? this.config.itemOptions.className : "parameters";
            this.$el.append(new app.view.ParameterItemView({
                model: model,
                view: this.view,
                tagName: this.tagName,
                className: this.className
            }).render().el);
        }
    },

    render: function() {
        var self = this;
        // if there are parameters, create a list of them.
        // if there is only one parameter, make sure it isn't an empty one from the default parameters.json
        if (this.collection.length > 1 || (this.collection.length == 1 && this.collection.models[0].id)) {
            this.$el.empty();

            // append each of the parameter Views
            _.each(this.collection.models, function(model) {
                if (model.get(self.view) != "ignore") {
                    self.append(model);
                }
            });
        } else {
            // if you are seeing this error, you need to specify the parameters you want to shown in /public/data/parameters.json
            this.$el.html(app.config.parameters.noneFound)
        }
        return this;
    }

});

// this "view" controls how we list one specific Data Source on the Search page (like "WATER QUALITY (epa)")
// each Data Source is a model, and this view ties that model to the UI
// SourcesView (below) does the job of inserting all of these views
// think of SourceView as an "li" and SourcesView is the "ul"
app.view.SourceItemView = Backbone.View.extend({
    tagName: 'tr',
    className: "sourceView",

    events: {
        // TODO: Need to create a progress bar icon on the map.
        "click [type='checkbox']": 'check'
    },

    initialize: function() {
        var self = this;
        // the whole model gets passed to this template as "model", so we can directly use any values we need like "model.id"
        this.template = _.template($('#search-sources-template').html());
        this.eventBus = app.event.eventBus;

        // TODO: add a loading gif on the map
        this.eventBus.on("map:loading:start", function() {
            $('#loading').removeClass("hidden");
        });

        // TODO: add a loading gif on the map
        this.eventBus.on("map:loading:stop", function() {
            $('#loading').addClass("hidden");
        });

        this.model.on("change", this.render, this);

        return this;
    },

    render: function() {
        // all model properties are directly available to this.template (defined above)
        var self = this;

        this.$el.empty();
        this.$el.html( this.template( { model: this.model.toJSON() } ) );

        return this;
    },

    // updates the colored badge with the number of sensors selected on the map
    renderSelectedCount: function() {
        // in the template above, we're assigning the colored badge an "id": sensors-epa-selected
        var id = "#sensors-" + this.model.get('id') + "-selected";
        // the selected() function returns the number of sensors whose selected attribute is true
        $(id).html(this.model.getSelectedSensors().length);
    },

    // this function handles the action of actually clicking the div for this Data Source - like "checking" an input box
    check: function() {
        var self = this;

        self.eventBus.trigger("map:loading:start");

        // when you "check" the Data Source, it will highlight only the sensors that are "selectable".
        // if the user hasn't drawn a region on the map, everything is selectable. But if the user has drawn a region, 
        // only those sensors will be selectable
        var allSelectableSensors = app.collection.sensors.getSelectableSensors();
        var sourceSelectableSensors = self.model.getSelectableSensors();
        var areasChecked = app.collection.areas.getChecked();

        // we have to do this next step because if you look at the sensor.js file, each sensor's isChecked function
        // listens to areaChecked, sourceChecked or parameterChecked to determine if it should show its color marker
        // I'd rather not do this, because after you uncheck a source, the sensor's areaChecked doesn't go back to true
        // and the sensor won't draw its color marker again, even though the area has selected it, but I don't see any other way around this yet.
        if (areasChecked) {
            _.each(allSelectableSensors, function(sensor) {
                sensor.set("areaChecked", false);
            });
        }

        // the model is currently checked...
        if (this.model.get("checked")) {
            // "uncheck" it...
            this.model.set("checked", false);

            // ...then deselect each of its sensors...
            _.each(sourceSelectableSensors, function(sensor) {
                sensor.set("sourceChecked", false);
            });

        } else { // this model was not "checked"
            // "check" it...
            this.model.set("checked", true);

            // ...select each of its "selectable" sensors
            _.each(sourceSelectableSensors, function(sensor) {
                sensor.set("sourceChecked", true);
            });

        }
        // always update the buttons when something gets checked
        app.view.searchView.updateDownloadLink();

        self.eventBus.trigger("map:loading:stop");


    }
});

app.view.SourcesCollectionView = Backbone.View.extend({
    el: "table#searchSourcesView",
    className: 'lite-1',

    initialize: function() {
        this.listenTo(this.collection, 'add', this.append, this);
        this.listenTo(this.collection, 'remove', this.render, this);
    },

    render: function() {
        var self = this;
        // if there are sources, create a list of them.
        // if there is only one source, make sure it isn't an empty one from the default sources.json
        if (this.collection.length > 1 || (this.collection.length == 1 && this.collection.models[0].id)) {
            this.$el.empty();

            // append each of the sources to the view (like adding "li" elements to a "ul")...
            _.each(this.collection.models, function(model) {
                self.append(model);
            });
        } else {
            this.$el.html(app.config.sources.noneFound);
        }

        return this;
    },

    append: function(model) {
        if (model.id){
            this.$el.append(new app.view.SourceItemView({model: model}).render().el);
        }

    }
});

// this view is an experiment to see if we can bind a sensor directly to the map 
// so that it shows up when the model property "selected" changes
// UPDATE: this is working - it is being called at the bottom of SearchView
app.view.SensorItemView = Backbone.View.extend({

    initialize: function() {

        // get the color value from this sensor's Data Source
        // stored in app.config.sources - each source has a color
        var sourceModel = app.collection.sources.findWhere({id: this.model.get('type').id });
        var sourceColor = (sourceModel && sourceModel.get('color')) ? sourceModel.get('color') : 'blue';
        var geojsonMarkerOptions = {
            radius: 2,
            fillColor: sourceColor,
            color: sourceColor,
            weight: 5,
            opacity: 1,
            fillOpacity: 1
        };

        var geoJsonLayer = L.geoJson(this.model.toJSON(), {
            onEachFeature: function (feature, layer) {
                layer.bindPopup("<div class='search-popup'>"+feature.name+'</div>');
            },
            pointToLayer: function (feature, latlng) {
                return L.circleMarker(latlng, geojsonMarkerOptions);
            }
        });

        // we set this marker on the model so that we can use Leaflet's "bringToFront" function when switching between Pages
        this.model.set("marker", geoJsonLayer);
        this.listenTo(this.model, "change:selected", this.render, this);
        this.listenTo(this.model, "change:checked", this.render, this);
    },

    render: function() {
        if (this.model.get("checked")) {
            this.model.get("marker").addTo(app.view.searchView.map);
        } else {
            app.view.searchView.map.removeLayer(this.model.get("marker"));
        }

    }
});
// this View gets created by SearchView.renderSensorViews as a "container" for the Sensor views
// it prevents them from getting recreated when switching between Explore and Search
app.view.SensorsCollectionView = Backbone.View.extend({

    initialize: function() {
        _.each(this.collection.models, function(sensor) {
            // creating this view will build the marker and attach it to the sensor model
            // even without a reference to this view, we can do: sensor.get("marker")
            new app.view.SensorItemView({model: sensor});
        });
    },

    render: function() {
        var selected = this.collection.selected();
        _.each(selected, function(sensor) {
            // the marker gets added to the sensor model when the sensor view is created in sensor.js
            // bringToFront() is a Leaflet function
            sensor.get("marker").bringToFront();
        });
    }
});
app.view.HomeView = Backbone.View.extend({
    id: 'home-view',

  initialize: function(options) {
    var self = this;
    this.mapPin = options.mapPin;
    this.router = options.router;
    this.stationService = options.stationService;
    this.geoStreamService = options.geoStreamService;
    this.config = options.config;
    this.explore = options.config.explore;
    this.accordionItems = [];
    this.mapConfigs = {};
    this.selection = {};

    this.render = _.bind(this.render, this);
    this.$el.append(_.template($("#explore-template").html())());

    if (this.explore.accordion && this.explore.accordion.items && this.explore.accordion.items.length) {
        _.each(this.explore.accordion.items, function(accordionItem) {
            // inherit options from the accordion if not specified
            accordionItem.template = '#explore-accordion-item-template';
            accordionItem.attributes.class = 'accordion-group';

            var accordionItemView = new app.view.ExploreAccordionItemView(accordionItem);

            self.$('#explore-accordion-container').append(accordionItemView.el);
            self.accordionItems.push(accordionItemView);
            if (accordionItem.contents && accordionItem.contents.viewConfig && accordionItem.contents.viewConfig.map) {
                self.mapConfigs[accordionItem.attributes.id] = accordionItem.contents.viewConfig.map;
                self.mapConfigs[accordionItem.attributes.id].title = accordionItem.attributes.id;
                self.selection[accordionItem.attributes.id] = {};
            }
        });
    }

    app.view.exploreMapView = new app.view.ExploreMapView({
      parentView : this,
      mapPin : this.mapPin,
      router : this.router,
      geoStreamService : this.geoStreamService,
      geoJsonFeature : [],
      mapOptions: this.explore.map
    });

    this.$("#explore-map-view").append(app.view.exploreMapView.el);

    if(self.mapConfigs['trends'] && self.mapConfigs['trends'].aboutTrendData) {
      this.$el.append(_.template($(self.mapConfigs['trends'].aboutTrendData).html())());
    }

    // catch event fired by explore-pill or map-pin clicks
    // use this to fire the detail page route.
    app.event.eventBus.bind("sensors:refresh", (function(self) {
        return function() {
            _.each(self.accordionItems, function(accordionItem) {
                accordionItem.render();
            });
          self.render();
        };
    })(this));

    app.event.eventBus.on("change:trends", function(sensors, parameterTitle) {
        if ($('#trends').find('.data-nav-header-active').length) {
          if (sensors) {
            sensors = _.isArray(sensors) ? new Backbone.Collection(sensors) : sensors;
            self.selection.trends.sensors = sensors;
          } else  {
            sensors = self.selection.trends.sensors || app.collection.sensors;
          }

          if (parameterTitle) {
            self.selection.trends.parameter = parameterTitle;
          }

          var parameterModel = app.collection.parameters.findWhere({title: self.selection.trends.parameter});
          if (parameterModel) {
            app.event.eventBus.trigger('trends:changeLegendTitle', parameterModel.get('id'));
          }

          app.view.exploreMapView.setGeoJsonFeature(sensors.toJSON(), self.mapConfigs['trends'], self.selection.trends.parameter);
          app.view.exploreMapView.render(self.mapConfigs['trends']);

        }
    });



  },
  render: function(options) {
      var checkedSources = [];
      var sensors, visibleSensors;

      $('input[name="source"]:checked').each(function() {
        checkedSources.push($(this).attr("value"));
      });

      if (checkedSources.length) {
        sensors = app.collection.sensors.filter(function(sensor) {
          return _.contains(checkedSources, sensor.get('type').id);
        });
      } else {
        sensors = app.collection.sensors.models;
      }

      visibleSensors = _.map(sensors, function(sensor) {
        return sensor.toJSON();
      });
      app.view.exploreMapView.setGeoJsonFeature(visibleSensors, options);
      var self = this;
      _.each(app.config.sources.items, function(source) {
          self.$('#about-' + source.id + '-modal').draggable({handle: '.modal-header'});
      });

  }
});

app.view.LimnologyView = Backbone.View
    .extend({
      tagName : 'div',
      id : 'main-graphs',
      className : 'v11n-title',

      initialize : function() {
        this.render = _.bind(this.render, this);

        this.data = [];

        // d3.dispatch handles events passed from d3 charts
        this.dispatch = d3.dispatch("changeDepth");

        // initialize depth app.graphs with height 400, width 300
        this.depthGraph = app.graphs.depth().height(300).width(800)
          .dispatch(this.dispatch);

        // this almost certainly belongs in the detail_view.scala.html
        this.$el.append('<h4 id="depth-profile-title">Depth Profile</h4><div id="timestamp"></div>');
      },

      render : function() {
          var container = d3.select(this.el);
          var graph = container.selectAll(".depth-chart").data(this.data),
          graphEnter = graph.enter().append("div").attr("class", "depth-chart"),
          graphExit = graph.exit().remove();          
          graph.call(this.depthGraph);
      },

      setData : function(data) {
        if (data && data.length > 0) {
          if (data[0].values.length > 0) {
            $("#timestamp").empty();
            this.data = data;
          } else {
            $("#timestamp").text("no data found for " + data[0].key.toUTCString());
            this.data = [];
          }
        } else {
          $("#timestamp").text("no data found");
          this.data = [];
        }
      },

      changeDepth : function(callback) {
        this.dispatch.on("changeDepth", callback);
      }
    });
app.view.MapView = Backbone.View.extend({
      tagName : "div",
      className : "leaflet_map",

      initialize : function(options) {
        this.mapOptions = options.mapOptions;
        this.eventBus = options.eventBus;
        this.mapPin = $('#map-pin-template').html();
        this.mapArrowUp = _.template($('#map-arrow-up-template').html())();
        this.mapArrowDown = _.template($('#map-arrow-down-template').html())();
        this.mapLocation = _.template($('#map-location-template').html())();
        this.mapArrowUpThresh = _.template($('#map-arrow-up-thresh-template').html())();
        this.mapArrowDownThresh = _.template($('#map-arrow-down-thresh-template').html())();



        this.render = _.bind(this.render, this);

        this.map = L.map(this.el, this.mapOptions);

        var mapTileSource = this.mapOptions.mapTileSource || '//{s}.tile.osm.org/{z}/{x}/{y}.png';
        var mapAttribution = this.mapOptions.mapAttribution || '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors';
        this.tileLayer = L
            .tileLayer(mapTileSource,
                {
                  attribution: mapAttribution,
                  maxZoom: Math.min(16, app.config.views.home.explore.map.maxZoom),
                  maxBounds: app.config.views.home.explore.map.maxBounds,
                  minZoom: app.config.views.home.explore.map.minZoom
                });

        this.tileLayer.addTo(this.map); //check if this is necessary.

        this.setGeoJsonFeature(options.geoJsonFeature);
        return this;
      },

      render : function() {
        this.map.invalidateSize(true);

        if (this.geoJsonLayer) this.geoJsonLayer.addTo(this.map);
        return this;
      },

      setGeoJsonFeature : function(geoJsonFeature, parameter) {
        if (this.geoJsonLayer) this.map.removeLayer(this.geoJsonLayer);

        if (geoJsonFeature instanceof app.view.SensorsCollectionView) {
            this.geoJsonFeature = geoJsonFeature.toJSON();
        } else {
            this.geoJsonFeature = geoJsonFeature;
        }

        this.geoJsonLayer = L.geoJson(this.geoJsonFeature, this.geoJsonOptions(this, parameter));
      },

      getSourcePin: function(sensor) {
        var source, sourceColor;
        if (sensor && sensor.properties && sensor.properties.type && sensor.properties.type.id) {
          source = app.collection.sources.findWhere({id: sensor.properties.type.id});
          // alert(source.values());
          if (source) {
            sourceColor = source.getColor();
            //alert(this.mapPin);
            if (sensor.properties.online_status  == "offline"){
              return _.template(this.mapPin)({fill: sourceColor, stroke: "red"});
            } else {
              return _.template(this.mapPin)({fill: sourceColor});
            }
          }
        }

        return _.template(this.mapPin)();
      }
});

app.view.ExploreMapView = app.view.MapView.extend({
    tagName : "div",
    className : "leaflet_map",

    markers : {},

    events: {
        'click .btn': 'viewDetails'
    },

    // limits the
    latlngFormat : d3.format(".4f"),

    // this function gets called in MapView.js (we're using MapView.js like a super-class)
    // It is used with: L.geoJson(geoJsonFeature, geoJsonOptions)
    // So this sets the options for the Map, specifically
    geoJsonOptions : function(self, parameter) {
        var pin = {};
        return {
            pointToLayer: function(feature, latlng) {
                pin.template = self.pinTemplate || self.getSourcePin(feature);
                if (self.pinFunction) {
                    pin = self.pinFunction(feature, parameter);
                } else {
                    pin.template = self.getSourcePin(feature);
                }
                var marker = L.marker(latlng, {
                    icon : L.divIcon({
                        html : '<div id="' + ("station-" + feature.properties.name.replace(/\W/g, "-")) + '" class="' + feature.properties.type.id + '">' + pin.template + '</div>'
                    }),
                    title : feature.properties.name,
                    riseOnHover : true
                });

                self.markers["station-" + feature.properties.name.replace(/\W/g, "-")] = marker;

                return marker;
            },
            onEachFeature: function(feature, layer) {

                if (feature.properties && feature.properties.popupContent) {
                    var popupContent = self.populatePopupTemplate(feature, pin.threshold, 'fetching...', ['fetching...']);

                    layer.bindPopup(popupContent, { autoPanPadding: [30, 30], offset: new L.Point(160, -50)});
                    layer.on("mouseover", function(event) {
                        if(event.originalEvent.target.className.baseVal === "mouseCapture") {
                            self.hoverMarker("station-" + feature.properties.name.replace(/\W/g, "-"));
                        }
                    });
                    layer.on("mouseout", function(event) {
                        if(event.originalEvent.target.className.baseVal === "mouseCapture") {
                            self.unhoverMarker("station-" + feature.properties.name.replace(/\W/g, "-"));
                        }
                    });
                    layer.on("click", function(event) {
                        self.highlightMarker("station-" + feature.properties.name.replace(/\W/g, "-"), self.title);
                    });
                }
            }
        };
    },

    initialize : function(options) {
        var self = this;
        _.bindAll(this, "setGeoJsonFeature");
        this.options = options;
        this.popupTemplate = _.template($("#map-popup-template").html());

        this.geoStreamService = options.geoStreamService;

        // essesntially super.initialize(options)
        app.view.MapView.prototype.initialize.apply(this, arguments);

        app.event.eventBus.bind("marker:mouseover", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.hoverMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:mouseout", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.unhoverMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:click", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;

                self.highlightMarker(markerId);
            };
        })(this));

        app.event.eventBus.bind("marker:unclick", (function(self) {
            return function(markerId) {
                markerId = "station-" + markerId;
                self.unhighlightMarker(markerId);
            };
        })(this));


        app.event.eventBus.bind("station:update", (function(self) {
          return function(sensor) {
            var popupContent = self.populatePopupTemplate(sensor);

            self.markers["station-" + sensor.get('name').replace(/\W/g, "-")].setPopupContent(popupContent, { offset: new L.Point(160, -50)});
          };
        })(this));

        $(window).on('scroll', this, function(event) {
            event.data.resizemap(false);
        });

        app.event.eventBus.bind("change:region", (function(self) {
            return function(regionName) {
                ;
            };
        })(this));

        app.event.eventBus.on("change:layer", (function(self) {
            return function(layerModel) {
                if (layerModel.get("checked")) {
                    layerModel.get("layer").addTo(self.map);
                    if (app.config.layers && app.config.layers.zoomInOnClick) {
                        self.map.panTo(new L.LatLng(layerModel.get("geometry").coordinates[1], layerModel.get("geometry").coordinates[0]));
                    }

                } else {
                    self.map.removeLayer(layerModel.get("layer"));
                }
            }
        })(this));

        app.event.eventBus.on("change:layer:opacity", (function(self) {
            return function(layerModel) {
                var opacity = layerModel.get("properties").opacity;
                layerModel.get("layer").setOpacity(opacity);
            }
        })(this));

        var mapSpinner = app.config.views.home.explore.map.spinnerOptions || {
          lines: 17, // The number of lines to draw
          length: 0, // The length of each line
          width: 22, // The line thickness
          radius: 17, // The radius of the inner circle
          corners: 1, // Corner roundness (0..1)
          rotate: 28, // The rotation offset
          direction: 1, // 1: clockwise, -1: counterclockwise
          color: '#FFF', // #rgb or #rrggbb or array of colors
          speed: 1.3, // Rounds per second
          trail: 70, // Afterglow percentage
          shadow: true, // Whether to render a shadow
          hwaccel: true, // Whether to use hardware acceleration
          className: 'spinner', // The CSS class to assign to the spinner
          zIndex: 2e9, // The z-index (defaults to 2000000000)
          top: '50%', // Top position relative to parent
          left: '50%' // Left position relative to parent
        };
        this.spinner = new Spinner(mapSpinner);

        app.event.eventBus.on("explore:map:spinner:start", function() {
          self.$el.append(self.spinner.spin().el);
        });
        app.event.eventBus.on("explore:map:spinner:stop", function() {
          self.spinner.stop();
        });
    },

    resizemap : function(callrender) {
        // constants, only need to be done once
        var exploreMapViewEl = $("#explore-map-view");
        var header = $("#header");
        if (!this.extra) {
            this.extra = exploreMapViewEl.outerHeight(false) - exploreMapViewEl.height();
            this.padding = 10;
            this.marginheader = parseInt(header.css('margin-bottom'), 10);
        }

        // compute map size and offset
        var windowHeight = $(window).height();
        if (header && header.length) {
            var headerBounds = header[0].getBoundingClientRect();
            var offset = headerBounds.height + this.padding + Math.max(this.marginheader, -headerBounds.height - headerBounds.top);
            var footerTop = $(".footer")[0].getBoundingClientRect().top;
            var height = windowHeight - this.marginheader - 2*this.padding - this.extra - Math.max(0, windowHeight - footerTop) - Math.max(0, headerBounds.bottom);

            // set the map size
            exploreMapViewEl.offset({ top: offset });
            exploreMapViewEl.height(height);

            // render map
            this.map.invalidateSize(true);
            if (callrender) {
                app.view.MapView.prototype.render.apply(this, arguments);
            }
        }

    },

    render : function() {
        if(!this.geoJsonFeature.length) return; //if geoJsonLayer doesn't exist, don't render this view
        // this.geoJsonLayer gets set in MapView.setGeoJsonFeature
        this.map.fitBounds(new L.featureGroup([this.geoJsonLayer]).getBounds().pad(0.05),
            {maxZoom: ((this.options && this.options.map && this.options.map.maxZoom) ? this.options.map.maxZoom : 16)});
        this.resizemap(true);

        if (app.collection.layers && app.collection.layers.length > 0) {
            app.collection.layers.forEach((function(self) {
                return function(layer) {
                    if (!layer.get("initialized")) {
                        layer.set("initialized", true);
                        if (layer.get("checked")) layer.get("layer").addTo(self.map);
                    }
                }
            })(this));
        }
    },
    setGeoJsonFeature : function(geoJsonFeature, options, parameter) {
        if (this.pinFunction) delete this.pinFunction;
        if (options) {
            if (options.popupTemplate) this.popupTemplate = _.template($(options.popupTemplate).html());
            if (options.pinTemplate) this.pinTemplate = _.template($(options.pinTemplate).html())();
            if (options.pinFunction && (typeof this[options.pinFunction] == 'function')) this.pinFunction = this[options.pinFunction];
            if (options.title) this.title = options.title;
            if (options.legend) {
                var legend = _.template($(options.legend).html());
                //Add legend if it doesn't already exist
                if($('#trends-map-legend').length == 0) {
                    $("#explore-map-view").prepend(legend);

                    // TODO this needs to be changed to something more specific
                    // what this is doing is using app.config.binning.time to determine
                    // what to put in the dropdown on the trends map
                    // instead, this should go in something like app.config.views.trendsMap
                    // since it isn't really binning
                    if (app.config.binning.time && app.config.binning.time != 'none') {
                        //Add season drop down
                        if (!app.view.trendsSelectSeason) {
                          app.view.trendsSelectSeason = new app.view.SelectCollectionView({
                            collection: new Backbone.Collection(getBinningDesignators(app.config.binning.time).items),
                            trigger: "trends:redrawMapArrows"
                          });
                        }
                        $("#trends-map-season-control-div").append(app.view.trendsSelectSeason.el);
                        $("select").addClass("trends-map-season-control");
                    }
                    //Add SVG icons.
                    var trendUp = _.template($("#map-arrow-up-template").html());
                    var trendDown = _.template($("#map-arrow-down-template").html());
                    var trendLocation = _.template($("#map-location-template").html());
                    var trendUpThresh = _.template($("#map-arrow-up-thresh-template").html());
                    var trendDownThresh = _.template($("#map-arrow-down-thresh-template").html());

                    $("#trends-map-legend-up-icon").append(trendUp);
                    $("#trends-map-legend-down-icon").append(trendDown);
                    $("#trends-map-legend-no-trend-icon").append(trendLocation);
                    $("#trends-map-legend-threshold-icon-up").append(trendUpThresh);
                    $("#trends-map-legend-threshold-icon-down").append(trendDownThresh);
                }
            } else {
                $('#map-legend').remove();
            }

        }
        app.view.MapView.prototype.setGeoJsonFeature.call(this, geoJsonFeature, parameter);

        if(this.geoJsonFeature && this.geoJsonFeature.length) {
            this.render();
        }
    },

    hoverMarker: function(markerId) {
        d3.selectAll('#' + markerId + ' svg').attr("class", "map-pin-hover");
    },

    highlightMarker : function(markerId, title) {
        this.markers[markerId].openPopup();

        if (title && (title == 'trends')) {
            this.panToMarker(markerId);
            return;
        }

        if( app.collection.sensors.findWhere({ name: this.markers[markerId].options.title }).get("parameters").length == 0 ) {
            $("#detail-button").text("No Data Available")
                .attr('disabled', true)
                .bind('click', false);
        }
        else {
            app.event.eventBus.trigger("station:fetch", this.markers[markerId].options.title);
            // disable button for now
            $("#detail-button").text("Loading Data ...")
                .attr('disabled', true)
                .bind('click', false);

            // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
            // are averaged before being drawn in the graphs.
            // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
            // as an argument. From there, you can return 'smart', 'semi', or some other binning value
            // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
            // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
            this.binningTime = "";
            if (app.config.binning.filterFunction) {
              this.binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: markerId.slice(8)}));
            } else if (app.config.binning.time) {
              this.binningTime = app.config.binning.time;
            } else {
              this.binningTime = 'smart';
            }

            // this.binningTime is 'smart', then determine aggregation by temporal range
            if (this.binningTime == 'smart') {
                this.binningTime = getAggregationBin(
                  new Date(this.markers[markerId].feature.properties.min_start_time),
                  new Date(this.markers[markerId].feature.properties.max_end_time)
                );
            }

            // Use default depth if no depth value provided
            var sensorDepth = app.config.binning.depth || 1;

            // fetch data, when fetched enable button
            this.geoStreamService.getBinnedData(this.binningTime, sensorDepth, this.markers[markerId].feature.id, true, function(data) {
                $("#detail-button").text("View Data")
                    .attr('disabled', false)
                    .unbind('click');
            });
        }
    },

    unhoverMarker: function(markerId) {
        d3.selectAll('#' + markerId + ' svg').attr("class", "map-pin");
    },

    unhighlightMarker : function(markerId) {
        this.markers[markerId].closePopup();
    },

    panToMarker : function(markerId) {
        // The 280 size is the size of the popup width.
        var targetPoint = this.map.project(this.markers[markerId].getLatLng(), this.map.getZoom()).add([280 / 2, 0]);
        var targetLatLng = this.map.unproject(targetPoint, this.map.getZoom());
        this.map.panTo(targetLatLng);
    },

    populatePopupTemplate : function(sensor, threshold) {
        if (sensor instanceof app.model.Sensor) {
            sensor = sensor.toJSON();
        }
        return this.popupTemplate({model: sensor, threshold: threshold, sourceColor: sensor.properties.color});
    },

    viewDetails: function(e) {
        e.preventDefault();
        var location = $(e.currentTarget).data('location');
        app.event.eventBus.trigger("change:location", location);
    },

    getTrendsPin: function(sensor, parameter) {
        var areaModel, parameterModel, thresholdParameter;
        _.each(app.config.areas.items, function(area) {
            if (area.properties.id == "er") return;
            if (gju.pointInPolygon(sensor.geometry, area.geometry)) {
              areaModel = app.collection.areas.findWhere({id: area.properties.id});
            }
        });

        if (parameter) {
            parameterModel = app.collection.parameters.findWhere({title: parameter});
            if (parameterModel) {
              thresholdParameter = parameterModel.get('trendThreshold');
            } else {
              thresholdParameter = undefined;
            }
        }

        if (sensor && sensor.properties && sensor.properties.trends && sensor.properties.trends[parameter]
            && sensor.properties.trends[parameter].percentage_change) {
            if (+sensor.properties.trends[parameter].percentage_change > 0 && areaModel) {
                if (+sensor.properties.trends[parameter].last_average > areaModel.get('threshold')[parameter] && thresholdParameter) {
                    return {template: this.mapArrowUpThresh, threshold: areaModel.get('threshold')[parameter]};
                } else {
                    return {template: this.mapArrowUp, threshold: areaModel.get('threshold')[parameter]};
                }
            } else if (+sensor.properties.trends[parameter].percentage_change < 0 && areaModel) {
                if (+sensor.properties.trends[parameter].last_average > areaModel.get('threshold')[parameter] && thresholdParameter) { 
                    return {template: this.mapArrowDownThresh, threshold: areaModel.get('threshold')[parameter]};
                } else {
                    return {template: this.mapArrowDown, threshold: areaModel.get('threshold')[parameter]};
                }
            } else if (+sensor.properties.trends[parameter].percentage_change > 0 && !areaModel) {
                return {template: this.mapArrowUp};
            } else if (+sensor.properties.trends[parameter].percentage_change < 0 && !areaModel) {
                return {template: this.mapArrowDown};
            } else {
                return {template: this.mapLocation};
            }
        } else { 
            return {template: this.mapLocation};
        }
    }

});

app.view.MiniMapView = app.view.MapView.extend({
    geoJsonOptions : function(self) {
        return {
            pointToLayer : function(feature, latlng) {
                var marker = L.marker(latlng, {
                    icon : L.divIcon({
                        html : '<div id="' + feature.properties.name + '" class="' + feature.properties.type.id + '">' + self.getSourcePin(feature) + '</div>'
                    })
                });
                
                return marker;
            },
            onEachFeature : function(feature, layer) {
                
            }
        };
    },
    
    initialize : function(options) {
        app.view.MapView.prototype.initialize.apply(this, arguments);
        return this;
    },
    
    render : function() {
        app.view.MapView.prototype.render.apply(this, arguments);
        return this;
    },
    
    setGeoJsonFeature : function(geoJsonFeature) {
        if(geoJsonFeature.length === 0) return;
      
        app.view.MapView.prototype.setGeoJsonFeature.apply(this, arguments);
        
        var coordinates = geoJsonFeature[0].geometry.coordinates;
        this.map.setView(new L.LatLng(coordinates[1], coordinates[0]), 5);
        this.map.invalidateSize();
    }
});
app.view.TimeSeriesView = Backbone.View.extend({
	tagName : 'div',
	id : 'time',
    
    defaultOptions : {
        height : 250,
        width : 525,
        boxWhiskerMarginH : 60,
        boxWhiskerMarginV : 60
    },
	
	initialize : function(options) {
		this.render = _.bind(this.render, this);
        
        this.data = [];
        this.showEP = true;
        this.haveEP = true;
        
        this.chartOptions = _.extend({}, this.defaultOptions, options.chartOptions);
        
		this.timeSeries = app.graphs.timeSeries().height(this.chartOptions.height).width(this.chartOptions.width).max(15).padding("30 10 50 60");
		this.boxWhisker = app.graphs.boxwhisker().height(this.chartOptions.height).width(175).margin(this.chartOptions.boxWhiskerMarginH, this.chartOptions.boxWhiskerMarginV);
		this.$el.append('<h4 id="time-series-title" class="span9">Time Series</h4><h4 id="box-whisker-title" class="span3">Box and Whisker<a href="#bw-modal" role="button" data-toggle="modal"><i class="icon-info-sign bw-info"></i></a></h4>');
	},
	
	render : function() {
        if(!this.data || !this.data.length) {
            this.$el.html('<div class="no-data">No data for the selected attributes.</div>');
        } else {
            this.$('.no-data').remove();
            this.$('.graphs-mouseOverVerticalLine').remove();
            var that = this;
            
            var container = d3.select(this.el).selectAll(".time-series-distribution-charts").data(this.data, function(d) { return d.key; });
            container.enter().append("div").attr("class", "time-series-distribution-charts row-fluid");
            container.exit().remove();
            
            var mytimeseries = container.selectAll(".time-series-chart").data(function(d) { return that.showEP ? [d.ep] : [d.all]; }, function(d) { return d.key; });
            mytimeseries.enter().append("div").attr("class", "time-series-chart span9");
            mytimeseries.exit().remove();
            mytimeseries.call(this.timeSeries);

            // comment this line off to disable switching between all data and filtered data
            mytimeseries.on("click", function(e) { that.showEP = !that.showEP; that.render(); });
            
            var boxWhiskerChart = container.selectAll(".box-whisker-chart").data(function(d) { return [d.all]; }, function(d) { return d.key; });
            boxWhiskerChart.enter().append("div").attr("class", "box-whisker-chart span3");
            boxWhiskerChart.exit().remove();
            boxWhiskerChart.call(this.boxWhisker);
        }
        return this;
	},

    setData : function(alldata, summerEPdata) {
        _.each(alldata, function(d) {
            var parameter = app.collection.parameters.findWhere({id: d.values[0].key});
            if (parameter) {
                d.values[0].title = parameter.get("title").replace(/<(.*?)>/g, '');
            } else {
                d.values[0].title = d.values[0].key;
            }

            if(app.config.graphs.timeSeries.showUnits.graphTitle && d.values[0].unit)
                d.values[0].title += " (" + d.values[0].unit + ")";
        });
        if (_.isUndefined(summerEPdata)) {
            summerEPdata = alldata;
            this.haveEP = false;
        } else {
            if (!this.haveEP) {
                this.showEP = true;
            }
            this.haveEP = true;
            _.each(summerEPdata, function(d) {
                var parameter = app.collection.parameters.findWhere({id: d.values[0].key});
                if (parameter) {
                    d.values[0].title = parameter.get("title").replace(/<(.*?)>/g, '');
                } else {
                    d.values[0].title = d.values[0].key;
                }

                if(app.config.graphs.timeSeries.showUnits.graphTitle && d.values[0].unit)
                    d.values[0].title = d.values[0].title + " (" + d.values[0].unit + ")"

                d.values[0].title += " (summer displays epilimnion data only)";
            });
        }
        this.data = [];
        for (var i = 0; i < alldata.length; i++) {
            this.data.push({key: i, all: alldata[i], ep: summerEPdata[i]});
        }
        this.render();
    }
});

app.view.SearchView = Backbone.View.extend({

    attributes: function() {
        return {
            tagName: this.options.attributes.tagName,
            className: this.options.attributes.className,
            id: this.options.attributes.id
        }
    },

    events : {
        "click #download-permalink:not(.disabled)" : "setPermalink",
        "click #download-visualize:not(.disabled)" : "visualizeResults",
        "click .download:not(.disabled)": app.utils.interceptDownload
    },

    initialize : function(options) {
        var self = this;
        this.config = options.config;
        this.resultsVisualized = false;
        _.bindAll(this, "setPermalink", "updateDownloadLink", "updateDatesFromData");
        this.$el.append(_.template($(this.config.templateSelector).html())());

        this.eventBus = app.event.eventBus;
        this.stationService = options.stationService;

        this.dateRangeSelector = this.config.dateRangeSelector;
        this.layers = this.config.layers;
        this.downloadButtons = this.config.downloadButtons;

        this.geojsonMarkerOptions = {
            fillColor: this.config.map.markerFillColor,
            color: this.config.map.markerColor,
            radius: this.config.map.radius,
            weight: this.config.map.weight,
            opacity: this.config.map.opacity,
            fillOpacity: this.config.map.fillOpacity
        };

        // container for currently selected sensors
        this.sensorsfound = L.layerGroup();
        this.sensors = {};
        this.coordinates = {};

        // the format specified in app.config.views.search.dateRangeSelector.format
        // is passed to startDate and endDate, however, if you change the format
        // in `this.startDate = moment('2010....'`  <-- here then this.format can't be changed
        this.format = this.dateRangeSelector.format;
        this.startDate = moment('1970-10-01 1:00:00', this.format);
        this.endDate = moment();
        this.dataStartDate = this.startDate;
        this.dataEndDate = this.endDate;

        // map
        this.map = L.map(this.$(this.config.map.id)[0], {
            maxBounds: this.config.map.maxBounds,
            minZoom: this.config.map.minZoom,
            maxZoom: this.config.map.maxZoom,
            scrollWheelZoom: this.config.map.scrollWheelZoom
        });

        // Initialise the FeatureGroup to store editable layers
        this.drawnItems = new L.FeatureGroup();
        this.map.addLayer(this.drawnItems);

        // tileLayer
        var mapTileSource = this.config.map.mapTileSource || '//{s}.tile.osm.org/{z}/{x}/{y}.png';
        var mapAttribution = this.config.map.mapAttribution || '&copy; <a href="http://osm.org/copyright">OpenStreetMap</a> contributors';
        var tileLayer = L.tileLayer(mapTileSource,
            {
                attribution: mapAttribution,
                maxZoom: Math.min(16, this.config.map.maxZoom)
            });
        tileLayer.addTo(this.map);

        // controls
        var baseMaps = {
            "BaseMap": tileLayer
        };

        this.mapControls = L.control.layers(baseMaps);
        this.mapControls.addTo(self.map);

        this.resultsView = new app.view.ResultsView({geostreamService: options.geostreamService});

        app.event.eventBus.bind("sensors:refresh", (function(self) {
            return function() {
                app.view.sensors = new app.view.SensorsCollectionView({collection: app.collection.sensors});
                self.render();
            };
        })(this));

        this.eventBus.on("dates:new", this.updateDatesFromData);

        this.eventBus.on("sensors:selected", (function(self) {
            return function(sensors) {
                self.sensors = sensors;
            };
        })(this));

        this.eventBus.on("map:circle:select", (function(self) {
            return function(coordinates) {
                self.coordinates = coordinates;
                self.clearLayers();
                var params = {geocode: coordinates.latlng.lat + ',' + coordinates.latlng.lng + ',' + (coordinates.radius/1000)};
                app.collection.sensors.select(params, function() {
                    if (app.config.areas.zoomInOnClick) {
                        app.event.eventBus.trigger("centerMap");
                    }
                    self.updateDownloadLink();
                });
            };
        })(this));

        this.eventBus.on("map:polygon:select", (function(self) {
            return function(coordinates) {
                self.coordinates = coordinates;
                self.clearLayers();
                var params = {geocode: coordinates.points.map(function(x) { return x.lat + "," + x.lng; }).join()};
                app.collection.sensors.select(params, function() {
                    if (app.config.areas.zoomInOnClick) {
                        app.event.eventBus.trigger("centerMap");
                    }
                    self.updateDownloadLink();
                });
            };
        })(this));

        this.eventBus.on("check:parameter", function(options) {
            if (options.view && options.view == "searchView"){
                self.checkParameter(options);
            }
        });

        this.shapeOptions = this.config.shapeOptions;
        this.drawOptions = {
            position: this.config.drawOptions.position,
            draw: {
                polyline: this.config.drawOptions.draw.polyline,
                polygon: {
                    shapeOptions: this.shapeOptions
                },
                circle: {
                    shapeOptions: this.shapeOptions
                },
                rectangle: {
                    shapeOptions: this.shapeOptions
                },
                marker: this.config.drawOptions.draw.marker
            },
            edit: {
                featureGroup: this.drawnItems, //REQUIRED!!
                remove: this.config.drawOptions.edit.remove
            }
        };

        // Initialise the draw control and pass it the FeatureGroup of editable layers
        var drawControl = new L.Control.Draw(this.drawOptions);
        this.map.addControl(drawControl);

        this.resetMapButtonOptions = {
            text: this.config.resetMapButtonOptions.text,
            iconUrl: this.config.resetMapButtonOptions.iconUrl,
            glyph: this.config.resetMapButtonOptions.glyph,
            onClick: function() {
                $('input[name=areaRadios]').prop("checked", false);
                self.resetSearch();
                self.resultsView.render();
            },  // callback function
            hideText: this.config.resetMapButtonOptions.hideText,
            maxWidth: this.config.resetMapButtonOptions.maxWidth,
            doToggle: this.config.resetMapButtonOptions.doToggle,
            toggleStatus: this.config.resetMapButtonOptions.toggleStatus
        };

        var resetMapButton = new L.Control.Button(this.resetMapButtonOptions);
        resetMapButton.addTo(this.map);

        var colors = d3.scale.category10();
        // basins
        // originally from http://www.great-lakes.net/gis/data/refdata.html
        // TODO: extract this - check for options.layers && options.layers.items

        if (app.config.basins && app.config.basins.items && app.config.basins.items.length > 0) {
            self.basins = L.geoJson(app.config.basins.items,{
                style: function(feature) {
                    switch (feature.properties.LAKEBASIN) {
                        case 'Erie': return {color: colors(0),"weight": 1, "opacity": 1};
                        case 'Huron': return {color: colors(1),"weight": 1, "opacity": 1};
                        case 'Michigan': return {color: colors(2),"weight": 1, "opacity": 1};
                        case 'Ontario': return {color: colors(3),"weight": 1, "opacity": 1};
                        case 'St. Lawrence': return {color: colors(4),"weight": 1, "opacity": 1};
                        case 'Superior': return {color: colors(5),"weight": 1, "opacity": 1};
                        default: return {color: colors(Math.round(Math.random()*10)),"weight": 1,};
                    }
                },
                onEachFeature: function (feature, layer) {
                    layer.bindPopup("<div class='" + self.layers.bindPopupClass + "'>"+feature.properties.LAKEBASIN+'</div>');
                }
            });
            // self.basins.addTo(self.map);
            self.mapControls.addOverlay(self.basins, app.config.basins.title);
        }
    },

    render : function() {
        var self = this;

        var map = this.map; // FIXME
        this.sensorsfound.addTo(map);
        this.map.on('draw:created', function (e) {
            var type = e.layerType,
                layer = e.layer;

            if ((type === 'polygon') || (type === 'rectangle')) {
                self.eventBus.trigger("map:polygon:select", {'points': layer.getLatLngs()});
            }

            if (type === 'circle') {
                self.eventBus.trigger("map:circle:select", {'latlng': layer.getLatLng(), 'radius': layer.getRadius()});
            }

            $('input[name=areaRadios]').prop("checked", false);
            // Do whatever else you need to. (save to db, add to map etc)
            e.layer.addTo(self.drawnItems);
        });

        map.on('draw:edited', function () {
            // Update db to save latest changes.
        });

        map.on('draw:deleted', function (e) {
            self.clearMarkers();
            self.resetButtons();
            // app.event.eventBus.trigger("zoomOut");
        });


        app.collection.sensors.each(function(sensor) {
            var geoJsonLayer = L.geoJson(sensor.toJSON(), {
                onEachFeature: function (feature, layer) {
                    layer.bindPopup("<div class='" + self.layers.bindPopupClass + "'>"+feature.name+'</div>');
                },
                pointToLayer: function (feature, latlng) {
                    return L.circleMarker(latlng, self.geojsonMarkerOptions);
                }
            }).addTo(self.map);
        });


        this.eventBus.on("centerMap", function() {
            self.map.fitBounds(self.drawnItems.getBounds(), {
                padding: [ 5, 5 ]
            });
        });

        this.eventBus.on("zoomOut", function() {
            self.map.fitBounds(self.map.maxBounds);
        });

        // draw default map
        if (app.config.views.search.map.maxBounds == undefined) {
            var bbox = new L.LatLngBounds();
            app.collection.sensors.each(function(sensor) {
                bbox.extend([sensor.attributes.geometry.coordinates[1],
                             sensor.attributes.geometry.coordinates[0]]);
            });
            if (bbox.isValid()) {
                app.config.views.search.map.maxBounds = bbox.pad(0.05);
            }
        }
        this.map.fitBounds(app.config.views.search.map.maxBounds);
        this.map.invalidateSize();

        this.renderDates();
        this.renderAreas();
        this.renderSources();
        this.renderParameters();
        this.renderSensorViews();

    },

    // this is registered in the events up in the top of SearchView
    updateDownloadLink : function(event) {
        // if the user has drawn a area, this.coordinates will be populated
        // if the user hasn't drawn a area, we shouldn't filter by geocode
        var params = {};
        if (this.coordinates) {
            if (this.coordinates.radius) {
                // the user drew a circle
                var latlng = this.coordinates.latlng;
                var radius = this.coordinates.radius;
                params["geocode"] = latlng.lat + ',' + latlng.lng + ',' + (radius/1000);

            } else if (this.coordinates.points) {
                // the user drew a polygon
                var points = this.coordinates.points.map(function(x) {
                    return x.lat + "," + x.lng;
                }).join();
                params["geocode"] = points;
            }
        }

        // query the sources collection for models that have been checked, and return the id's in an array
        var sources = _.pluck(_.where(app.collection.sources.toJSON(), {checked: true}), 'id');
        if (sources) params["sources"] = sources;

        // get selected attributes from the collection
        var parameters = _.flatten(_.pluck(_.where(app.collection.parameters.toJSON(), {checked: true}), 'title'));
        if (parameters) {
            params["attributes"] = parameters;
            parameters = _.flatten(_.pluck(_.where(app.collection.parameters.toJSON(), {checked: true}), 'id'));
            if (parameters) params["attributes"] = _.union(params["attributes"], parameters);
        }

        // verify start and end date are not only values set
        if ( (params["attributes"].length < 1) && (params["sources"].length < 1) && (!params["geocode"]) ||
                ((app.collection.sensors.getChecked().length +
                app.collection.sensors.where({selected: true}).length) == 0 ) ) {
            this.resetButtons();
        }
        else {

            if (this.dataStartDate) params["since"] = this.dataStartDate.format(this.format);
            if (this.dataEndDate) params["until"] = this.dataEndDate.format(this.format);

            // create the url
            var url = app.datapoints.url + '?' + $.param(params, true);

            // update the DOM
            $(this.downloadButtons.json.id).attr('href', url + "&format=json");
            $(this.downloadButtons.json.id).removeClass('disabled');
            $(this.downloadButtons.csv.id).attr('href', url + "&format=csv");
            $(this.downloadButtons.csv.id).removeClass('disabled');
            $(this.downloadButtons.permalink.downloadID).removeClass('disabled');
            $(this.downloadButtons.visualize.id).removeClass('disabled');
        }
    },

    setPermalink : function(event) {
        var template = $(this.downloadButtons.permalink.template).html();
        $(this.downloadButtons.permalink.id).html(_.template(template,{url: $(this.downloadButtons.json.id).attr('href')}));
        $(this.downloadButtons.permalink.modalID).modal('show');
    },

    // this function is the equivalent to this.sensorsfound.clearLayers(), but works with app.collection.sensors
    // would be nice to keep track of the layer id and related sensors when drawing a area on the map
    // so that clearing a drawn area would only deselect the sensors underneath it. for now, we're clearing all selected sensors
    clearMarkers: function() {
        app.collection.sensors.uncheckAll();
    },

    /**
     * Clear all drawn layers. Similar to just calling this.drawnItems.clearLayers() in initialize or render,
     * but as a function, it can be called by the ResetMapButton
     */
    clearLayers: function() {
        // create a reference to this view as 'self'
        var self = app.view.searchView;
        if (self.drawnItems && (typeof self.drawnItems.clearLayers == 'function')) {
            self.drawnItems.clearLayers();
        }
        // clearMarkers unchecks all sensors, sources, parameters and areas
        self.clearMarkers();

        // app.event.eventBus.trigger("zoomOut");
    },

    resetSearch: function() {
        this.clearLayers();
        app.collection.sources.uncheckAll();
        app.collection.parameters.uncheckAll();
        this.coordinates = {};
        //app.event.eventBus.trigger("zoomOut");
        this.resetButtons();
    },

    resetButtons: function() {
        $(this.downloadButtons.csv.id).removeAttr('href');
        $(this.downloadButtons.csv.id).addClass('disabled');
        $(this.downloadButtons.json.id).removeAttr('href');
        $(this.downloadButtons.json.id).addClass('disabled');
        $(this.downloadButtons.permalink.downloadID).addClass('disabled');
        $(this.downloadButtons.visualize.id).addClass('disabled');
    },

    renderAreas : function() {
        if (!app.view.areasView) {
            app.view.areasView = new app.view.AreasCollectionView({el: "div#areasView", collection: app.collection.areas});
        }
        app.view.areasView.render();
    },

    renderSources : function() {
        if (!app.view.sourcesView) {
            if (app.collection.sources.getSourcesWithSensors().length > 0) {
                app.view.sourcesView = new app.view.SourcesCollectionView({collection: app.collection.sources.getSourcesWithSensors()});
            }
        }
        app.view.sourcesView.render();
    },

    renderParameters: function() {
        if (!this.parametersView) {
            this.parametersView = new app.view.ParametersCollectionView({
                el: "table#parametersView",
                collection: app.collection.parameters,
                view: "searchView",
                itemOptions: {
                    tagName: 'tr',
                    className: 'parameterView'
                }
            });
        }
        this.parametersView.render();

    },

    checkParameter: function(options) {
        this.model = options.model;

        if (app.collection.sensors) var allSelectableSensors = app.collection.sensors.getSelectableSensors();
        var parameterSelectableSensors = this.model.getSelectableSensors();
        if (app.collection.areas) var areasChecked = app.collection.areas.getChecked();

        // we have to do this next step because if you look at the sensor.js file, each sensor's isChecked function
        // listens to areaChecked, sourceChecked or parameterChecked to determine if it should show its color marker
        // I'd rather not do this, because after you uncheck a parameter, the sensor's areaChecked doesn't go back to true
        // and the sensor won't draw its color marker again, even though the area has selected it, but I don't see any other way around this yet.
        if (areasChecked) {
            _.each(allSelectableSensors, function(sensor) {
                sensor.set("areaChecked", false);
            });
        }

        if (this.model.get("checked")) {
            this.model.set("checked", false);
            // before you set parameterChecked to false, you need to see if all other parameters for this sensor are unchecked.
            _.each(parameterSelectableSensors, function(sensor) {
                sensor.set("parameterChecked", false);
            });


        } else {
            this.model.set("checked", true);
            _.each(parameterSelectableSensors, function(sensor) {
                sensor.set("parameterChecked", true);
            });
        }
        // always update the buttons when something gets checked
        this.updateDownloadLink();

    },

    updateDatesFromData: function(dateRange) {
        if (this.isRendering) return;
        this.isRendering = true;
        this.dataStartDate = dateRange.min;
        this.dataEndDate = dateRange.max;
        this.renderDates();
        this.isRendering = false;
    },

    renderDates: function() {
        var self = this;
        var sliderRange = $( self.dateRangeSelector.id );
        sliderRange.slider(
            {
                range: true,
                min: self.startDate.year(),
                max: self.endDate.year(),
                values: [self.dataStartDate.year(), self.dataEndDate.year()],
                slide: function(event, ui)
                {
                    $("#searchtimeSelectedStart").text(ui.values[ 0 ]);
                    $("#searchtimeSelectedStop").text(ui.values[ 1 ]);
                    self.dataStartDate = moment(ui.values[0].toString() + '-01-01 1:00:00', self.format);
                    self.dataEndDate = moment(ui.values[1].toString() + '-12-31 23:59:59', self.format);
                    self.resultsView.renderDates(self.dataStartDate, self.dataEndDate);
                    self.updateDownloadLink();
                }
            });

        this.$("#searchtimeStart").text(self.startDate.year());
        this.$("#searchtimeStop").text(self.endDate.year());
        this.$("#searchtimeSelectedStart").text(sliderRange.slider("values", 0));
        this.$("#searchtimeSelectedStop").text(sliderRange.slider("values", 1));

    },

    renderSensorViews: function() {
        if (!app.view.sensors) {
            app.view.sensors = new app.view.SensorsCollectionView({collection: app.collection.sensors});
        }
        app.view.sensors.render();
    },

    visualizeResults: function() {
        this.resultsVisualized = !this.resultsVisualized;
        if (this.resultsVisualized) {
            this.resultsView.visualizeResults();
        } else {
            this.resultsView.hideVisualizations();
        }
    }
});

// experimental view on the Search page. Shows the results of selected sensors and their properties (date, attributes, etc.)
app.view.ResultsView = Backbone.View.extend({

    id: "searchResults",

    initialize: function(options) {
        this.geostreamService = options.geostreamService;
        this.listenTo(app.collection.sensors, 'change:checked', this.render);
        this.listenTo(app.collection.parameters, 'change:checked', this.render);
        this.listenTo(app.collection.parameters, 'change:sourceChecked', this.render);
        this.listenTo(app.collection.sources, 'change:checked', this.render);

        // load sparkline visualization
        this.sparkline = app.graphs.sparkline().height('100px').width('175px');

        this.timeSeries = app.graphs.timeSeries().height('100px').width('175px').max(15).padding("30 10 50 60");

        this.parametersSelected = [];

        this.checked = {sensors: [], parameters: [], bounds: []};
    },

    render: function() {
        if (this.isRendering) return;
        this.isRendering = true;

        var checked = {sensors: [], parameters: [], bounds: []};

        checked.sensors = app.collection.sensors.getChecked();
        checked.sensors.forEach(function(sensor) {
            if (sensor.get('max_end_time')) {
                var max = moment(sensor.get('max_end_time'));
                if (checked.max === undefined || max > checked.max) checked.max = max;
            }
            if (sensor.get('min_start_time')) {
                var min = moment(sensor.get('min_start_time'));
                if (checked.min === undefined || min < checked.min) checked.min = min;
            }
        });

        var sourcesChecked = app.collection.sources.where({checked: true});
        var sourcesSelected = sourcesChecked;
        if ( (sourcesSelected && sourcesSelected.length == 0)) {
            sourcesSelected = app.collection.sources.where({selected: true});
        }

        var parametersChecked = app.collection.parameters.where({checked: true});
        parametersChecked = _.filter(parametersChecked, function(parameter) {
            return parameter.get("searchView") != "ignore";
        });
        var parametersSelected = parametersChecked;
        if ( parametersChecked && parametersChecked.length > 0) {
            parametersSelected = parametersChecked;
        } else if ( (parametersChecked && parametersChecked.length == 0) && (sourcesChecked && sourcesChecked.length == 0)) {
            parametersSelected = app.collection.parameters.where({selected: true});
        } else {
            parametersSelected = app.collection.parameters.where({sourceChecked: true});
        }
        parametersSelected = _.filter(parametersSelected, function(parameter) {
            return parameter.get("searchView") != "ignore";
        });

        var searchResultsDiv = $('#searchResults');
        searchResultsDiv.html('<div class="row search-results-row"><strong>Sites:</strong> ' + (app.collection.sensors.getChecked().length || app.collection.sensors.where({selected: true}).length) + '</div>');
        if (sourcesSelected.length >= 1) {
            searchResultsDiv.append('<div class="row search-results-row"><strong>' + app.config.sources.title + ':</strong>');
            _.each(sourcesSelected, function(source, index) {
                searchResultsDiv.append('<div class="badge">' + source.id + '</div>');
                if (index <= (sourcesSelected.length - 2)) { searchResultsDiv.append(' OR '); }
            });
            searchResultsDiv.append('</div>');
        }

        // show Measurements if we have some
        if (parametersSelected.length >= 1) {
            searchResultsDiv.append('<div class="row search-results-row"><strong>' + app.config.parameters.title + ':<strong> </div>').html();
            _.each(parametersSelected, function(parameter, index) {
                searchResultsDiv.append('<div class="badge">' + parameter.get("title") + '</div>').html();
                if (index <= (parametersSelected.length - 2)) { searchResultsDiv.append(' OR '); }
            });
        }

        this.parametersSelected = parametersSelected;
        this.checked = checked;

        // show date range if it exists
        if (checked.min && checked.max) {
            this.renderDates(checked.min, checked.max);
            app.event.eventBus.trigger('dates:new', {min: checked.min, max: checked.max});
        }

        this.hideVisualizations();

        this.isRendering = false;

    },

    renderDates: function(start, end) {
      var searchResultsDates = $('#search-results-dates');
      if (searchResultsDates.length) {
        searchResultsDates.html('<strong>Dates Selected:</strong> ' + start.format('MMMM Do YYYY') + ' - ' + end.format('MMMM Do YYYY')).html();
      } else {
        $('#searchResults').append('<div id="search-results-dates" class="row search-results-row"><strong>Dates Available:</strong> ' + start.format('MMMM Do YYYY') + ' - ' + end.format('MMMM Do YYYY') + '</div>').html();
      }

    },

    visualizeResults: function() {

        var that = this;

        // draw graphs for each parameter and station
        if(this.parametersSelected.length > 0) {

            var searchResultsVisualizationsDiv = $('#searchResultsVisualizations');

            searchResultsVisualizationsDiv.append('<div><h3>Visualizations</h3></div>').html();

            _.each(this.parametersSelected, function(parameter) {


                searchResultsVisualizationsDiv.append('<div><strong>' + parameter.get("title") + '</strong></div>').html();

                // place divs on the page for all the sensors/parameters matches
                _.each(that.checked.sensors, function(sensor) {
                    if (_.contains(parameter.get("sensors").pluck("id"), sensor.get("id"))) {
                        searchResultsVisualizationsDiv.append('<div class="searchResultStation">' +
                                                                '<div id="vis_' + sensor.get("id") +  '_' + parameter.get("id") + '"></div>').html();
                    }
                });

                // retrieve data by sensor and draw graphs
                _.each(that.checked.sensors, function(sensor) {

                  // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
                  // are averaged before being drawn in the graphs.
                  // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
                  // as an argument. From there, you can return 'smart', 'semi', or some other binning value
                  // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
                  // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
                  var time = "";
                  if (app.config.binning.filterFunction) {
                    time = app.config.binning.filterFunction(sensor);
                  } else if (app.config.binning.time) {
                    time = app.config.binning.time;
                  } else {
                    time = "smart";
                  }

                  // If app.config.binning.time is 'smart', then determine aggregation by temporal range
                  if (time == 'smart') {
                    time = getAggregationBin(new Date(sensor.get('min_start_time')), new Date(sensor.get('max_end_time')));
                  }

                  // bin depth in one by of 1000 meters below the surface and by year
                    that.geostreamService.getBinnedData(time, 1000, sensor.get("id"), false, function(data) {
                        // console.log("Got data for " + sensor.get("id"));
                        // console.log(data);

                    if (_.contains(parameter.get("sensors").pluck("id"), sensor.get("id"))) {

                        console.log("Drawning graph for sensor " + sensor.get("id") + " and parameter " + parameter.get("id"));


                        var dataBySensor;
                        dataBySensor = data[0].properties[parameter.get("id")];
                        if (!dataBySensor) {
                          while(!dataBySensor) {
                            _.each(parameter.get('alternates'), function(alternate) {
                              dataBySensor = data[0].properties[alternate];
                            });
                          }
                        }

                        // order by year
                        // dataBySensor = _.sortBy(dataBySensor, function(obj) { return obj.year });

                        var margin = {top: 20, right: 5, bottom: 20, left: 30},
                            width = 175 - margin.left - margin.right,
                            height = 100 - margin.top - margin.bottom,
                            chart_id = "#vis_" + sensor.get("id") +  '_' + parameter.get("id");

                        // X scale 
                        var x = d3.time.scale()
                            .domain(d3.extent(dataBySensor, function(d) { return new Date(d.date); }))
                            .range([0, width]); 

                        // Y scale 
                        var y = d3.scale.linear()
                            .domain(d3.extent(dataBySensor, function(d) { return d.average; }))
                            .range([height, 0]);

                        // x-axis
                        var xAxis = d3.svg.axis()
                            .scale(x)
                            .ticks(3)
                            .orient("bottom");

                        // y-axis
                        var yAxis = d3.svg.axis()
                            .scale(y)
                            .ticks(2)
                            .orient("left");

                        // main svg element
                        var graph = d3.select(chart_id)
                            .append("svg:svg").attr("width", width + margin.left + margin.right).attr("height", height + margin.top + margin.bottom)
                            .append("g")
                            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

                        // draw x-axis
                        graph.append("g")
                            .attr("class", "x axis")
                            .attr("transform", "translate(0," + height + ")")
                            .call(xAxis);

                        // draw y-axis
                        graph.append("g")
                            .attr("class", "y axis")
                            .call(yAxis)

                        // line object
                        var line = d3.svg.line()
                            .x(function(d) { return x(new Date(d.date)); })
                            .y(function(d) { return y(d.average); });
    
                        // path with data
                        graph.append("svg:path")
                            .datum(dataBySensor)
                            .attr("d", line)
                            .attr("class", "sparklines");

                        // link to detail page
                        graph
                            .append("svg:a").attr("xlink:href", function(d){ return window.location.origin + window.location.pathname + "#detail/location/" + sensor.get("name") + "/"})
                            .append("svg:text")
                            .text(function(d) { return sensor.get("name"); })
                            .attr("y", - 5)
                            .attr("x", - margin.left)
                            .attr("text-anchor", "start");

                    }

                });

                });
            });
        }
    },

    hideVisualizations: function() {
        $('#searchResultsVisualizations').empty();
    }

});

app.view.AboutView = Backbone.View.extend({
    id: "about-view",
    tag: "div",
    
    template: _.template($('#about-template').html()),

    initialize: function(options) {
        return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template({sources: app.collection.sources.getSourcesWithSensors().toJSON()}));
        return this;
    }
});
app.view.CompareView = Backbone.View.extend({
    id: "compare-view",
    tag: "div",

    template: _.template($('#compare-template').html()),

    events: {
        'change #compareDropdownList': 'renderSubViews'
    },

    initialize: function() {
        var self = this;
        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        this.collection.on('add', this.renderComparisonDescription, this);
        this.collection.on('remove', this.renderComparisonDescription, this);

        _.bindAll(this, "render", "renderSubViews");
        // define the subviews that appear on the CompareView
        // this sets up the relationship between the views for the controls and the graphs
        // and hands off to renderSubViews (below)
        if (app.config.views && app.config.views.compare) {
            this.subViews = app.config.views.compare.subviews;
        }

        app.view.compareSidebarView = new app.view.CompareSidebarView({
            // here we are passing an array of strings to the CompareSidebarView so it can display
            // a dropdown of our choices
            el: "#compare-sidebar",
            items: _.map(this.subViews, function(subview) { return subview.label; }),
            views: this.subViews
        });

        app.event.eventBus.on("sensors:refresh", function(){
            self.render();
        });

        return this;
    },

    render: function() {
        this.$el.html(this.template());
        app.view.compareSidebarView.setElement(this.$('#compare-sidebar')).render();
        this.renderSubViews();

        return this;
    },

    renderSubViews: function(view) {
        // the first time renderSubViews is called, there will be no view (view is actually an "event" sent by Backbone)
        // when a user clicks the dropdown, renderSubViews will contain the freeform text of the desired view (ie: "Lakes")
        // we match the text from the dropdown back to the label in this.subViews and render the appropriate subviews
        if (view && view.currentTarget) {
            // in this case, a view was passed, so map the text to an item in this.subViews
            // TODO: Check this. The tabViews should show up here.
            view = _.findWhere(this.subViews, {label: view.currentTarget.value });
        } else {
            // in this case, no view was passed, so just get the first item in this.subViews with a label
            view = _.find(this.subViews, function(subview) { return subview.label; });
        }

        // the variable "view" should now contain a text label and the correct templates for the sidebar and the graph

        // we don't have to empty $el, since that is handled by CompareSidebarControlsView
        // the initialize method in CompareSidebarControlsView calls render for us
        // CompareSidebarControlsView will insert this into div#compare-sidebar-controls
        // TODO: Check the multiple subview rendering here.
        if (!app.view["compareSidebarControlsView" + view.label]) {
            app.view["compareSidebarControlsView" + view.label] = new app.view.CompareSidebarControlsView({
                template: view.sidebar.template,
                label: view.label,
                clickableDOMItems: view.sidebar.clickableDOMItems
            });
        }
        this.$('#compare-sidebar-controls').append(app.view["compareSidebarControlsView" + view.label].render().el);


        // we don't have to empty $el, since that is handled by CompareGraphView
        // the initialize method in CompareGraphView calls render for us
        // CompareGraphView will insert this into div#compare-graph
        if (!app.view["compareGraphView" + view.label]) {
            app.view["compareGraphView" + view.label] = new app.view.CompareGraphView({
                template: view.graph.template
            });
        }
        app.view["compareGraphView" + view.label].setElement(this.$('#compare-graph')).render();
    },

    renderComparisonDescription: function() {
        var selectedSensors = app.collection.compareSelectedSensors.map(function(sensor) {
            return sensor.get('name');
        });

        var comparisonDescription = $('#comparison-description');
        var comparisonDescriptionItems = $('#comparison-description-items');

         if (selectedSensors.length == 0 ) {
            comparisonDescription.text("Compare Parameters");
            comparisonDescriptionItems.text('');
         } else if (selectedSensors.length > 0) {
            comparisonDescriptionItems.text(selectedSensors.join(" & "));
         }

         if (selectedSensors.length == 1) {
            comparisonDescription.text("Showing Parameters for ");
         } else if (selectedSensors.length > 1) {
            comparisonDescription.text("Comparing Parameters at ");
         }

         if (selectedSensors.length > 3) {
            comparisonDescriptionItems.addClass("small");
         } else {
           comparisonDescriptionItems.removeClass("small");
         }
    }
});

app.view.CompareSidebarView = Backbone.View.extend({

    template: _.template($('#compare-sidebar-template').html()),

    initialize: function(options) {
        var self = this;
        _.bindAll(this, "render", "renderMap");
        this.items = options.items;
        this.miniMap = app.config.views.compare.miniMap;

        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        this.collection.on('add', this.renderMap, this);
        this.collection.on('remove', this.renderMap, this);

        app.collection.onHoverSensors = app.collection.onHoverSensors || new Backbone.Collection();
        this.onHoverSensors = app.collection.onHoverSensors;
        this.onHoverSensors.on('add', this.renderMap, this);
        this.onHoverSensors.on('remove', this.renderMap, this);

        return this;
    },

    render: function() {
        this.$el.html( this.template() );
        this.map = L.map(self.$('#compare-sidebar-minimap')[0], {
            zoomControl: false,
            attributionControl: false,
            dragging: true,
            touchZoom: false,
            scrollWheelZoom: true,
            doubleClickZoom: false,
            boxZoom: false
        }).setView(this.miniMap.config.mapOptions.center, this.miniMap.config.mapOptions.zoom);
        this.renderMap();
        return this;
    },

    renderMap: function() {
      var self = this;

      var activeSensors = (app.collection.compareSelectedSensors) ? app.collection.compareSelectedSensors.toJSON() : [];

      if (self.geoJsonLayer) {
        self.map.removeLayer(self.geoJsonLayer);
      }

      var onHoverSensors = (app.collection.onHoverSensors) ? app.collection.onHoverSensors.toJSON() : [];
      if(self.onHoverLayer) {
        self.map.removeLayer(self.onHoverLayer)
      }
      self.onHoverLayer = L.geoJson(onHoverSensors, {
        pointToLayer: function(feature, latlng) {
          var badge = L.divIcon({className: 'badger-outer', html: '<div class="badger-inner-hover"><p class="badger-map-badge"> * </p></div>'});
          return L.marker(latlng, {icon: badge})
        }
      });

      self.onHoverLayer.addTo(self.map);
      self.geoJsonLayer = L.geoJson(activeSensors, {
        pointToLayer: function (feature, latlng) {
          var badge = L.divIcon({className: 'badger-outer', html: '<div class="badger-inner"><p class="badger-map-badge">'+ ( _.indexOf(activeSensors, feature) + 1) + '</p></div>'});
          return L.marker(latlng, {icon: badge});
        }
      });

      self.geoJsonLayer.addTo(self.map);

      

      L.tileLayer(self.miniMap.config.mapOptions.mapTileSource, {
          maxZoom: 18
      }).addTo(self.map);

      self.$("#" + this.miniMap.id).html(self.map.el);

    }
});

app.view.CompareSidebarControlsView = Backbone.View.extend({
    el: "div#compare-sidebar-controls",
    // required arguments:
    // template: the script id for the template this will use
    // label: a string or variable that resolves to a string that is used to show in the dropdown
    // ie: new app.view.CompareByTypeView({
    //      template: "#compare-sidebar-lakes-template
    //      label: app.config.areas.title
    // });
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        this.label = options.label;
        this.clickableDOMItems = options.clickableDOMItems;
        this.template = _.template($(options.template).html());
        // unbind click prevents the duplicate "clicking" of the event
        this.$el.unbind('click');

        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        app.collection.onHoverSensors = app.collection.onHoverSensors || new Backbone.Collection();
        
        // this.collection.on('add', this.render, this);
        // this.collection.on('remove', this.render, this);

        return this;
    },

    render: function() {
        this.$el.html(
            this.template({
                label: this.label,
                sources: app.collection.sources.getAreas(),
                activeItems: app.collection.compareSelectedSensors.pluck('id')
            })
        );

        // we could have setup the defaults "events" object, but this allows us to store in app.config what we want to be clickable
        this.$(this.clickableDOMItems).on('click', this.clickItem);
        this.$(this.clickableDOMItems).on('mouseenter', this.mouseEnter);
        this.$(this.clickableDOMItems).on('mouseleave', this.mouseLeave);
        this.$('.nodata-wrapper').tooltip({delay: { show: 150, hide: 100 }});
        this.$('.showHide').on('click', this.toggleButton);
        return this;

    },

    toggleButton: function() {
        if(this.innerHTML.indexOf("+") >=0) {
            this.innerHTML= "-";
        } else {
            this.innerHTML="+";
        }
    },
    mouseEnter: function() {
        event.preventDefault();
        
        //The id of the sensor
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});
        app.collection.onHoverSensors.add(sensor);
        var clickedOrder = _.indexOf(app.collection.onHoverSensors.models, sensor);
        if (clickedOrder > -1) {
          $(this).append('<div class="badger-outer-hover" id="Badger_'+clickedOrder+'"><div class="badger-inner-hover"><p class="badger-badge" id="Badger_'+clickedOrder+'_badge">*</p></div></div>');
        } 
    },
    mouseLeave: function() {
        event.preventDefault();
        $(this).find('.badger-outer-hover').remove();
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});
        app.collection.onHoverSensors.remove(sensor); 
    },

    clickItem: function(event) {
        // preventDefault ensures that the button doesn't do its default (ie: we control what it should do)
        event.preventDefault();

        // start the loading spinner
        app.event.eventBus.trigger("compare:spinner:start");

        // if the sensor is already active when this click begins, this is an "unclick"
        if ($(this).hasClass('active')) {
          $(this).find('.badger-outer').remove();
        }
        
        $(this).toggleClass('active');
        
        // the name of the sensor:
        // console.log(event.currentTarget.value);

        // the id of the sensor (in case we'd want to directly query the datapoints endpoint later:
        // console.log(event.currentTarget.getAttribute('data-id'));
        // or it's jQuery equivalent:
        // console.log($(event.target).data('id'));
        var sensor = app.collection.sensors.findWhere({name: event.currentTarget.value});

        if (sensor) {
            // if the clicked sensor was already selected, we will remove it immediately
            if (app.collection.compareSelectedSensors.contains(sensor)) {
                app.collection.compareSelectedSensors.remove(sensor);

                // now we need to determine if we can remove any of this sensor's parameters from the selection
                // other sensors might still be selected and they might offer the same parameters as this sensor
                // we will look at the remaining sensors to determine parameters that should still be selected
                var remainingParameters = [];
                app.collection.compareSelectedSensors.each(function(s){
                    s.get('parameters').each(function(p) {
                        remainingParameters.push(p.id);
                    });
                });

                remainingParameters = _.uniq(remainingParameters);

                // if there are no more parameters available,
                // then we can infer that all of the sensors have been unclicked, so reset everything
                if (!remainingParameters.length) {
                    app.collection.parameterBins.reset();
                    app.collection.compareSelectedDates.reset();
                    app.collection.compareSelectedSensors.reset();
                    app.collection.compareSelectedParameters.reset();
                } else {
                    // in this case, we have unchecked a sensor, but there are other parameters available,
                    // so we need to remove only the parameters that were unique to this sensor
                    sensor.get('parameters').each(function(parameter){
                        if (!_.contains(remainingParameters, parameter.get('id'))) {
                            app.collection.compareSelectedParameters.remove(parameter);
                        }
                    });
                }

                // stop the loading spinner
                app.event.eventBus.trigger("compare:spinner:stop");
                return this;
            } else {
                // the user is clicking this sensor for the first time
                app.collection.compareSelectedSensors.add(sensor);
            }
        }

        // this.binningTime will be something like 'smart', 'semi', 'yearly', etc. It is how the datapoints
        // are averaged before being drawn in the graphs.
        // Projects can setup a filterFunction that accepts the current sensor (the Backbone Sensor Model)
        // as an argument. From there, you can return 'smart', 'semi', or some other binning value
        // based on a condition. For example: return (sensor.get('type')['id'] == 'epa') ? 'semi' : 'smart';
        // would return 'semi' as the binningTime for epa sensors, all others would use 'smart' binning
        this.binningTime = "";
        if (app.config.binning.filterFunction) {
          this.binningTime = app.config.binning.filterFunction(app.collection.sensors.findWhere({name: event.currentTarget.value}));
        } else if (app.config.binning.time) {
          this.binningTime = app.config.binning.time;
        } else {
          this.binningTime = 'smart';
        }

        // If this.binningTime is 'smart', then determine aggregation by temporal range
        if (this.binningTime == 'smart') {
            // getAggregationBin is a global function in DetailView - that should probably change, but it works ok
            // this will just tell us if we should bin by year, month, day, etc.
            this.binningTime = getAggregationBin(new Date(sensor.get('min_start_time')),
              new Date(sensor.get('max_end_time')));
        }

        // request the bin endpoint
        app.model.geoStreamService.getBinnedData(this.binningTime, 1, sensor.get('id'), true, function(data) {
            // the bin endpoint will return a bunch of parameters in the "properties" object
            // each parameter uses an "id" format, but it can also be an original formatted string, as it came
            // from the source, so it might be one of the alternate spellings found in the "alternates"
            // array for a parameter
            _.each(data[0]["properties"], function(values, parameterID) {

                // if the parameter contains data...
                if (data[0].properties[parameterID].length) {

                    // search for this parameter against the collection by id
                    var parameterModel = app.collection.parameters.findWhere({id: parameterID});

                    // if it wasn't found by id, check to see if it matches one of the parameters alternate spellings
                    // if no match is found, we can go on to the next parameter
                    if (_.isUndefined(parameterModel)) {
                        parameterModel = app.collection.parameters.filter(function(p) {
                            return _.contains(p.get('alternates'), parameterID);
                        });
                        if (parameterModel.length) {
                            parameterModel = parameterModel[0];
                        } else {
                            console.log("CompareSidebarControlsView:clickItem - parameterModel was not found for " + parameterID);
                            return;
                        }
                    }

                    // make sure we aren't using a parameter that is either ignored or a depth-based parameter
                    if (parameterModel.get('detailView') == 'time') {
                        // adding to the compareSelectedParameters collection will trigger an event internally
                        // this collection is listened to by views and will take care of updating lists or graphs
                        // whenever this changes
                        app.collection.compareSelectedParameters.add(parameterModel.toJSON());
                    }

                    // a parameterBin is a way of organizing data so that we can compare multiple
                    // sensors for the same parameter. Since these are two separate calls to the API, we persist
                    // them in a collection. A parameterBin is organized like this:
                    //   id: the parameter id. If an alternate was used, we conform it to the ID above as parameterModel
                    //   dates: a Backbone Collection.
                    //         - each model in this collection uses the unique date string as it's ID
                    //         - each sensor that has a parameter on this date will be an attribute on this model.
                    //         - ie: sensor 1 will show up as an attribute: "1: 0.5", where 1 is the sensor's ID
                    //               and 0.5 is the value returned by the bin endpoint for this parameter and parameter

                    // the "add" method on a collection won't duplicate a unique ID.
                    var parameterBin = app.collection.parameterBins.add({id: parameterModel.get('id')});
                    // create the dates collection if it doesn't already exist
                    if (!parameterBin.get('dates')) {
                        parameterBin.set('dates', new Backbone.Collection());
                    }

                    // inside the response from the bin endpoint, this parameter will contain several values and metadata
                    // we are interested in the "date" and "average" values
                    for (var i = 0; i < data[0]["properties"][parameterID].length; i++) {
                        var parameterBinDatesCollection = parameterBin.get('dates');

                        // add this date as a unique ID in the parameterBin's "dates" collection
                        var selectedDate = parameterBinDatesCollection.add({
                            id: data[0]["properties"][parameterID][i]['date']
                        });

                        // also add this to the compareSelectedDates collection
                        // so that our slider range gets updated
                        app.collection.compareSelectedDates.add({id: data[0]["properties"][parameterID][i]['date']});

                        // set the value of the "average" on this date: {1: 0.5}
                        // any additional sensors with measurements of this parameter at this time will get added here
                        selectedDate.set(sensor.get('id'), data[0]["properties"][parameterID][i]['average']);
                    }

                }
            });
            // if we listen directly to the parameterBins collection, it would fire events too aggressively, even before
            // we are done assessing the data. Instead, wait to notify until after all of the parameters have been parsed above.
            app.event.eventBus.trigger("parameterBins:loaded");

            // stop the loading spinner
            app.event.eventBus.trigger("compare:spinner:stop");
        });

        var clickedOrder = _.indexOf(app.collection.compareSelectedSensors.models, sensor);
        if (clickedOrder > -1) {
          clickedOrder++;
          $(this).append('<div class="badger-outer" id="Badger_'+clickedOrder+'"><div class="badger-inner"><p class="badger-badge" id="Badger_'+clickedOrder+'_badge">'+clickedOrder+'</p></div></div>');
        }
    }
});

app.view.CompareGraphView = Backbone.View.extend({
    className: "compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        this.template = _.template($(options.template).html());

        // this view will listen to the compareSelectedSensors collection
        app.collection.compareSelectedSensors = app.collection.compareSelectedSensors || new Backbone.Collection();
        this.collection = app.collection.compareSelectedSensors;

        // create the three parameter dropdowns
        // they will listen to changes in the compareSelectedParameters collection
        this.parameterDropdown1 = new app.view.SelectCollectionView({
          id: "compare-parameter-1",
          className: "span4 margin21R",
          trigger: "change:compare:parameter:1",
          collection: app.collection.compareSelectedParameters
        });
        this.parameterDropdown2 = new app.view.SelectCollectionView({
          id: "compare-parameter-2",
          className: "span4 margin21R",
          trigger: "change:compare:parameter:2",
          placeholder: "Additional Parameter",
          collection: app.collection.compareSelectedParameters
        });
        this.parameterDropdown3 = new app.view.SelectCollectionView({
          id: "compare-parameter-3",
          className: "span4",
          trigger: "change:compare:parameter:3",
          placeholder: "Additional Parameter",
          collection: app.collection.compareSelectedParameters
        });

        // when a parameter changes, update the corresponding graph
        app.event.eventBus.on('change:compare:parameter:1', function(data) {
          if (data && data.target && data.target.value) {
            self.graph1.update(data.target.value);
          }
        });
        app.event.eventBus.on('change:compare:parameter:2', function(data) {
            if (data && data.target && data.target.value) {
              self.graph2.update(data.target.value);
            }
        });
        app.event.eventBus.on('change:compare:parameter:3', function(data) {
            if (data && data.target && data.target.value) {
              self.graph3.update(data.target.value);
            }
        });

        // when a sensor is unclicked, update each graph based on the selected parameter
        this.collection.on('remove', function() {
          self.graph1.update($("#compare-parameter-1").val());
          self.graph2.update($("#compare-parameter-2").val());
          self.graph3.update($("#compare-parameter-3").val());
        }, this);


        // on a pristine page, if you click a sensor, this will load the graph for the first parameter
        app.event.eventBus.on("parameterBins:loaded", function() {
          self.graph1.update($("#compare-parameter-1").val());
          self.graph2.update($("#compare-parameter-2").val());
          self.graph3.update($("#compare-parameter-3").val());
        });

        // setup a loading spinner we can turn on and off
        var spinner = app.config.views.compare.spinnerOptions || {
            lines: 13 // The number of lines to draw
            , length: 28 // The length of each line
            , width: 14 // The line thickness
            , radius: 42 // The radius of the inner circle
            , scale: 1 // Scales overall size of the spinner
            , corners: 1 // Corner roundness (0..1)
            , color: '#000' // #rgb or #rrggbb or array of colors
            , opacity: 0.25 // Opacity of the lines
            , rotate: 0 // The rotation offset
            , direction: 1 // 1: clockwise, -1: counterclockwise
            , speed: 1 // Rounds per second
            , trail: 60 // Afterglow percentage
            , fps: 20 // Frames per second when using setTimeout() as a fallback for CSS
            , zIndex: 2e9 // The z-index (defaults to 2000000000)
            , className: 'spinner' // The CSS class to assign to the spinner
            , top: '60%' // Top position relative to parent
            , left: '57%' // Left position relative to parent
            , shadow: false // Whether to render a shadow
            , hwaccel: false // Whether to use hardware acceleration
            , position: 'absolute'
          };
        this.spinner = new Spinner(spinner);

        // the spinner isn't appended to the DOM until the first time it gets called.
        app.event.eventBus.on("compare:spinner:start", function() {
          $("#compare-parameter-graph-1").append(self.spinner.spin().el);
        });
        // use the eventBus to stop the spinner from anywhere
        app.event.eventBus.on("compare:spinner:stop", function() {
          self.spinner.stop();
        });

        // compareSelectedDates keeps track of the date range available to the graphs
        // whenever those dates change, update the date slider
        app.collection.compareSelectedDates.on('all', function() {
          var slider = $("#slider-range-stations");
          var minBounds = new Date(new Date().getFullYear() - 44, new Date().getMonth(), new Date().getDate());
          var minSelected = new Date(new Date().getFullYear() - 10, new Date().getMonth(), new Date().getDate());
          var last = new Date();
          // if compareSelectedDates has models, then we have data to look at and graph.
          // set the bounds to that of the compareSelectedDates collection and move the selection to the bounds too
          if (app.collection.compareSelectedDates.length) {
            minBounds = minSelected = new Date(app.collection.compareSelectedDates.first().get('id'));
            last = new Date(app.collection.compareSelectedDates.last().get('id'));
          }
          // bounds: the range of available dates
          slider.dateRangeSlider("bounds", minBounds, last);
          // values: the selected start and end times
          slider.dateRangeSlider("values", minSelected, last);

        });

        return this;
    },

    render: function() {
        this.$el.empty();
        this.$el.append( this.template() );
        if (app.config.views.compare.removeBinDropdown) {
            this.$('#compare-season-dropdown').remove();
        }
        this.renderParameterDropdowns(this);

        // create a dateRangeSlider, setting the bounds fairly large, and selecting the last year.
        // once data starts getting selected, this will get udpated
        var sliderRange = $( "#slider-range-stations" );
        sliderRange.dateRangeSlider({
          bounds: {
            min: new Date(new Date().getFullYear() - 44, new Date().getMonth(), new Date().getDate()),
            max: new Date()
          },
          defaultValues: {
            min: new Date(new Date().getFullYear() - 10, new Date().getMonth(), new Date().getDate()),
            max: new Date()
          },
          step: {
            days: 1
          },
          arrows: false
        });

        // when the slider range changes, data contains data.values.min and data.values.max
        sliderRange.on("userValuesChanged", function(e, data) {
          app.event.eventBus.trigger("compare:date:changed", data);
        });

        // create the three graphs in their empty state and render them
        this.graph1 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-1"});
        this.graph2 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-2"});
        this.graph3 = new app.view.CompareGraphMGTimeSeriesView({id: "#compare-parameter-graph-3"});
        this.graph1.render();
        this.graph2.render();
        this.graph3.render();

        return this;
    },

    renderParameterDropdowns: function(self) {
        var container = this.$('#availableParametersDefaultTitle');

        container.append(self.parameterDropdown1.render().el);
        container.append(self.parameterDropdown2.render().el);
        container.append(self.parameterDropdown3.render().el);

        self.$('#compare-parameter-1').on('change', function(event) {
          self.parameterDropdown1.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:1', event);
        });
        self.$('#compare-parameter-2').on('change', function(event) {
          self.parameterDropdown2.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:2', event);
        });
        self.$('#compare-parameter-3').on('change', function(event) {
          self.parameterDropdown3.setSelected(event.target.value);
          app.event.eventBus.trigger('change:compare:parameter:3', event);
        });
      return self;

    }
});

app.view.CompareGraphTimeSeriesView = Backbone.View.extend({
//    el: "div#compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        this.template = _.template('<h4><%= name %></h4>');
        _.bindAll(this, "render", "setData", "getData");
        this.timeSeriesView = new app.view.TimeSeriesView({
            parentView : this
        });
        app.event.eventBus.on("compareGraph:data:" + this.model.id, this.setData);
        app.event.eventBus.on("change:compare:parameter", function(event) {
            self.setData(self.model, event);
        });

        $("#compare-hide-box-plot").on('click', function() {
            $(".box-whisker-chart").toggle();
        });

		    $("#compare-season-dropdown").on('change', function() {
			      self.setData(self.model, null, $(this).val());
		    });

        $("#slider-range-stations").on('slidechange', function() {
            self.setData(self.model);
        });


        return this;
    },

    render: function() {

        this.$el.empty();
        this.$el.html( this.template({name: this.model.get('popupContent') }) );
        this.getData(this.model);

        this.modals = _.template($("#box-whisker-modal-template").html())();

        this.$el.append(this.modals);
        return this;
    },

    append: function() {
      this.$el.append(this.timeSeriesView.render().el);

    },

    getData: function(model) {
        var self = this;
        if (!self.data) {
          app.model.geoStreamService.getByLocation(model.toJSON(), function(data) {
              self.data = data;
              app.event.eventBus.trigger("compareGraph:data:" + model.id, model);
          });
        } else {
          app.event.eventBus.trigger("compareGraph:data:" + model.id, model);
        }
    },

    setData: function(model, parameter, season) {
        season = season || $("#compare-season-dropdown").val();
        var self = this;
        if (model.id != this.model.id) return;
        if (!this.data) return;

        var parameter1 = $('.compare-parameter-1').val();
        var parameter2 = $('.compare-parameter-2').val();
		var parameter3 = $('.compare-parameter-3').val();

        var parameters = [];
        _.each([parameter1, parameter2, parameter3], function(parameter) {
          if (parameter && parameter.length) {
            parameters.push(parameter);
          }
        });

	    var startTime = new Date($("#slider-range-stations").slider('values')[0], null);
        var endTime = new Date($("#slider-range-stations").slider('values')[1], 11, 31, 23, 59, 59);

        // create time series values
        // 'data' is the original data from the API. Here we're just filtering it down to just the data with "detailView" of "time"
        var time_series_data = _.where(this.data, {detailView: "time"});
        if (time_series_data) {
            // convert data to following data structure. We need two
            // of these structures in summer, one with all values for
            // box-whisker plot and one with only EP for time series.
            // time-series data: [ {
            //   key: "0",
            //   values: [ {
            //     color: "red",
            //     key: "parameter name",
            //     sources: [ sources ],
            //     values: [ {
            //       x: date,
            //       y: value
            //     }]
            //   }]
            // }]

            // get a unique list of the parameters found in the time data
            var time_series_params = _.unique(_.pluck(_.where(time_series_data, {detailView: "time"}), "parameter"));

            // if a parameter is selected, but this sensor doesn't have that parameter, omit it
            _.each(parameters, function(parameter) {
                if (!_.contains(time_series_params, parameter)) {
                  parameters = _.without(parameters, parameter);
                }
            });

            var parameterTitles = app.collection.parameters.where({detailView: "time"});
            parameterTitles = _.pluck(parameterTitles, "id");
            // use d3 to create colors for each of the unique parameters
            var parameters_colors = parameterTitles.length > 10 ? d3.scale.category20().domain(parameterTitles) : d3.scale.category10().domain(parameterTitles);

            var time_season_data = this.data;
            if (!_.isUndefined(season) && (season != null) && (season != "all") && (season != "")) {
                time_season_data = _.where(this.data, {season: season});
            }

            // alldata is where we'll stuff all the time_series_data as we iterate through the time_series_params
            // summerepdata is where we'll add all spring data, but only summer ep data
            var alldata = [], summerEPdata = [];

            // sources is where a list of sources are kept, pointing back to clowder
            var sources = [];


          // if no start or endtime is given compute start/end time
            if (_.isUndefined(startTime) || _.isUndefined(endTime)) {
                _.each(this.data, function(d) {
                    if (_.isUndefined(startTime) || (startTime > d.date)) startTime = d.date;
                    if (_.isUndefined(endTime) || (endTime < d.date)) endTime = d.date;
                });
            }


            var filterWrongDates = function(dataset, startTime, endTime) {
                // filter any wrong date information
                if (!_.isUndefined(startTime) || !_.isUndefined(endTime)) {
                    dataset = _.filter(dataset, function (x) {
                        if (!_.isUndefined(startTime)) {
                            if (startTime.getUTCFullYear() > x.year) {
                                return false;
                            }
                            if (startTime.getUTCFullYear() == x.year) {
                                if (!_.isUndefined(x.month)) {
                                    if (startTime.getUTCMonth() > x.month) {
                                        return false;
                                    }
                                    if (startTime.getUTCMonth() == x.month) {
                                        if (!_.isUndefined(x.day)) {
                                            if (startTime.getUTCDate() > x.day) {
                                                return false
                                            }
                                            if (startTime.getUTCDate() == x.day) {
                                                if (!_.isUndefined(x.hour)) {
                                                    if (startTime.getUTCHours() > x.hour) {
                                                        return false;
                                                    }
                                                    if (startTime.getUTCHours() == x.hour) {
                                                        if (!_.isUndefined(x.minute) && startTime.getUTCMinutes() > x.minute) {
                                                            return false;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        if (!_.isUndefined(endTime)) {
                            if (endTime.getUTCFullYear() < x.year) {
                                return false;
                            }
                            if (endTime.getUTCFullYear() == x.year) {
                                if (!_.isUndefined(x.month)) {
                                    if (endTime.getUTCMonth() < x.month) {
                                        return false;
                                    }
                                    if (endTime.getUTCMonth() == x.month) {
                                        if (!_.isUndefined(x.day)) {
                                            if (endTime.getUTCDate() < x.day) {
                                                return false
                                            }
                                            if (endTime.getUTCDate() == x.day) {
                                                if (!_.isUndefined(x.hour)) {
                                                    if (endTime.getUTCHours() < x.hour) {
                                                        return false;
                                                    }
                                                    if (endTime.getUTCHours() == x.hour) {
                                                        if (!_.isUndefined(x.minute) && endTime.getUTCMinutes() < x.minute) {
                                                            return false;
                                                        }
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        return true;
                    });
                }
                return [dataset, startTime, endTime];
            };

          var data_filtered_for_dates = filterWrongDates(time_season_data, startTime, endTime);
          startTime = data_filtered_for_dates[1];
          endTime = data_filtered_for_dates[2];
          data_filtered_for_dates = data_filtered_for_dates[0];

          // iterate over each of the unique parameters
            var foundEP = false;
            for (var i = 0; i < parameters.length; i++) {
                // 'parameters' is what gets passed in via the URL (ie: the Backbone router gives us these)
                // if there are parameters in the URL, but the param we are on isn't one of them, skip it
//                if (!_.isEmpty(parameters) && (_.contains(parameters, time_series_params[i]))) {
//                    parameters = [time_series_params[i]];
//                }

                // create a 'valuesAll' object. It will filter the time_series_data to get this one param we are on,
                // creating a sum and count of the number of datapoints so we can create an average. The same is done
                // for the 'valuesEPsummer' however in this case all values in the summer that have depthcode that is
                // NOT EP will be ignored.
                var valuesAll = [], valuesEPsummer = [];
                _.each(_.where(data_filtered_for_dates, {parameter: parameters[i]}), function (x) {
                    var countAll = 0, countEPdata = 0;
                    var sumAll = 0, sumEPdata = 0;
                    var rawAll = [], rawEP = [];
                    _.each(x.values, function (v) {
                        countAll += v.count;
                        sumAll += v.value;
                        rawAll.push(v.raw);
                        sources.push(v.source);
                        if (x.season == "summer" && !_.isUndefined(v.depthcode) && (v.depthcode == "EP")) foundEP = true;
                        if ((x.season != "summer") || _.isUndefined(v.depthcode) || (v.depthcode == "EP") || (v.depthcode == "NA")) {
                            countEPdata += v.count;
                            sumEPdata += v.value;
                            rawEP.push(v.raw);
                        }
                    });
                    valuesAll.push({ x: x.date, y: sumAll / countAll, label: x.label, raw: _.flatten(rawAll) });
                    if (countEPdata != 0) {
                        valuesEPsummer.push({ x: x.date, y: sumEPdata / countEPdata, label: x.label, raw: _.flatten(rawEP) });
                    }
                });
                var yrange = d3.extent(valuesAll.concat(valuesEPsummer).concat({ y: 0}), function (d) {
                    return d.y;
                });
                var xrange = [startTime, endTime];

                // push the computed values into alldata/summerepdata
                alldata.push({
                    key: i.toString(),
                    values: [
                        {
                            color: parameters_colors(parameters[i]),
                            key: parameters[i],
                            xrange: xrange,
                            yrange: yrange,
                            values: valuesAll,
                            sources: _.uniq(_.flatten(sources)),
                            unit: ((app.collection.parameters.findWhere({id: parameters[i]})) ? (app.collection.parameters.findWhere({id: parameters[i]})).get("unit") : 'unit')
                        }
                    ]
                });
                if (!_.isEmpty(valuesEPsummer)) {
                    summerEPdata.push({
                        key: i.toString(),
                        values: [
                            {
                                color: parameters_colors(parameters[i]),
                                key: parameters[i],
                                xrange: xrange,
                                yrange: yrange,
                                values: valuesEPsummer,
                                unit: ((app.collection.parameters.findWhere({id: parameters[i]})) ? (app.collection.parameters.findWhere({id: parameters[i]})).get("unit") : 'unit')
                            }
                        ]
                    });
                }
            }

            // sends the formatted data to generate the graphs
            if ((season == "spring") || !foundEP) {
                self.timeSeriesView.setData(alldata, undefined);
            } else {
                self.timeSeriesView.setData(alldata, summerEPdata);
            }


            self.append();
//                self.controlsView.setParameters(parameters, _.isEmpty(parameters) ? time_series_params : parameters, time_series_params_colors);

        }

    }
});

app.view.CompareGraphMGTimeSeriesView = Backbone.View.extend({
//    el: "div#compare-graph",
    // template must be passed in when creating an instance of this view
    // ie: new app.view.CompareGraphView({template: "#compare-graph-lakes-template});
    // this view should be called after it's parent is rendered

    initialize: function(options) {
        var self = this;
        // the MG graph below will reference the "result" and "legend" classes to insert the graph
        this.template = _.template('<div class="result"></div><div class="legend"></div>');
        _.bindAll(this, "render");

        // default options to show an intentionally empty graph.
        // This indicates to the user that no selection has been made.
        this.graphOptions = {
            title: 'Parameter',
            width: 900,
            height: 400,
            target: this.id,
            missing_text: "Click a " + ((app.config.sensors.title) ? app.config.sensors.title.toLowerCase().substring(0, app.config.sensors.title.length - 1) : "sensor") + " to generate a graph" ,
            y_label: "Average",
            y_extended_ticks: true,
            x_accessor: 'date',
            legend_target: this.id + "-legend",
            chart_type: 'missing-data',
            aggregate_rollover: true
        };

        // this gets fired when the sliders are either moved by a user or reset by the app
        // if a user clicks on a sensor, then the dates get updated as we change the bounds
        app.event.eventBus.on("compare:date:changed", function(data) {
            // here we are calling update(parameterID, min, max)
            // since the eventBus doesn't pass in the value of this parameter, we need to get it from the DOM
            // this view's ID is "#compare-parameter-graph-1" (or 2 or 3), so we extract the number to create
            // "#compare-parameter-1" so we can get that value from the DOM
            self.update($("#compare-parameter-" + self.id.substr(self.id.length - 1, 1)).val(), data.values.min, data.values.max);
        });

        return this;
    },

    render: function() {
        // render is intentionally pretty sparse. This uses the MetricsGraphics library to call data_graphic.
        // all that does is inserts the graph into the DOM using the options we specified in the initialize function.
        // anytime we re-render this graph, it will default to this "no data" view.
        MG.data_graphic(this.graphOptions);

        return this;

    },

    update: function(parameterID, min, max) {
        // parameterID should be the ID of the parameterModel, but it can also be one of the alternate strings
        // min and max are date objects
        var self = this;
        var compareDateRangeSlider = $( "#slider-range-stations" );
        // if min or max isn't passed, get the values directly from the slider's built-in functions ("min", "max")
        min = min || compareDateRangeSlider.dateRangeSlider("min");
        max = max || compareDateRangeSlider.dateRangeSlider("max");

        // if we don't have a parameterID, then there is nothing to graph
        if (!parameterID) return self.render();

        // rendering begins here
        this.$el.html( this.template() );

        // find a model for this parameter. If ID doesn't find it, check the alternates, otherwise exit.
        var parameterModel = app.collection.parameters.findWhere({id: parameterID});
        if (_.isUndefined(parameterModel)) {
            parameterModel = app.collection.parameters.filter(function(p) {
                return _.contains(p.get('alternates'), parameterID);
            });
            if (parameterModel.length) {
                parameterModel = parameterModel[0];
            } else {
                return;
            }
        }

        // parameterBins are created when you click a sensor in CompareSidebarControlsView
        // each parameterBin uses the parameter ID as it's unique ID
        var parameterBin = app.collection.parameterBins.findWhere({id: parameterModel.get('id')});

        // we have the desired parameter, but not all sensors will have this parameter
        // if we don't filter out the sensors without the parameter, they will still be in the list
        // and they will show up in the legend, and may show the graph from the previously selected parameter
        var sensorsWithTheSelectedParameter = app.collection.compareSelectedSensors.filter(function(sensor) {
            return sensor.get('parameters').findWhere({id: parameterBin.get('id') });
        });

        // a parameterBin keeps track of all data for a parameter, from multiple sensors across a range of dates
        // we only want to graph sensors that are still selected, and have data for the selected parameter
        // so we will create an array of names and IDs that we can filter against below
        var selectedSensorNames = _.map(sensorsWithTheSelectedParameter, function(sensor) {
            return sensor.get('name');
        });

        var selectedSensorIDs = _.map(sensorsWithTheSelectedParameter, function(sensor) {
            return sensor.get('id');
        });

        // we will store the data from the parameterBin's dates collection in a local variable
        // these are all the values for each sensor, and we will format them before giving them to
        // MetricsGraphics to create the graph
        var data = [];
        if (!parameterBin || !parameterBin.get('dates')) {
            // if we never found a parameterBin for the parameter passed in, or it has no dates (ie: no data)
            // reset this graph and exit
            self.render();
            return this;
        }

        // the parameter will have a dates collection, but as the slider moves, we want to filter out those dates
        parameterBin.get('dates').each(function (bin) {
            var binDate = new Date(bin.get('id'));
            // only use dates if they are within the slider range
            if (min && max && ((min < binDate) && (binDate < max))) {

                app.collection.compareSelectedDates.add({id: bin.get('id')});

                var response = {"date": binDate};
                _.each(selectedSensorIDs, function(sensorID) {
                    if (bin.get(sensorID)) {
                        response[sensorID] = bin.get(sensorID);
                    }
                });
                if (response.date && (Object.keys(response).length > 1)) {
                    data.push(response);
                }
            }
        });

        var graphOptions = _.clone(self.graphOptions);
        if (data.length > 1) {
            graphOptions.title = parameterModel.get('title') + " " + parameterModel.get('unit');
            graphOptions.chart_type = 'line';
            graphOptions.interpolate = 'linear';
            graphOptions.data = data;
            graphOptions.y_accessor = selectedSensorIDs;
            graphOptions.legend = selectedSensorNames;
        } else {
            // if no data, render this graph with the default options and exit
            // the default options are placeholder options that say 'no data'
            return self.render();
        }

        MG.data_graphic(graphOptions);

        return this;

    }


});

app.view.FooterView = Backbone.View.extend({
    el: "#footer-container",
    
    initialize: function(options) {
        this.template = _.template($('#footer-template').html());
        this.render();
        return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template);
        return this;
    }
});
app.view.NavBarView = Backbone.View.extend({
    el: "#navigation",

    initialize: function() {
      this.template = _.template("<li><a href='<%= link %>'><%= title %></a></li>");
      this.dropdownTemplate = _.template(
        "<li><div class='dropdown'>" +
          "<a class='dropdown-toggle' data-toggle='dropdown' href='<%= link %>'><%= title %> <b class='caret'></b></a>" +
          "<ul class='dropdown-menu' role='menu' aria-labelledby='dLabel'>" +
            "<li><a href='<%= link %>'><%= title %></a>" +
            "<% _.each(children, function(item) { %>" +
              "<li><a href='<%= link %>/<%= item.link.replace('#', '') %>'><%= item.title.toUpperCase() %></a>" +
            "<% }); %>" +
          "</ul></div></li>");
    },

    render: function() {
        var self = this;

        _.each(app.config.views, function(item) {
            if (item.title && item.link) {
              if (item.children) {
                self.$el.append(self.dropdownTemplate({
                  title: item.title.toUpperCase(),
                  link: item.link,
                  children: item.children
                }));
              } else {
                self.$el.append(self.template({
                  title: item.title.toUpperCase(),
                  link: item.link
                }));
              }

            }
        });
    }

});
app.view.PageView = Backbone.View.extend({
    initialize: function(options) {
      this.config = options.config ? options.config : options;
      if (this.config.title) this.id = this.config.title.toLowerCase().replace(/ /g, '-') + "view";
      this.template = _.template($(this.config.template).html());
      return this;
    },
    
    render: function() {
        this.$el.empty();
        this.$el.html(this.template());
        return this;
    }
});
app.view.StackedBarView = Backbone.View.extend({
    tagName: 'div',
    id: 'biology-area',

    initialize: function(options) {
        this.render = _.bind(this.render, this);
        this.scale = options.scale;
        this.graphId = options.id;
        this.data = options.data;
        this.title = options.title;
        this.scaleNames = options.scaleNames;
        this.z = options.z;
        this.unit = options.unit;
    },

    render: function() {
        this.$el.empty();
        var margin = {top: 50, right: 30, bottom: 50, left: 70},
            width = 850 - margin.left - margin.right,
            height = 500 - margin.top - margin.bottom;

        var x = d3.scale.ordinal()
            .rangeRoundBands([0, width-160], .1);

        var y = d3.scale.linear()
            .rangeRound([height, 0]);

        var color = this.z;

        var xAxis = d3.svg.axis()
            .scale(x)
            .orient("bottom")
            .ticks(3)
            .tickFormat(d3.time.format("%Y"));

        var yAxis = d3.svg.axis()
            .scale(y)
            .orient("left");

        var varNames = this.scaleNames;

        var nestFunction = d3.nest().key(function(d) { return d.date;});

        var data = nestFunction.entries(this.data);

        data.forEach(function(d) {
            var y0 = 0;
            var length = d.values.length;

            d.mapping = varNames.map(function(name) {
                var temp = 0;
                d.values.forEach(function(k) {
                    temp +=k[name];
                });
                return {
                    name: name,
                    label: new Date(d.key),
                    y0: y0,
                    y1: y0 += temp/length
                };
            });
            d.date = new Date(d.key);
            d.total = d.mapping[d.mapping.length-1].y1;
        });

        x.domain(data.map(function(d) {return d.date;}));
        y.domain([0, d3.max(data, function(d) {return d.total; })]);

        var svgBiology = d3.select("#"+this.graphId)
            .attr("width", width + margin.left + margin.right)
            .attr("height", height + margin.top + margin.bottom)
            .attr("class", "graphs-biology")
            .append("g")
            .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

        var selection = svgBiology.selectAll(".series")
            .data(data)
            .enter().append("g")
            .attr("class", "series, graphs-listener-biology")
            .attr("x", function(d) {return  x(d.date);})
            .attr("transform", function(d) {
                return "translate(" + x(d.date) + ",0)";
            })
            .style("stroke", "grey")
            .style("stroke-width", 0.2);

        selection.selectAll("rect")
            .data(function(d) {return d.mapping;})
            .enter().append("rect")
            .attr("width", x.rangeBand())
            .attr("y", function(d) {return y(d.y1);})
            .attr("x", function(d) {return  x(d.date);})
            .attr("height", function(d) {return y(d.y0) - y(d.y1);})
            .style("fill", function(d) {return color(d.name); })
            .style("stroke", "grey")
            .style("stroke-width", 0.2);

        // Adding Legend
        var legend = svgBiology.selectAll(".legend")
            .data(varNames.slice().reverse())
            .enter().append("g")
            .attr("class", "legend")
            .attr("transform", function (d, i) {
                return "translate(25, "  + (10 + i * 20) + ")";
            });

        legend.append("rect")
            .attr("x", width - 175)
            .attr("width", 10)
            .attr("height", 10)
            .style("fill", color);

        legend.append("text")
            .attr("x", width - 163)
            .attr("y", 6)
            .attr("dy", ".35em")
            .text(function (d) { return d; });

        svgBiology.append("g")
            .attr("class", "graphs-x-axis")
            .attr("transform", "translate(0," + height + ")")
            .call(xAxis)
            .append("text")
            .style("text-anchor", "end");

        svgBiology.append("g")
            .attr("class", "graphs-y-axis")
            .call(yAxis)
            .append("text")
            .attr("transform", "rotate(-90)")
            .attr("y", 6)
            .style("text-anchor", "end");

        // Adding Title
        svgBiology.append("text")
            .attr("x", 0)
            .attr("y", 0 - (margin.top / 2))
            .attr("text-anchor", "left")
            .style("font-size", "16px")
            .attr("class", "graphs-title")
            .text(this.title);

        //Adding labels
        var unit = this.unit;
        svgBiology.append("text")      // text label for the x axis
            .attr("x", width/2 - margin.left - 10)
            .attr("y",  height + 40)
            .attr("class", "graphs-xAxisLabel")
            .style("text-anchor", "middle")
            .text("Date");

        svgBiology.append("text")
            .attr("transform", "rotate(-90)")
            .attr("x", -height/2)
            .attr("y", -55)
            .attr("class", "graphs-yAxisLabel")
            .style("text-anchor", "middle")
            .text(unit);

        //Adding on Mouseover handle
        var popupBiologyDiv = d3.select('graphs-tooltip').data([0]);
        popupBiologyDiv.enter()
            .append("div")
            .attr("class", "graphs-tooltip")
            .style("opacity", "0");
        popupBiologyDiv.exit().remove();

        // TODO: Update when changed to seasons to be year - spring/summer
        var dateFormat = d3.time.format("%b-%y");

        svgBiology.selectAll(".graphs-listener-biology")
            .on("mouseover", function(d){

                var xPos = parseFloat(d3.select(this).attr("x"));
                var yPos = parseFloat(d3.select(this.children[this.children.length-1]).attr("y"));
                var valueToPrint = "<b>Date</b>: " + dateFormat(d.date) + "<br/>";
                for(x in d.mapping) {
                    var temp = d.mapping[x];
                    var curValue = Math.round(temp.y1 - temp.y0);
                    valueToPrint += "<b>" + temp.name + "</b>: " + curValue  + " (" + unit + ") Avg. <br/>";
                    d3.select(this.children[x]).style("stroke", "black").style("stroke-width", 2)
                }

                var topMargin = Math.min(yPos, 280);
                popupBiologyDiv.html(valueToPrint)
                    .style("left",this.parentElement.getBoundingClientRect().left + margin.left + xPos + "px")
                    .style("top", this.parentElement.getBoundingClientRect().top + margin.top + topMargin + "px");
                popupBiologyDiv.transition().style("opacity", 0.9);

            })
            .on("mouseout",function(d){
                for(x in d.mapping) {
                    d3.select(this.children[x]).style("stroke", "grey").style("stroke-width", 0.2);
                }
                popupBiologyDiv.transition().style("opacity", 0);

            });

    }
});
app.view.WelcomeView = Backbone.View.extend({
    id: "welcome-view",
    tag: "div",

    template: _.template($('#welcome-template').html()),

    initialize: function(options) {
        return this;
    },

    render: function() {
        this.$el.empty();
        this.$el.html(this.template({sources: app.collection.sources.getSourcesWithSensors().toJSON()}));
        return this;
    }
});
(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
})(window,document,'script','//www.google-analytics.com/analytics.js','ga');

ga('create', app.config.googleAnalytics, 'auto');

})(jQuery, Backbone, window);